                                       ############################################################
                                      ####                                                      ####
                                      ####  QUASI-EXPERIMENTS - MANIPULATION OF ENVIRONMENTAL   ####
                                      ####  CONDITIONS AND EGG PREDATION:                       ####
                                      ####                                                      ####
                                      ####  Lesser black-backed gull (Larus fuscus) on Horsvr, ####
                                      ####  Northern Norway                                     ####
                                      ####                                                      ####
                                      ####     Brd-Jrgen Brdsen (bjb@nina.no) & Jan Ove      ####
                                      ####     Bustnes, 22.02.2021                              ####
                                      ####                                                      ####
                                       ############################################################




# NOTES: For this script to function (i.e., for the required input files to load and to store the output in the correct directory), you must ensure that:
#       1) you keep the same structure (directories) as in the original .zip-file;
#       2) you define the current working directory of this R-session to be the same directory as the location of the script file (run 'getwd()' the check this, and use 'setwd(path to
#          this directory on your computer)' to fix it.

# NOTE: Environmental conditions are manipulated through the Environmental Goodness Index (EGI)
#  and the Environmental Harshness Index (EHI) and this manipulation occurred separately from egg
#  predation.

# In Experiment I, we kept environmental conditions constant as follows (see main text for details).
# 1. Control: In scenario, we manipulated egg predation rates from zero (predation was absent)
#    to 0.75 (i.e. 75% of all eggs were removed by predators) while keeping environmental conditions at
#    status quo (EGI=EHI=0).
# 2. Good: Egg predation rates was manipulated as in the Control-scenario, but good years had an
#    increased probability of being selected as EGI was constant at 0.5/15 (ca. 0.030).
# 3. Poor: The same as the Good-scenario except that poor years had an increased probability of
#    being selected (EHI=0.5/15).

# In Experiment II we added two-levels of predation based on our empirical data:
# 1. Control-EGI: Egg predation rates, as estimated and calculated from the empirical data, was present.
#    Consequently, we used the estimated clutch size at hatching as the basis for calculating fecundity.
#    Then, we assessed the effect of improved environmental conditions by varying EGI from zero (control)
#    to 1/15. In the extreme run, the probability of selecting the year with the lowest adult survival
#    (i.e., the poorest year) became zero, whereas the probability of selecting the best year was 2/15
#    (twice the probability compared to status quo).
# 2. No predation-EGI: Same as the first scenario except that no egg predation occurred. Hence, we used
#    laying clutch size as the basis for calculating fecundity.
# 3. Control-EHI: Egg predation were present as in the first scenario, but EHI was manipulated so we
#    increased the probability of selecting 'poor' years.
# 4. No predation-EHI: Same as the third scenario except that no egg predation occurred.

# In both experiments, a directory is created (default: 'Experiment I' and 'Experiment II') and within these
#  other sub-directories are created (one for each simulation or quasi-experimental manipulation). The end-
#  result (output) in each simulation is a directory called 'DATABASE' where some output is analysed and
#  stored as .tiff-files (see 'MatrixSimulationPlottingResults.R' for details). The raw output is stored as
#  'Simudata' and 'SimudataPop' sheets in the 'SimuData.xlsx' (also as separate data frames in
#  'SimuData.RData'). The metadata for 'SimuData' (i.e., data for each age class) is as follows:
#   'MaxMin'           - when 'Pred' is 'FALSE' this is either the probability if selecting poor (EHI; when
#                          'manip' equals 'decrease') or the the probability if selecting good (EGI; when
#                          'manip' equals 'increase'). When 'Pred' is 'TRUE' this is the proportion of eggs
#                          lost to predators from laying (start) to hatching (end).
#   'Class'            - class (age)
#   'nObs._'           - number of individuals
#   'Surv._'           - average survival
#   'Repr._'           - average reproduction (clutch size or chicks per nest/pair/female)
#   'nSim'             - number of simulations
#   'nYear'            - number of years in which data was collected ('Data.collection' below)

# The metadata for 'SimuDataPop' (i.e. population-level data) is as follows:
#   'MaxMin'           - same as for 'SimuData'
#   'N._'              - abundance based on the simulated populations over time
#   'N2._'             - abundance for the adults only based on the simulated populations over time
#   'Lambda.Tuljapurk' - extracts stochastic (log-) lambda using sing Tuljapukar's second order approximation
#                          from projection matrices for each year and simulation
#   'Lambda.sim'       - simulated stochastic (log-) lambda using based on simulation (the average over successive
#                         growth rates (both is based on the 'stoch.growth.rate' function in the popbio-package
#   'Lambda.simLCL'    - lower 95% Confidence Intervals for the simulated stochastic lambda
#   'Lambda.simUCL'    - upper 95% Confidence Intervals for the simulated stochastic lambda
#   'Lambda.sim_maxt'  - number of time intervals used as input to the 'stoch.growth.rate' function
#   'Lambda._'         - lambda based on the simulated populations over time
#   'Lambda2._'        - lambda based on N2 from tge simulated populations over time
#   'nSim'             - same as for 'SimuData'
#   'nYear'            - same as for 'SimuData'
#   'genTimeA.mean'    - generation time for a matrix consisting of the average value for the vital rates (based
#                          on the 'generation.time' function in the popbio-package)

# In both data sets the following means:
#    '_.median'        - is the across-year and -simulation median value for the variables above, e.g.
#                          'Surv.median' is the median value for survival in 'SimuData'.
#    '_.quant2'        - is the 25th percentiles (based on the same as '_.median' above)
#    '_.quant4'        - is the 75th percentiles (based on the same as '_.median' above)






                                        #####################################
                                        # INSTALLING THE NESSESARY PACKAGES #
                                        #####################################


install.packages( c("popbio", "gplots", "mgcv", "openxlsx" ),                 # getting the required packaged (from the CRAN sever in Bergen, Norway)
    repos = "http://cran.ii.uib.no/" )                                        #   another server can be selected by changing repos; or manually by running: 'install.packages(c("popbio", "gplots", "mgcv", "openxlsx"))'
library(popbio)                                                               # loads the package 'popbio: Construction and Analysis of Matrix Population Models'
library(gplots)                                                               # loads the package 'gplots: Various R Programming Tools for Plotting Data'
library(mgcv)                                                                 # loads the package 'mgcv: Mixed GAM Computation Vehicle with Automatic Smoothness Estimation'
library(openxlsx)                                                             # loads the package 'openxlsx: Read, Write and Edit xlsx Files'






                                        ###################################
                                        # INPUT COMMON TO ALL SIMULATIONS #
                                        ###################################


   ### USER DEFINED PARAMETERS (INPUT) COMMOM FOR ALL SIMLATIONS ####

MaxManip <- 1/15                                                              # the maximum manipulation for environmental conditions: EGI/EGI is manipulated from zero to 'MaxManip' equally spaced over 'nGroups'
nGroup <- 80                                                                  # the number of environmental regimes to simulate
nSim <- 200                                                                   # the number of population trajectories to use in the loop
Data.collection <- .50                                                        # the first 'Data.collection' (proportions) projection matrices will be excluded (if this is set to 0.5 the first 50% of the time series will be excluded)
TimeLoop <- 200                                                               # the number of years to run the loop
n0 <- 300
maxN <- 1000                                                                  # maximum number of birds in the oldest age class
simPrint <- round(seq(1, nGroup, length.out = 5))                             # the number in the loop to print output - i.e., which simulations to print detailed output from
Maxruns <- 50000                                                              # the number of times to simulate (see 'A.extin' below): (i) the cumulative distribution (for the quasi-
                                                                              #   extinction probability); and (ii) the number of iterations (i.e. populations) stochastic growth used
maxt <-  50000                                                                # the number of time intervals to simulate the log stochastic growth rate (see 'A.stoch' below)
ExtThres <- 0.7*n0                                                            # the quasi-extinction threshold (used when assessing the quasi-extinction probabilities)
AgeClData <- data.frame( Class = c( 1:4), Age = c( 0:3 ))                     # setting up a data frame containing information about age and the corresponding age classes


   ### LOADING FUNCTIONS AND DEFINNG WHAT OUTPUT TO BE STORED ###

plotLabel <- TRUE                                                             # if this is TRUE, GAM summary stat is added to the plot, if FALSE simpler labels of plots in Norwegian are used (see 'MatrixSimulationPlottingResults.R' for details)
limLam <- c(-0.118,0.10)                                                      # limit for the lambda-figure in MatrixSimulationPlottingResults.R (to make comparative plots across the simulations)
k <- 8                                                                        # the dimension of the basis used to represent the smooth term in the GAM (used when plotting/analysing the output). this limits the degree of complexity
                                                                              #   as this is the maximal possible df allowed to each model term (Wood 2006:224; see 'MatrixSimulationPlottingResults.R' for details).
printRawData <- TRUE                                                          # if this is TRUE, the raw data for a selection of simulation is stored in separate directories (see 'MatrixSimulationAnalyzing.R' for details).
wd <- getwd()                                                                 # defines the current working directory (this is needed in the loop below)
source("MatrixModel.R");source("Logit.R")                                     # defines the function to run the simulation
samp.size <- function(x) { sum(ifelse(is.na(x), 0, 1)) }                      # function for extracting sample size (removing NAs)
quant2 <- function(x)                                                         # function for extracting the second quartile (25% quartile)
  { ifelse( sum(is.na(x)) == length(x), NA,                                   #    returns NA if only NAs exist
       quantile(x, na.rm  = T)[2] )
  }
quant4 <- function(x)                                                         # function for extracting the fourth quartile (75% quartile)
  { ifelse( sum(is.na(x)) == length(x), NA,                                   #    returns NA if only NAs exist
       quantile(x, na.rm  = T)[4] )
  }






                                        ###############################################
                                        # INPUT DATA AND INITIALIZING THE SIMULATIONS #
                                        ###############################################


   ### EXTRACTING INFORMATION ON VITAL RATES, STABLE AGE CLASS DSITRIBUTION, GENERATION TIME AND EGG PREDATION RATES ###

load("VitalRates/VitalRates.RData")                                           # loads parameters used for reproduction and survival (based on empirical analyses: 'SurvivalAnalyses.R'
                                                                              #   and 'DataManipulation.R' the folder 'SILDEMAKE/2015')
AgeClData$N <- round(n0*StableAge,0)                                          # the number of individuals in each age class is defined according to the stable age class distribution based on the
AgeClData                                                                     #   average CONTROL matrix (see how StableAge is defiend in 'VitalRates.R')
(ExtTime <- round(genTime,0)*3)                                               # time frame in which extinction will be assessed: i.e., three generations based on the average matrix (see 'A.stock' below for
                                                                              #   where it is applied and 'VitalRates.R' for how generation time is calculated)

   ### MANIPULATES THE ENVIRONMENTS AND VITAL RATES ###

EnviData <- seq(0,MaxManip, length.out = nGroup)                              # gradient in environmental manipulations (see 'YearProb' in 'VitalRate.R' for details)
VitalAdult$recr <- VitalAdult$ClutchStart.mean*VitalAdult$EggPredProp         # 'recr' is reproduction in all simulations: here it is laying clutch size multiplied by the proportion of eggs lost to predation
# NOTE: 'recr' is common for the following in Experiment I: Laying (Start) clutch size multiplied by the proportion of eggs removed by predators. In Experiment II, however, 'recr' may represent
#       either: 1) Hatching (End) clutch size (i.e., the 'CONTROL (PREDATORS PRESENT)' scenario); or 2) Laying (Start) clutch size (i.e., the 'MANIPULATION (NO PREDATION)' scenario; see
#       'ExtinctionSensitivity.R' and 'VitalRate.R' for details). These differences will be clear in the two experiments below, but will be initiated as the mean clutch size


   ### INITIAL DATA ###

initYeaAd <- VitalAdult[1,]                                                   # data for the adults in the first (initiation) year: selects the first year as the starting year in all simulations
initYeaOf <- subset(VitalOffsp, year == as.character(initYeaAd$year[1]))      # data for the offspring in the first (initiation) year: the only difference from 'initYeaAd' is survival

data <- data.frame(
   Time             =    rep(0, dim(AgeClData)[1]),                           # time (year)
   Class            =    AgeClData$Class,                                     #   labels the age classes to be used in model
   nObs             =    AgeClData$N)                                         #   number of individuals per age class (defined by 'n0' and 'StableAge'; see 'AgeClData' above).
data$Survival <- c(                                                           # initial survival estimates if no estimate is available, survival is assumed zero (e.g. years when no reproduction
   ifelse( length(initYeaOf$survMean) == 0, 0, initYeaOf$survMean ),          #   was observed).
   rep(initYeaAd$survMean,3) )
data$Reproduction <-c(                                                        # initial reproduction estimates - i.e., the number of eggs laid, or chicks hatched per female/nest/pair. offspring do not
   0, 0, 0,                                                                   #   produce offspring, and neither does the 1st and 2nd age classes. if no estimate is available, reproduction is
   ifelse( length(initYeaAd$recr) == 0, 0, initYeaAd$recr ))                  #   assumed zero (e.g., years when no reproduction was observed; see comments under 'recr' above).
data$P <- data$Survival                                                       # the survival coefficients ('P') is the probability that an individual in age class 'i' survives
                                                                              #   from one year (t) to the next (t+1).
data$F <- data$Reproduction*0.5                                               # assumes a 50:50 sex ratio - hence Reproduction is multiplied by 0.5
data$F <- data$F*data$Survival[1]                                             # the fertility coefficients ('F') is calculated by removing the number of eggs subject to predation
                                                                              #   (after reducing predation proportionally as given in 'RedEggPred'), then this value is
                                                                              #   multiplied with survival from hatching to one year of age.

dataPop <- data.frame(
   Time             =    0,                                                   # time (year)
   Lambda           =    NA,                                                  #   population growth rate (i.e., 'Lambda = Nt+1/Nt')
   Lambda2          =    NA )                                                 #   population growth rate considering age-class 2 only (i.e., 'Lambda = N2t+1/N2t'; see 'N2' below)
dataPop$N <- sum(data$nObs)                                                   # population abundance
dataPop$N2 <- data$nObs[data$Class == 4]                                      # abundance of individuals in the adult age class
dataPop$year <- initYeaAd$year                                                # the year-specific empirically-based estimates selected; used for testing purposes only)
dataPop$SurvRank <-initYeaAd$SurvRank                                         # the year-specific empirically-based rank (for survival; used for testing purposes only)

idata <- data;idataPop <- dataPop                                             # defines initial populations (to be kept similar if several simulations are to be ran)






                                        ################
                                        # EXPERIMENT I #
                                        ################


# NOTE: In each run, egg predation was varying from zero to 'MaxMin', whereas environmental conditions was constant.


   ### INPUT: SPECIAL TO THIS EXPERIMENT ###

Pred <- TRUE                                                                  # defines whether manipulation is predation (TRUE) or environment (FALSE) - if TRUE MaxMin in the output file means the proportion of eggs lost to predation else MaxMin in the output file means the value for EHI/EGI
Manip <- data.frame(
  EnviIndex = c(0, rep(0.5/15,2)),
  Manip = c("increase", "increase", "decrease"),                              # defines what to manipulate: EGI is manipulated if 'increase' is used and EHI is manipulated if 'decrease' is defined (i.e. this is used as input to the YearProb-fucntion (see 'VitalRates.R' for details).
  Label = c("Control",  "Good",     "Poor"))                                  # labels the output directory and the resulting figures
MaxManip <- .75                                                               # the maximum variable being manipulated (i.e., the maximum value used for the proportion of eggs removed by predators)
PredData <- seq(0,MaxManip, length.out = nGroup)                              # gradient in environmental manipulations (see 'YearProb' in 'VitalRate.R' for details)


   ### THE OUTPUT DIRECTORY ###

setwd(wd)
dir.create("Experiment I", showWarnings = F)                                  # creates a new directory within the current working directory
setwd("Experiment I")                                                         # defines this new directory as the current working directory
WD <- getwd()                                                                 # defines the directory in which these simulations are stored


   #### INITIAL DATA:

VitalAdult$recr <- VitalAdult$ClutchStart.mean                                # uses the 'MANIPULATION (NO PREDATION)' scenario laying (start) clutch size as the baseline
idata$Reproduction[4] <- initYeaAd$ClutchStart.mean                           # updates initial data with the correct metric for reproduction
idata$F[4] <- idata$Reproduction[4]*0.5*idata$Survival[1]                     # assumes a 50:50 sex ratio and fecundity also takes survival from t to t+1 into account


   ### RUNNING THE SIMULATIONS ###

for(m in 1:dim(Manip)[2])                                                     # m-loop: START (loops over manipulations)
  {
    manip <- as.character(Manip$Manip[m])                                     # defines the environmental condition to use)
    EnviIndex <- Manip$EnviIndex[m]
    Label <- as.character(Manip$Label[m])

    dir.create(as.character(Label), showWarnings = F)                         # creates a new directory within the current working directory
    setwd(as.character(Label))                                                # defines this new directory as the current working directory
    dir.create("DATABASE", showWarnings = F)                                  # creates a new directory within the current working directory
    Wd <- getwd()                                                             # the working directory for the output folder

    source(paste(wd, "/MatrixSimulation.R",sep=""))                           # running the simulations

    setwd(WD)
    graphics.off()

  }






                                        #################
                                        # EXPERIMENT II #
                                        #################

# NOTE: In each run, environmental conditions was varying from zero to 'MaxMin' , whereas egg predation was constant. Environmental conditions were either improved
#       by manipulating the Environmental Goodness Index (EGI) or worsened by manipulating the Environmental Harshness Index (EHI) by setting 'Manip' to 'increase' or 'decrease',
#       respectively.


   ### INPUT: SPECIAL TO THIS EXPERIMENT ###

Pred <- FALSE                                                                 # defines whether manipulation is predation (TRUE) or environment (FALSE) - if TRUE 'MaxMin' in the output file means the proportion of eggs lost to predation else 'MaxMin' in the output file means the value for EHI/EGI
pred <- c("Control", "No predation")                                          # defines whether we use: 1) Hatching (End) clutch size (i.e., the 'CONTROL (PREDATORS PRESENT)' scenario); or 2) Laying (Start) clutch size (i.e. the 'MANIPULATION (NO PREDATION)' scenario)
Manip <- c("increase", "decrease")                                            # defines what to manipulate: EGI is manipulated if 'increase'; EHI is manipulated if 'decrease' - this is
                                                                              #   input used in 'YearProb'-fucntion (see 'VitalRates.R' for details).
MaxManip <- 1/15                                                              # the maximum manipulattion for environmental conditions: EGI/EGI is manipulated from zero to 'MaxManip'
                                                                              #   equally spaced over 'nGroups'


   ### THE OUTPUT DIRECTORY ###

setwd(wd)
dir.create("Experiment II", showWarnings = F)                                 # creates a new directory within the current working directory
setwd("Experiment II")                                                        # defines this new directory as the current working directory
WD <- getwd()                                                                 # defines the directory in which these simulations are stored


   ### RUNNING THE SIMULATIONS ###

for(m in Manip)                                                               # m-loop: START (loops over environmental manipulations)
  {
    manip <- m
    tmp.lab <- ifelse(m=="increase", "EGI", "EHI")                            # if environmental conditions are improved ('incerase') 'EGI' is manipulated else if environmental conditions are worsened 'EHI' is manipulated

    for (e in pred)                                                           # e-loop: START (loops over predation manipulation)
      { Label <- paste(e, "-", tmp.lab, sep="")                               # labels the simulation to run
        dir.create(as.character(Label), showWarnings = F)                     # creates a new directory within the current working directory
        setwd(as.character(Label))                                            # defines this new directory as the current working directory
        dir.create("DATABASE", showWarnings = F)                              # creates a new directory within the current working directory
        Wd <- getwd()                                                         # the working directory for the output folder

        { if (e == "Control")                                                 # e-conditions: START (defining if fecundity should include predation or not)
            { VitalAdult$recr <- VitalAdult$ClutchStart.mean*                 # in the 'CONTROL (PREDATION PRESENT)' scenario hatching (end) clutch size is used
                VitalAdult$EggPredProp
              idata$Reproduction[4] <- initYeaAd$ClutchStart.mean*            # updates initial data with the correct metric for reproduction
                initYeaAd$EggPredProp
            }
          else if (e == "No predation")
            { VitalAdult$recr <- VitalAdult$ClutchStart.mean                  # in the 'MANIPULATION (NO PREDATION)' scenario laying (start) clutch size is used
              idata$Reproduction[4] <- initYeaAd$ClutchStart.mean             # updates initial data with the correct metric for reproduction
            }
        }                                                                     # e-conditions: END

        idata$F[4] <- idata$Reproduction[4]*0.5*idata$Survival[1]             # assumes a 50:50 sex ratio and fecundity also takes survival from t to t+1 into account

        source(paste(wd, "/MatrixSimulation.R",sep=""))                       # running the simulations

        setwd(WD)
        graphics.off()
      }                                                                       # e-loop: END
  }




