                                       ############################################################
                                      ####                                                      ####
                                      ####  ASSESSING QUASI-EXTINCTION AND LONG-TERM            ####
                                      ####  ASSYPTOTIC DYNAMICS:                                ####
                                      ####                                                      ####
                                      ####  Lesser black-backed gull (Larus fuscus) on Horsvr, ####
                                      ####  Northern Norway                                     ####
                                      ####                                                      ####
                                      ####     Brd-Jrgen Brdsen (bjb@nina.no) & Jan Ove      ####
                                      ####     Bustnes, 22.02.2022                              ####
                                      ####                                                      ####
                                       ############################################################


# NOTES: this script is used to assess quasi-extinction (by default, this is the probability of a 30%
# reduction in population size from 300 individuals over 36 years) and long-term viability as assessed using
# both stochastic and deterministic approaches.









                                        #####################################
                                        # INSTALLING THE NECESSARY PACKAGES #
                                        #####################################


install.packages( c("popbio", "gplots", "openxlsx"),              # getting the required packaged (from the CRAN sever in Bergen, Norway)
    repos = "http://cran.ii.uib.no/" )                            #   another server can be selected by changing repos; or manually by running: 'install.packages( c("popbio", "gplots", "openxlsx"))'
library(popbio)                                                   # loads the package 'popbio: Construction and Analysis of Matrix Population Models'
library(gplots)                                                   # loads the package 'gplots: Various R Programming Tools for Plotting Data'
library(openxlsx)                                                 # loads the package 'openxlsx: Read, Write and Edit xlsx Files'
source("PartialR2.R")                                             # function for calculating partial R-square values









                                        ###################################
                                        # INPUT COMMON TO ALL SIMULATIONS #
                                        ###################################


   ### USER-DEFINED PARAMETERS (INPUT) COMMON FOR ALL SIMULATIONS ####

n0 <- 300
Maxruns <- 50000                                                  # the number of times to simulate (see 'A.extin' below): (i) the cumulative distribution (for the quasi-
                                                                  #   extinction probability); and (ii) the number of iterations (i.e. populations) stochastic growth used
maxt <-  50000                                                    # the number of time intervals to simulate the log stochastic growth rate (see 'A.stoch' below)
ExtThres <- 210                                                   # the quasi-extinction threshold (used when assessing the quasi-extinction probabilities)
ExtTime <- 100                                                    # preliminary time frame in which extinction will be assessed (see 'A.stock' below): this will be updated to cover three generations below
AgeClData <- data.frame( Class = c( 1:4), Age = c( 0:3 ))         # setting up a data frame containing information about age and the corresponding age classes
AgeClData$N <- rep(round(n0/dim(AgeClData)[1],0),                 # preliminary number of ind. in each age-class (equal numbers per age class) - this will be changed in the calculations/simulations below
  dim(AgeClData)[1])









                                        ####################################
                                        # LOADING INPUT DATA AND FUNCTIONS #
                                        ####################################


   ### DATA ON VITAL RATES FROM THE LITERATURE: ADULT APPARENT SURVIVAL, REPRODUCTION AND NEST PREDATION RATES ###

wd <- getwd()                                                     # get current working directory
Tmp <- unlist(strsplit(wd,"/"))                                   # splits the current working directory into its sub-directories
Tmp <- Tmp[-length(Tmp)]                                          # removes that last one
Tmp <- paste(Tmp,collapse="/")                                    # puts the path together again (i.e., the same as moving to the parent directory from the current working directory)
dataLit <- read.xlsx(                                             # imports the data from the 'STATISTICAL_ANALYSES\Data\Literature' directory
  xlsxFile = paste(Tmp,
  "/STATISTICAL_ANALYSES/Data/Literature/LiteratureData.xlsx",
  sep=""), sheet = 1, colNames = T)
dataLit$Source2 <- ifelse(dataLit$Source == "This study",
  "Horsvr", "Other studies")
dataLit$Source2 <- as.factor(dataLit$Source2)                     # creates a factor variable separating our study ('Horsvr'; baseline) from the other ('Other studies')
rm(Tmp)
LittEst <-
  data.frame(
   SurvAdu = mean(dataLit$Estimate[                               # mean adult surival from the literature
     dataLit$Source2 == "Other studies" &
     dataLit$Type == "Adult survival"]),
   ClutchS = mean(dataLit$Estimate[                               # mean clutch size from the literature
     dataLit$Source2 == "Other studies" &
     dataLit$Type == "Clutch size"]),
   Flegdlings = mean(dataLit$Estimate[                            # mean fledglings per nest from the literature
     dataLit$Source2 == "Other studies" &
     dataLit$Type == "Fledglings"]),
   EggPred = mean(dataLit$Estimate[                               # mean egg predation from the literature
     dataLit$Source2 == "Other studies" &
     dataLit$Type == "Egg predation"]))


   ### EXTRACTING INFORMATION ON VITAL RATES AND SETTING UP FOLDERS IN WHICH TO STORE THE RESULTS ###

load("VitalRates/VitalRates.RData")                               # loads data on the vital rates (based on output from running the script: 'VitalRates.R' in the same directory as this script)
dir.create("ExtinctionSensitivity", showWarnings = F)             # creates a new directory within the current working directory
setwd("ExtinctionSensitivity")                                    # defines this new directory as the current working directory









                                        ####################################################################
                                        # EXTRACTING LAMBDAS AND MODELLING GROWTH USING THE AVERAGE MATRIX #
                                        ####################################################################



   ### ALL YEARS: CONTROL (NEST PREDATION AT ITS NATURAL LEVEL) ###

Tmp1 <- levels(VitalOffsp$year)                                   # all years
Tmp <- subset(VitalOffsp, year%in%Tmp1)                           # subset the offspring data (vital rates)
Tmp1 <- subset(VitalAdult, year%in%Tmp1)                          # subset the adult data (vital rates)

tmp <- data.frame(
    F = mean(Tmp$ClutchStart.mean) * 0.5 *                        # average across all years: average laying clutch size * average proportion of eggs removed by predators *
        mean(Tmp$EggPredProp,na.rm=T) *                           #   average juvenile survival * 0.5 (assuming 1:1 sex ratio)
        mean(Tmp$survMean),
    P0   = mean(Tmp$survMean, na.rm = T),
    P13  = mean(Tmp1$survMean, na.rm = T) )
A.mean <- matrix(rep(0,4), 4, nrow = 4,  byrow = TRUE)
A.mean[1,4] <- tmp$F
A.mean[2,1] <- tmp$P0
A.mean[3,2] <- tmp$P13
A.mean[4,3] <- tmp$P13
A.mean[4,4] <- tmp$P13
p <- pop.projection(A = A.mean, n = AgeClData$N, iterations =     # projecting population into the future for 'ExtTime' years using the matrix 'a' (depends on the popbio-package)
  ExtTime)
round(p$stable.stage,2)                                           # the stable age distribution used to set-up the number of ind. in each age-class for the remaining calculations/simulations
(AgeClData$N <- round(n0*p$stable.stage,0))                       # starts with the stable age distribution based on the average matrix )
(genTime <- generation.time(A.mean))                              # extracts extinction time for the population
(ExtTime <- round(genTime,0)*3)                                   # time frame in which extinction will be assessed: i.e., three gen. based on the average matrix (see 'A.stock' below
p <- pop.projection(A = A.mean, n = AgeClData$N, iterations =
  ExtTime)
lambda <- p$lambda                                                # extracts the lambda
Nstart <- p$pop.sizes[1]
Nend <- p$pop.sizes[ExtTime]
eig <- eigen.analysis(A.mean)                                     # calculate population growth rate and other demographic parameters from a projection matrix model using matrix algebra
Elasticity <- eig$elasticities                                    # extracts elasticities
Sensitivity <- eig$sensitivities                                  # extracts sensitivities


   ### ALL YEARS: MANIPULATION (NO NEST PREDATION) ###

tmp <- data.frame(                                                # average across all years: average laying clutch size * average juvenile survival * 0.5 (assuming 1:1 sex ratio)
    F = mean(Tmp$ClutchStart.mean)*0.5*
        mean(Tmp$survMean),
    P0   = mean(Tmp$survMean, na.rm = T),
    P13  = mean(Tmp1$survMean, na.rm = T) )
A.meanP <- A.mean
A.meanP[1,4] <- tmp$F

pP <- pop.projection(A = A.meanP, n = AgeClData$N,
  iterations = ExtTime)
genTimeP <- generation.time(A.meanP)
lambdaP <- pP$lambda
NstartP <- pP$pop.sizes[1]
NendP <- pP$pop.sizes[ExtTime]
eig <- eigen.analysis(A.meanP)

ElasticityP <- eig$elasticities
SensitivityP <- eig$sensitivities


   ### ALL YEARS: BOOKKEEPING & PLOTTING ###

dataExtTime <- data.frame(F4.contr = A.mean[1,4],
  F4.NoPred = A.meanP[1,4],                                       # creates a data frame consisting of all the information
  P0 = tmp$P0, P13 = tmp$P13,
  genTime.contr = genTime, genTime.NoPred = genTimeP,
  lambda.contr = lambda, lambda.NoPred = lambdaP, Nstart = Nstart,
  Nend.contr = Nend, Nend.NoPred = NendP,
  Time = ExtTime,
  ChangeAbs.contr = Nend-Nstart, ChangeAbs.NoPred = NendP-Nstart,
  ChangePerc.contr = (Nend-Nstart)/Nstart*100,
  ChangePerc.NoPred = (NendP-Nstart)/Nstart*100,
  RecrRank = NA, SurvRank = NA, year = "average",
  ElaF4.contr = Elasticity[1,4],ElaP0.contr = Elasticity[2,1],
  ElaP13.contr = Elasticity[3,2],
  ElaP4.contr = Elasticity[4,4],
  SenF4.contr = Sensitivity[1,4],SenP0.contr = Sensitivity[2,1],
  SenP13.contr = Sensitivity[3,2],
  SenP4.contr = Sensitivity[4,4],
  ElaF4.NoPred = ElasticityP[1,4],ElaP0.NoPred = ElasticityP[2,1],
  ElaP13.NoPred = ElasticityP[3,2],
  ElaP4.NoPred = ElasticityP[4,4],
  SenF4.NoPred = SensitivityP[1,4],SenP0.NoPred =SensitivityP[2,1],
  SenP13.NoPred = SensitivityP[3,2],
  SenP4.NoPred = SensitivityP[4,4]  )

dev.new(width=24/2.54, height=16/2.54)
par(bty = "l", cex.axis = 1.6,                                    # setting up the plot
  mar = c(4, 4, 2.5, 0.35), oma = c(1, 1.5, 1, 0.5),
  mfrow = c(1,2))
dimnames(Sensitivity)<- list(c("0", "1", "2", "3+"))
Sensitivity <- Sensitivity*ifelse(A.mean == 0, NA, 1)
dimnames(Sensitivity)[[2]] <- dimnames(Sensitivity)[[1]]
image2(ifelse(Sensitivity == 0, NA, Sensitivity),                 # plots the elasticities
  mar=c(0.5,1.5,5,1), box.offset=.15, labels = c(2,3) )
mtext(side = 3, expression("Sensitivity (Control)" ~ lambda~
  "("*bold(A)["2005-"]*")"),
  cex = 1.6, line = 1)
dimnames(SensitivityP)<- list(c("0", "1", "2", "3"))
dimnames(SensitivityP)[[2]] <- dimnames(SensitivityP)[[1]]
image2(ifelse(Sensitivity == 0, NA, SensitivityP),                # plots the elasticities
  mar=c(0.5,1.5,5,1), box.offset=.15, labels = c(2,3) )
mtext(side = 3, expression("Sensitivity (No predation)" ~
  lambda~ "("*bold(A)["2005-"]*")"),
  cex = 1.6, line = 1)
dev.copy(tiff, filename = "SensiAverage.AllYears.tiff",           # saves the plot externally as a .tiff-file
  units="cm",width=24, height=16, res=200, compression = c("lzw"))
dev.off()


   ### EARLY (2005-2010): CONTROL (NEST PREDATION AT ITS NATURAL LEVEL) ###

Tmp1 <- as.character(2005:2010)
Tmp <- subset(VitalOffsp, year%in%Tmp1)
Tmp1 <- subset(VitalAdult, year%in%Tmp1)

tmp <- data.frame(
    F = mean(Tmp$ClutchStart.mean)*0.5*mean(Tmp$EggPredProp,na.rm=T)*
        mean(Tmp$survMean),
    P0   = mean(Tmp$survMean, na.rm = T),
    P13  = mean(Tmp1$survMean, na.rm = T) )
A.2005_2010 <- matrix(rep(0,4), 4, nrow = 4,  byrow = TRUE)
A.2005_2010[1,4] <- tmp$F
A.2005_2010[2,1] <- tmp$P0
A.2005_2010[3,2] <- tmp$P13
A.2005_2010[4,3] <- tmp$P13
A.2005_2010[4,4] <- tmp$P13
p <- pop.projection(A = A.2005_2010, n = AgeClData$N,
  iterations = ExtTime)
genTime <- generation.time(A.2005_2010)
lambda <- p$lambda
Nstart <- p$pop.sizes[1]
Nend <- p$pop.sizes[ExtTime]
eig <- eigen.analysis(A.2005_2010)
Elasticity <- eig$elasticities
Sensitivity <- eig$sensitivities


   ### EARLY (2005-2010): MANIPULATION (NO NEST PREDATION) ###

tmp <- data.frame(
    F = mean(Tmp$ClutchStart.mean)*0.5*mean(Tmp$survMean),
    P0   = mean(Tmp$survMean, na.rm = T),
    P13  = mean(Tmp1$survMean, na.rm = T) )
A.2005_2010P <- A.2005_2010
A.2005_2010P[1,4] <- tmp$F
pP <- pop.projection(A = A.2005_2010P, n = AgeClData$N,
  iterations = ExtTime)
genTimeP <- generation.time(A.2005_2010P)
lambdaP <- pP$lambda
NendP <- pP$pop.sizes[ExtTime]
eig <- eigen.analysis(A.2005_2010P)
ElasticityP <- eig$elasticities
SensitivityP <- eig$sensitivities


   ### EARLY (2005-2011): BOOKKEEPING & PLOTTING ###

tmp <- data.frame(F4.contr = A.2005_2010[1,4],
  F4.NoPred = A.2005_2010P[1,4],
  P0 = tmp$P0, P13 = tmp$P13,
  genTime.contr = genTime, genTime.NoPred = genTimeP,
  lambda.contr = lambda, lambda.NoPred = lambdaP, Nstart = Nstart,
  Nend.contr = Nend, Nend.NoPred = NendP,
  Time = ExtTime,
  ChangeAbs.contr = Nend-Nstart, ChangeAbs.NoPred = NendP-Nstart,
  ChangePerc.contr = (Nend-Nstart)/Nstart*100,
  ChangePerc.NoPred = (NendP-Nstart)/Nstart*100,
  RecrRank = NA, SurvRank = NA, year = "Early (2005-2010)",
  ElaF4.contr = Elasticity[1,4],ElaP0.contr = Elasticity[2,1],
  ElaP13.contr = Elasticity[3,2],
  ElaP4.contr = Elasticity[4,4],
  SenF4.contr = Sensitivity[1,4],SenP0.contr = Sensitivity[2,1],
  SenP13.contr = Sensitivity[3,2],
  SenP4.contr = Sensitivity[4,4],
  ElaF4.NoPred = ElasticityP[1,4],ElaP0.NoPred = ElasticityP[2,1],
  ElaP13.NoPred = ElasticityP[3,2],
  ElaP4.NoPred = ElasticityP[4,4],
  SenF4.NoPred = SensitivityP[1,4],SenP0.NoPred = SensitivityP[2,1],
  SenP13.NoPred = SensitivityP[3,2],
  SenP4.NoPred = SensitivityP[4,4]  )
dataExtTime <- rbind.data.frame(dataExtTime, tmp)

dev.new(width=24/2.54, height=16/2.54);par(bty = "l",
  cex.axis = 1.6, mar = c(4, 4, 2.5, 0.35),
  oma = c(1, 1.5, 1, 0.5), mfrow = c(1,2))
dimnames(Sensitivity)<- list(c("0", "1", "2", "3+"))
Sensitivity <- Sensitivity*ifelse(A.mean == 0, NA, 1)
dimnames(Sensitivity)[[2]] <- dimnames(Sensitivity)[[1]]
image2(ifelse(Sensitivity == 0, NA, Sensitivity),
  mar=c(0.5,1.5,5,1), box.offset=.15, labels = c(2,3) )
mtext(side = 3, expression("Sensitivity (Control)" ~
  lambda~ "("*bold(A)[2005-2010]*")"),
  cex = 1.6, line = 1)
dimnames(SensitivityP)<- list(c("0", "1", "2", "3+"))
dimnames(SensitivityP)[[2]] <- dimnames(SensitivityP)[[1]]
image2(ifelse(Sensitivity == 0, NA, SensitivityP),
  mar=c(0.5,1.5,5,1), box.offset=.15, labels = c(2,3) )
mtext(side = 3, expression("Sensitivity (No predation)" ~
  lambda~ "("*bold(A)[2005-2010]*")"),
  cex = 1.6, line = 1)
dev.copy(tiff, filename = "SensiAverage.Early.tiff", units="cm",
  width=24, height=16, res=200, compression = c("lzw"));dev.off()


   ### LATE (2011-): CONTROL (NEST PREDATION AT ITS NATURAL LEVEL) ###

Tmp <- as.numeric(as.character(Tmp$year))
Tmp1 <- as.numeric(as.character(VitalAdult$year))
Tmp1 <- as.character((max(Tmp)+1):(max(Tmp1)))
Tmp <- subset(VitalOffsp, year%in%Tmp1)
Tmp1 <- subset(VitalAdult, year%in%Tmp1)
tmp <- data.frame(
    F = mean(Tmp$ClutchStart.mean)*mean(Tmp$EggPredProp,na.rm=T)*.5*
        mean(Tmp$survMean),
    P0   = mean(Tmp$survMean, na.rm = T),
    P13  = mean(Tmp1$survMean, na.rm = T) )
A.2011_ <- matrix(rep(0,4), 4, nrow = 4,  byrow = TRUE)
A.2011_[1,4] <- tmp$F
A.2011_[2,1] <- tmp$P0
A.2011_[3,2] <- tmp$P13
A.2011_[4,3] <- tmp$P13
A.2011_[4,4] <- tmp$P13
p <- pop.projection(A = A.2011_, n = AgeClData$N,
  iterations = ExtTime)
genTime <- generation.time(A.2011_)
lambda <- p$lambda
Nend <- p$pop.sizes[ExtTime]
eig <- eigen.analysis(A.2011_)
Elasticity <- eig$elasticities
Sensitivity <- eig$sensitivities


   ### LATE (2011-): MANIPULATION (NO NEST PREDATION) ###

tmp <- data.frame(
    F = mean(Tmp$ClutchStart.mean)*0.5*mean(Tmp$survMean),
    P0   = mean(Tmp$survMean, na.rm = T),
    P13  = mean(Tmp1$survMean, na.rm = T) )
A.2011_P <- A.2011_
A.2011_P[1,4] <- tmp$F
pP <- pop.projection(A = A.2011_P, n = AgeClData$N,
  iterations = ExtTime)
genTimeP <- generation.time(A.2011_P)
lambdaP <- pP$lambda
NendP <- pP$pop.sizes[ExtTime]
eig <- eigen.analysis(A.2011_P)
ElasticityP <- eig$elasticities
SensitivityP <- eig$sensitivities


   ### LATE (2011-): BOOKKEEPING & PLOTTING ###

tmp <- data.frame(F4.contr = A.2011_[1,4],
  F4.NoPred = A.2011_P[1,4],
  P0 = tmp$P0, P13 = tmp$P13,
  genTime.contr = genTime, genTime.NoPred = genTimeP,
  lambda.contr = lambda, lambda.NoPred = lambdaP, Nstart = Nstart,
  Nend.contr = Nend, Nend.NoPred = NendP,
  Time = ExtTime,
  ChangeAbs.contr = Nend-Nstart, ChangeAbs.NoPred = NendP-Nstart,
  ChangePerc.contr = (Nend-Nstart)/Nstart*100,
  ChangePerc.NoPred = (NendP-Nstart)/Nstart*100,
  RecrRank = NA, SurvRank = NA, year = "Late (2011-)",
  ElaF4.contr = Elasticity[1,4],ElaP0.contr = Elasticity[2,1],
  ElaP13.contr = Elasticity[3,2],
  ElaP4.contr = Elasticity[4,4],
  SenF4.contr = Sensitivity[1,4],SenP0.contr = Sensitivity[2,1],
  SenP13.contr = Sensitivity[3,2],
  SenP4.contr = Sensitivity[4,4],
  ElaF4.NoPred = ElasticityP[1,4],ElaP0.NoPred = ElasticityP[2,1],
  ElaP13.NoPred = ElasticityP[3,2],
  ElaP4.NoPred = ElasticityP[4,4],
  SenF4.NoPred = SensitivityP[1,4],SenP0.NoPred = SensitivityP[2,1],
  SenP13.NoPred = SensitivityP[3,2],
  SenP4.NoPred = SensitivityP[4,4]  )
dataExtTime <- rbind.data.frame(dataExtTime, tmp)

dev.new(width=24/2.54, height=16/2.54);par(bty = "l",
  cex.axis = 1.6, mar = c(4, 4, 2.5, 0.35),
  oma = c(1, 1.5, 1, 0.5), mfrow = c(1,2))
dimnames(Sensitivity)<- list(c("0", "1", "2", "3+"))
Sensitivity <- Sensitivity*ifelse(A.mean == 0, NA, 1)
dimnames(Sensitivity)[[2]] <- dimnames(Sensitivity)[[1]]
image2(ifelse(Sensitivity == 0, NA, Sensitivity),
  mar=c(0.5,1.5,5,1), box.offset=.15, labels = c(2,3) )
mtext(side = 3, expression("Sensitivity (Control)" ~
  lambda~ "("*bold(A)["2011-"]*")"),
  cex = 1.6, line = 1)
dimnames(SensitivityP)<- list(c("0", "1", "2", "3+"))
dimnames(SensitivityP)[[2]] <- dimnames(SensitivityP)[[1]]
image2(ifelse(Sensitivity == 0, NA, SensitivityP),
  mar=c(0.5,1.5,5,1), box.offset=.15, labels = c(2,3) )
mtext(side = 3, expression("Sensitivity (No predation)" ~
  lambda~ "("*bold(A)["2011-"]*")"),
  cex = 1.6, line = 1)
dev.copy(tiff, filename = "SensiAverage.Late.tiff", units="cm",
  width=24, height=16, res=200, compression = c("lzw"));dev.off()












                                        ########################################################################
                                        # EXTRACTING LAMBDAS AND MODELLING GROWTH USING YEAR-SPECIFIC MATRICES #
                                        ########################################################################

for (y in levels(VitalData$year) )                                # 1) YEAR-loop: START
  {

   ### EXTRACTING THE VITAL RATES ###

    tmp1 <- subset(VitalData, year == y)
    tmp11 <- subset(tmp1, ageClass == "[2,-)")                    # data on adults only
    tmp12 <- subset(tmp1, ageClass == "[0-1]")                    # data on offspring only
    { if (dim(tmp12)[1]==0)
       { tmp <- data.frame(AgeClData, F = c(rep(0,4)),            # CONTROL (NEST PREDATION AT ITS NATURAL LEVEL): vital rates if there is no data for the sub-adults
           P = c(0, rep(tmp11$survMean,3)))
         tmpP <- data.frame(AgeClData, F = c(rep(0,4)),           # MANIPULATION (NO NEST PREDATION): vital rates if there is no data for the sub-adults
           P = c(0, rep(tmp11$survMean,3)))
       }
      else
       { tmp <- data.frame(AgeClData,                             # CONTROL (PREDATORS PRESENT): vital rates if there is data for the sub-adults
           F = c(rep(0,3),
             tmp11$ClutchStart.mean*
             0.5*tmp12$survMean)*tmp12$EggPredProp,
           P = c(tmp12$survMean, rep(tmp11$survMean,3)))
         tmpP <- data.frame(AgeClData,                            # MANIPULATION (NO NEST PREDATION): vital rates if there is data for the sub-adults
           F = c(rep(0,3),tmp11$ClutchStart.mean*
            0.5*tmp12$survMean),
           P = c(tmp12$survMean, rep(tmp11$survMean,3)))
       }
    }


   ### SETTING UP THE PROJECTION MATRIX FOR THE CONTROL (NEST PREDATION AT ITS NATURAL LEVEL) SCENARIO ###

    a <- matrix(rep(0, (dim(tmp)[1])^2), nrow = dim(tmp)[1],      # constructing the projection matrix for a given year ('a')
      byrow = TRUE, dimnames = list(c(tmp$Class), c(tmp$Class)))
    a[1,] <- c(tmp$F)                                             # setting up the fecundities ('F') in the top row
    for(i in 2:(dim(a)[1]) )                                      # setting up the survival rates ('P' or 'S')
      { a[(i),(i-1)] <- tmp$P[(i-1)]
      }
    a[dim(a)[1],dim(a)[1]] <- tmp$P[length(tmp$P)]
    p <- pop.projection(A = a, n = AgeClData$N,
     iterations = ExtTime)
    genTime <- generation.time(a)
    lambda <- p$lambda
    Nstart <- p$pop.sizes[1]
    Nend <- p$pop.sizes[ExtTime]
    eig <- eigen.analysis(a)
    Elasticity <- eig$elasticities
    Sensitivity <- eig$sensitivities


    ### SETTING UP THE PROJECTION MATRIX FOR THE MANIPULATION (NO NEST PREDATION) SCENARIO ###

    aP <- a                                                       # fecundity is the only difference between the TREATMENTS (CONTROL: PREDATORS PRESENT; MANIPULATION: NO NEST PREDATION) matrices
    aP[1,4] <- tmpP$F[4]
    pP <- pop.projection(A = aP, n = AgeClData$N,
     iterations = ExtTime)
    genTimeP <- generation.time(aP)
    lambdaP <- pP$lambda
    NendP <- pP$pop.sizes[ExtTime]
    eig <- eigen.analysis(aP)
    ElasticityP <- eig$elasticities
    SensitivityP <- eig$sensitivities


    ### BOOKKEEPING ###

    tmp <- data.frame(F4.contr = a[1,4], F4.NoPred = aP[1,4],
      P0 = a[2,1], P13 = a[3,2],
      genTime.contr = genTime, genTime.NoPred = genTimeP,
      lambda.contr = lambda, lambda.NoPred = lambdaP,
      Nstart = Nstart,
      Nend.contr = Nend, Nend.NoPred = NendP,
      Time = ExtTime,
      ChangeAbs.contr = Nend-Nstart, ChangeAbs.NoPred =
      NendP-Nstart, ChangePerc.contr = (Nend-Nstart)/Nstart*100,
      ChangePerc.NoPred = (NendP-Nstart)/Nstart*100,
      RecrRank = tmp11$ClutchStartRank, SurvRank = tmp11$SurvRank,
      year = y,
      ElaF4.contr = Elasticity[1,4],ElaP0.contr = Elasticity[2,1],
      ElaP13.contr = Elasticity[3,2],
      ElaP4.contr = Elasticity[4,4],
      SenF4.contr = Sensitivity[1,4],SenP0.contr = Sensitivity[2,1],
      SenP13.contr = Sensitivity[3,2],
      SenP4.contr = Sensitivity[4,4],
      ElaF4.NoPred = ElasticityP[1,4],ElaP0.NoPred = ElasticityP[2,1],
      ElaP13.NoPred = ElasticityP[3,2],
      ElaP4.NoPred = ElasticityP[4,4],
      SenF4.NoPred = SensitivityP[1,4],SenP0.NoPred = SensitivityP[2,1],
      SenP13.NoPred = SensitivityP[3,2],
      SenP4.NoPred = SensitivityP[4,4]  )
    dataExtTime <- rbind.data.frame(dataExtTime, tmp)

    { if (y == levels(VitalData$year)[1])                         # storing information in the first run of the loop
       { A <- c( list(a) )                                        # CONTROL (NEST PREDATION AT ITS NATURAL LEVEL): a list of matrices consisting of the projection matrices for each year is created ('a' is defined as 'A')
         AP <- c( list(aP))                                       # MANIPULATION (NO NEST PREDATION): a list of matrices consisting of the projection matrices for each year is created ('aP' is defined as 'AP')
       }
      else                                                        # storing information in the remaining runs of the loop
       { A <- c(A, list(a))                                       # CONTROL (NEST PREDATION AT ITS NATURAL LEVEL): adds information about 'a' to 'A'
         AP <- c(AP, list(aP))                                    # MANIPULATION (NO NEST PREDATION): adds information about 'aP' to 'AP'
       }
    }
  }                                                               # 1) YEAR-loop: END
rm(YearProb, p, genTime, lambda, Nstart, Nend, pP, genTimeP,
   lambdaP, NendP, y, i, Sensitivity, Elasticity, SensitivityP,
   ElasticityP, eig, wd, NstartP, a, aP)
rm(list=ls(pattern="tmp"));rm(list=ls(pattern="Tmp"))


   ### STORING THE DATA EXTERNALLY ###

# NOTES: Metadata for the output ('dataExtTime'; also stored as
# 'dataExtTime.xlsx' in the 'ExtinctionSensitivity' directory

Metadata <- data.frame(
  Variables = c(
    "_.contr", "_.NoPred",
    "F4._", "P0._", "P13._", "genTimeA.mean:_", "Lambda._",
    "Nstart", "Nend._", "Time", "ChangeAbs._", "ChangePerc._",
    "RecrRank", "SurvRank", "year", "SenF4._/ElaF4._",
    "SenP0._/ElaP0._", "SenP13._/ElaP13._", "SenP4._/ElaP4._"),
  Interpretation = c(
    "Marks the variables '._' for the 'CONTROL (NEST PREDATION AT ITS NATURAL LEVEL)' scenario: e.g., 'Lambda.Contr' is growth based on these matrices",
    "Similarly, this marks the variables  for the 'MANIPULATION (NO NEST PREDATION)' scenario",
    "Fecundity (oldest age class)", "Offspring apparent survival",
    "Survival for the remaining age-classes",
    "Generation time for a matrix consisting of the vital rates (based on the 'generation.time' function in the popbio-package)",
    "Lambda based on the vital rates",
    "Number of individuals at initiation",
    "Calculated number of individuals after 'Time' number of years",
    "Number of years in which extinctions were assessed (see 'ExtTime' below)",
    "Absolute difference between the number of individuals at the start and end",
    "Percentage difference between the number of individuals at the start and end",
    "Ranked value for the annual estimated laying clutch size",
    "Ranked value for the annual estimated adult apparent survival",
    "Year of interest either as a single year or a period (in which the estimates represent average values)",
    "Sensitivity/Elasticity for fecundity (based on the 'eigen.analysis' function in the popbio-package)",
    "Sensitivity/Elasticity for offspring survival",
    "Sensitivity/Elasticity for the remaining survival rates for the in-between age classes",
    "Sensitivity/Elasticity for the adult age class"))

wb <- createWorkbook()
addWorksheet(  wb, sheetName = "dataExtTime")                     # creates the sheet
freezePane(    wb, sheet = "dataExtTime", firstRow = TRUE,
  firstCol = TRUE)                                                # freeze first row and column
setColWidths( wb,  sheet = "dataExtTime",widths ="auto",
  cols = 1:10)                                                    # sets column width
writeDataTable(wb, sheet = "dataExtTime", x = dataExtTime)        # adds data to the sheet
addWorksheet(  wb, sheetName = "Metadata")
freezePane(    wb, sheet = "Metadata",
  firstRow = TRUE, firstCol = TRUE)
setColWidths( wb,  sheet = "Metadata", widths ="auto",
  cols = 1:10)
writeDataTable(wb, sheet = "Metadata",
  x = Metadata)
saveWorkbook(wb, "dataExtTime.xlsx", overwrite = TRUE);rm(wb)     # saves the file









                                        #########################################################
                                        # STOCHASTIC SENSITIVITIES USING YEAR-SPECIFIC MATRICES #
                                        #########################################################


   ### CONTROL: NEST PREDATION AT ITS NATURAL LEVEL (AND INITIATING THE PLOT) ###

StochasticSens <- stoch.sens(A, tlimit=100)                       # calculates the sensitivity of the stochastic growth rate using the 'stoch.sens' function in the popbio-packages
Elasticity <- StochasticSens$elasticities
Sensitivity <- StochasticSens$sensitivities

dev.new(width=16/2.54, height=18/2.54);par(bty = "l",
  cex.axis = 1.4, mar = c(4, 4, 2.5, 0.35),
  oma = c(.4, 1.5, 1, 0.5), mfrow = c(2,2))
dimnames(Sensitivity)<- list(c("0", "1", "2", "3+"))
Sensitivity <- Sensitivity*ifelse(A.mean == 0, NA, 1)
dimnames(Sensitivity)[[2]] <- dimnames(Sensitivity)[[1]]
image2(ifelse(Sensitivity == 0, NA, Sensitivity),
  mar=c(0.5,1.5,5,1), box.offset=.15, labels = c(2,3) )
mtext(side = 3, expression("a)"~ italic("Control")*":"~lambda[
  "senstivities"]), cex = 1.3, line = 1)
dimnames(Elasticity)<- list(c("0", "1", "2", "3+"))
Elasticity <- Elasticity*ifelse(A.mean == 0, NA, 1)
dimnames(Elasticity)[[2]] <- dimnames(Elasticity)[[1]]
image2(ifelse(Elasticity == 0, NA, Elasticity),
  mar=c(0.5,1.5,5,1), box.offset=.15, labels = c(2,3) )
mtext(side = 3, expression("b)"~ italic("Control")*":"~lambda[
  "elasticities"]), cex = 1.3, line = 1)


   ### MANIPULATION: NO NEST PREDATION ###

StochasticSensP <- stoch.sens(AP, tlimit=100)
Elasticity <- StochasticSensP$elasticities
Sensitivity <- StochasticSensP$sensitivities

dimnames(Sensitivity)<- list(c("0", "1", "2", "3+"))
Sensitivity <- Sensitivity*ifelse(A.mean == 0, NA, 1)
dimnames(Sensitivity)[[2]] <- dimnames(Sensitivity)[[1]]
image2(ifelse(Sensitivity == 0, NA, Sensitivity),
  mar=c(0.5,1.5,5,1), box.offset=.15, labels = c(2,3) )
mtext(side = 3, expression("c)"~ italic("No predation")*":"~
  lambda["senstivities"]), cex = 1.3, line = 1)
dimnames(Elasticity)<- list(c("0", "1", "2", "3+"))
Elasticity <- Elasticity*ifelse(A.mean == 0, NA, 1)
dimnames(Elasticity)[[2]] <- dimnames(Elasticity)[[1]]
image2(ifelse(Elasticity == 0, NA, Elasticity),
  mar=c(0.5,1.5,5,1), box.offset=.15, labels = c(2,3) )
mtext(side = 3, expression("d)"~ italic("No predation")*
  ":"~lambda["elasticities"]), cex = 1.3, line = 1)


   ### STORING AND ADDING ADDITIONAL LABELLING TO THE PLOT ###

par(new=T, bty="n", mfrow=c(1,1), oma=rep(0,4), mar=rep(0,4))
plot(1,1, ylim=c(0,1), xlim=c(0,1), xlab="", ylab ="",
  xaxt="n", yaxt="n", type ="n")
text(x = .075, y = 0.85, expression("Based on "*bold(A)[bold(
  italic(t))["2011-19"]]), cex = 1, adj = 0)
dev.copy(tiff, filename = "StochSensiElast.tiff",
  units="cm", width=16, height=18, res=800, compression =
  c("lzw"));dev.off()
rm(list=ls(pattern="Sens"));rm(Elasticity)









                                        ########################################################################################################
                                        # QUASI-EXTINCTIONS USING YEAR-SPECIFIC MATRICES FROM THE ENTIRE TIME SERIES (EXCLUDING THE LAST YEAR) #
                                        ########################################################################################################


# NOTES: The output from these runs is based on simulations and slightly different from the reported results (Figure 5 in the main text).


   ### CONTROL: NEST PREDATION AT ITS NATURAL LEVEL (AND INITIATING THE PLOT) ###

A.stoch <- stoch.growth.rate(A[-length(A)], maxt = maxt,
  verbose = F)                                                    # calculates the log stochastic growth rate by Tuljapukar's approximation and by simulation
sink("StochasticLambda.txt")
cat("-----------------------------\n")
cat("  --> ALL YEARS (-LAST) <--  \n")
cat("-----------------------------\n");cat("\n")
cat("   --> CONTROL (PREDATORS PRESENT) <--   \n")
print(A.stoch);cat("\n");sink()

A.stoch.proj <- stoch.projection(A[-length(A)], tmax = ExtTime,   # simulates stochastic growth by projection using whole matrix selection techniques in an independently
    AgeClData$N, nreps = Maxruns, verbose = F)                    #   and identically distributed (iid) environment from a set of 2 or more projection matrices
                                                                  #   matrices('A') is a list with the  projection matrices
                                                                                                                                    #   'tmax' is the number of time steps or projection intervals to predict future population size
                                                                  #   nreps ('Maxruns') is the number of iterations (i.e. populations)
                                                                  #   'prob' is a vector of probability for selecting the projection matrices (we assume equal probabilities
A.extin <- stoch.quasi.ext(A[-length(A)], n0 = AgeClData$N,       # estimate the quasi-extinction probability by simulation for a structured population in an independently
  Nx = ExtThres, tmax = ExtTime, maxruns = 10, nreps = Maxruns,   #   and identically distributed stochastic environment
  prob = NULL, sumweight = NULL, verbose = F)                     #   'n0' is the initial population vector; 'A', 'n0','tmax' and 'prob' are the same as above

cex.axis <- 1.8                                                   # size of legends for the axes
dev.new(width=16/2.54, height=24/2.54)
par(bty = "l", cex.axis = 1.6,                                    # setting up the plot
  mar = c(4, 4, 2.5, 0.35), oma = c(1, 1.5, 4, 0.5),
  mfrow = c(3,2))
matplot(A.extin, ylab = "", xlab = "", type = "l", ylim = c(0,1),
  col = rainbow(dim(A.extin)[2]))
tmp <- data.frame(time=1:ExtTime, Ext = apply(A.extin,
  FUN = mean, 1))                                                 # gets the average Q-ext. prob as a function of time
tmp1 <- 0.75                                                      # setting a target prob. of 0.75
tmp <- tmp[which.min(abs(tmp$Ext-mean(tmp1))),]                   # gets the row where these values are closest to being equal the defined target
tmp$time <- ifelse(abs(tmp$Ext-tmp1)<0.10, tmp$time, tmp$time*2)  # if this difference is not even close no line is plotted in the next step
lines(x = rep(tmp$time,2), y = c(-1, tmp$Ext), lwd = 2, lty = 2)  # indicates on the x-axis where this occurs
mtext(side = 3, text = "Control",                                 # adds a label to top axis
  line = 3.7, cex = cex.axis*1.5, outer = FALSE)
mtext(side = 3, text = "(A) 2005-2018",                           # adds a label to top axis
  line = .7, cex = cex.axis, outer = FALSE)
mtext( side = 3, adj = 1.2, cex = cex.axis*.4, line = -10,
  as.expression(substitute(
    lambda[sim] * "=" * Value1 * " (95% CI=" * Value2 * "," *
      Value3 * ")",
    list( sim    = "s",
       Value1 = formatC(A.stoch$sim,  format = "f", digits = 3),
       Value2 = formatC(A.stoch$sim.CI[1],format = "f",
         digits = 3),
       Value3 = formatC(A.stoch$sim.CI[2], format = "f",
         digits = 3)))))


   ### MANIPULATION: NO NEST PREDATION ###

A.stochP <- stoch.growth.rate(AP[-length(AP)], maxt = maxt,
  verbose = F)
sink("StochasticLambda.txt", append = T)
cat("   --> MANIPULATION (NO NEST PREDATION) <--   \n")
print(A.stochP);cat("\n");cat("\n");sink()
A.stoch.projP <- stoch.projection(AP[-length(AP)],
  tmax = ExtTime, AgeClData$N, nreps = Maxruns, verbose = F)
A.extinP <- stoch.quasi.ext(AP[-length(AP)], n0 = AgeClData$N,
  Nx = ExtThres, tmax = ExtTime, maxruns = 10, nreps = Maxruns,
  prob = NULL, sumweight = NULL, verbose = F)

matplot(A.extinP, ylab = "", xlab = "", type = "l", ylim = c(0,1),
  col = rainbow(dim(A.extinP)[2]))
tmp <- data.frame(time=1:ExtTime, Ext = apply(A.extinP,
  FUN = mean, 1))
tmp <- tmp[which.min(abs(tmp$Ext-mean(tmp1))),]
tmp$time <- ifelse(abs(tmp$Ext-tmp1)<0.10, tmp$time, tmp$time*2)
mtext(side = 3, text = "No predation",
  line = 3.7, cex = cex.axis*1.5, outer = FALSE)
mtext(side = 3, text = "(B) 2005-2018",
  line = .7, cex = cex.axis, outer = FALSE)
mtext(side = 2, expression(italic("Prob.(")*"quasi-extinction)"),
  line = -.7, cex = cex.axis, outer = T)
mtext(side = 1, "Time (year)", line = -0.3, cex = cex.axis, outer = T)
mtext( side = 3, adj = 1, cex = cex.axis*.4, line = -10,
  as.expression(substitute(
    lambda[sim] * "=" * Value1 * " (95% CI=" * Value2 * "," * Value3 * ")",
    list( sim    = "s",
          Value1 = formatC(A.stochP$sim,       format = "f", digits = 3),
          Value2 = formatC(A.stochP$sim.CI[1], format = "f", digits = 3),
          Value3 = formatC(A.stochP$sim.CI[2], format = "f", digits = 3)))))









                                        #################################################################################
                                        # QUASI-EXTINCTIONS USING YEAR-SPECIFIC MATRICES FROM THE EARLY PERIOD: 2005-10 #
                                        #################################################################################


   ### SELECTION OF YEARS FOR THE EARLY PERIOD ###

tmp <- levels(VitalAdult$year)[                                   # gets early years (2005-2010) from 'VitalAdult'
  as.numeric(levels(VitalAdult$year))<=2010]
Tmp <- as.numeric(row.names(subset(VitalAdult, year%in%tmp)))


   ### CONTROL: NEST PREDATION AT ITS NATURAL LEVEL ###

A.2005_2010 <- A[Tmp]
A.stoch.2005_2010 <- stoch.growth.rate(A.2005_2010,
  maxt = maxt, verbose = F)
sink("StochasticLambda.txt",append = T)
cat("-----------------------------\n")
cat("   -->   EARLY PERIOD   <--  \n")
cat("-----------------------------\n");cat("\n")
cat("   --> CONTROL (PREDATORS PRESENT) <--   \n")
print(A.stoch.2005_2010);cat("\n");sink()
A.stoch.proj.2005_2010 <- stoch.projection(A.2005_2010,
  tmax = ExtTime, AgeClData$N, nreps = Maxruns, verbose = F)
A.extin.2005_2010 <- stoch.quasi.ext(A.2005_2010,
  n0 = AgeClData$N, Nx = ExtThres, tmax = ExtTime,
  maxruns = 10, nreps = Maxruns, prob = NULL,
  sumweight = NULL, verbose = F)

matplot(A.extin.2005_2010, ylab = "", xlab = "", type = "l",
  ylim = c(0,1),col = rainbow(dim(A.extin.2005_2010)[2]))
tmp <- data.frame(time=1:ExtTime, Ext = apply(A.extin.2005_2010,
  FUN =mean,1))
tmp <- tmp[which.min(abs(tmp$Ext-mean(tmp1))),]
tmp$time <- ifelse(abs(tmp$Ext-tmp1)<0.10, tmp$time, tmp$time*2)
mtext(side = 3, "(C) 2005-2010",
  line = .7, cex = cex.axis, outer = FALSE)
mtext( side = 3, adj = 1, cex = cex.axis*.4, line = -10,
  as.expression(substitute(
    lambda[sim] * "=" * Value1 * " (95% CI=" * Value2 * "," *
      Value3 * ")",
    list( sim    = "s",
      Value1 = formatC(A.stoch.2005_2010$sim,
        format = "f", digits = 3),
      Value2 = formatC(A.stoch.2005_2010$sim.CI[1],
        format = "f", digits = 3),
      Value3 = formatC(A.stoch.2005_2010$sim.CI[2],
        format = "f", digits = 3)))))


   ### MANIPULATION: NO NEST PREDATION ###

A.2005_2010P <- AP[Tmp]
A.stoch.2005_2010P <- stoch.growth.rate(A.2005_2010P,
  maxt = maxt,verbose = F)
sink("StochasticLambda.txt",append = T)
cat("   --> MANIPULATION (NO NEST PREDATION) <--   \n")
print(A.stoch.2005_2010P);cat("\n");cat("\n");sink()
A.stoch.proj.2005_2010P <- stoch.projection(A.2005_2010P,
  tmax = ExtTime, AgeClData$N, nreps = Maxruns, verbose = F)
A.extin.2005_2010P <- stoch.quasi.ext(A.2005_2010P, n0 =
  AgeClData$N, Nx = ExtThres, tmax = ExtTime, maxruns = 10,
  nreps = Maxruns, prob = NULL,
  sumweight = NULL, verbose = F)

matplot(A.extin.2005_2010P, ylab = "", xlab = "", type = "l",
  ylim = c(0,1),col = rainbow(dim(A.extin.2005_2010P)[2]))
tmp <- data.frame(time=1:ExtTime, Ext = apply(A.extin.2005_2010P,
  FUN =mean,1))
tmp <- tmp[which.min(abs(tmp$Ext-mean(tmp1))),]
tmp$time <- ifelse(abs(tmp$Ext-tmp1)<0.10, tmp$time, tmp$time*2)
mtext(side = 3, "(D) 2005-2010",
  line = .7, cex = cex.axis, outer = FALSE)
mtext( side = 3, adj = 1, cex = cex.axis*.4, line = -10,
  as.expression(substitute(
    lambda[sim] * "=" * Value1 * " (95% CI=" * Value2 * "," *
      Value3 * ")",
    list( sim    = "s",
      Value1 = formatC(A.stoch.2005_2010P$sim,
        format = "f", digits = 3),
      Value2 = formatC(A.stoch.2005_2010P$sim.CI[1],
        format = "f", digits = 3),
      Value3 = formatC(A.stoch.2005_2010P$sim.CI[2],
        format = "f", digits = 3)))))









                                        ################################################################################
                                        # QUASI-EXTINCTIONS USING YEAR-SPECIFIC MATRICES FROM THE LATE PERIOD: 2011-18 #
                                        ################################################################################


   ### SELECTION OF YEARS FOR THE LATE PERIOD ###

tmp <- levels(VitalAdult$year)[                                   # gets the years for the late period: i.e., the years after the early period excluding the last year from 'VitalAdult'
  as.numeric(levels(VitalAdult$year)) >
    as.numeric(levels(VitalAdult$year)[Tmp[length(Tmp)]]) &
  as.numeric(levels(VitalAdult$year)) <
    as.numeric(levels(VitalAdult$year)[dim(VitalAdult)[1]])]
Tmp <- as.numeric(row.names(subset(VitalAdult, year%in%tmp)))


   ### CONTROL: NEST PREDATION AT ITS NATURAL LEVEL ###

A.2011_ <- A[Tmp]
A.stoch.2011_ <- stoch.growth.rate(A.2011_, maxt = maxt,
  verbose = F)
sink("StochasticLambda.txt",append = T)
cat("-----------------------------\n")
cat("   -->   LATE PERIOD   <--   \n")
cat("-----------------------------\n");cat("\n")
cat("   --> CONTROL (PREDATORS PRESENT) <--   \n")
print(A.stoch.2011_);cat("\n");sink()
A.stoch.proj.2011_ <- stoch.projection(A.2011_,
  tmax = ExtTime, AgeClData$N, nreps = Maxruns, verbose = F)
A.extin.2011_ <- stoch.quasi.ext(A.2011_,
  n0 = AgeClData$N, Nx = ExtThres, tmax = ExtTime, maxruns = 10,
  nreps = Maxruns, prob = NULL,
  sumweight = NULL, verbose = F)

matplot(A.extin.2011_, ylab = "", xlab = "", type = "l",
  ylim = c(0,1),col = rainbow(dim(A.extin.2011_)[2]))
tmp <- data.frame(time=1:ExtTime, Ext = apply(A.extin.2011_,
  FUN =mean,1))
tmp <- tmp[which.min(abs(tmp$Ext-mean(tmp1))),]
tmp$time <- ifelse(abs(tmp$Ext-tmp1)<0.25, tmp$time, tmp$time*2)
lines(x = rep(tmp$time,2), y = c(-1, tmp$Ext), lwd = 2, lty = 2)
mtext(side = 3, "(E) 2011-2018",
  line = .7, cex = cex.axis, outer = FALSE)
mtext( side = 3, adj = 1, cex = cex.axis*.4, line = -10,
  as.expression(substitute(
    lambda[sim] * "=" * Value1 * " (95% CI=" * Value2 * "," *
      Value3 * ")",
    list( sim    = "s",
      Value1 = formatC(A.stoch.2011_$sim,
        format = "f", digits = 3),
      Value2 = formatC(A.stoch.2011_$sim.CI[1],
        format = "f", digits = 3),
      Value3 = formatC(A.stoch.2011_$sim.CI[2],
        format = "f", digits = 3)))))


   ### MANIPULATION: NO NEST PREDATION (AND SAVING THE PLOT) ###

A.2011_P <- AP[Tmp]
A.stoch.2011_P <- stoch.growth.rate(A.2011_P, maxt = maxt,
  verbose = F)
sink("StochasticLambda.txt",append = T)
cat("   --> MANIPULATION (NO NEST PREDATION) <--   \n")
print(A.stoch.2011_P);sink()
A.stoch.proj.2011_P <- stoch.projection(A.2011_P,
  tmax = ExtTime, AgeClData$N, nreps = Maxruns, verbose = F)
A.extin.2011_P <- stoch.quasi.ext(A.2011_P,
  n0 = AgeClData$N, Nx = ExtThres, tmax = ExtTime, maxruns = 10,
  nreps = Maxruns, prob = NULL,
  sumweight = NULL, verbose = F)

matplot(A.extin.2011_P, ylab = "", xlab = "", type = "l",
  ylim = c(0,1),col = rainbow(dim(A.extin.2011_P)[2]))
tmp <- data.frame(time=1:ExtTime, Ext = apply(A.extin.2011_P,
  FUN =mean,1))
tmp <- tmp[which.min(abs(tmp$Ext-mean(tmp1))),]
tmp$time <- ifelse(abs(tmp$Ext-tmp1)<0.1, tmp$time, tmp$time*2)
mtext(side = 3, "(F) 2011-2018",
  line = .7, cex = cex.axis, outer = FALSE)
mtext( side = 3, adj = 1, cex = cex.axis*.4, line = -10,
  as.expression(substitute(
    lambda[sim] * "=" * Value1 * " (95% CI=" * Value2 * "," *
      Value3 * ")",
    list( sim    = "s",
      Value1 = formatC(A.stoch.2011_P$sim,
        format = "f", digits = 3),
      Value2 = formatC(A.stoch.2011_P$sim.CI[1],
        format = "f", digits = 3),
      Value3 = formatC(A.stoch.2011_P$sim.CI[2],
        format = "f", digits = 3)))))
dev.copy(tiff, filename = "QuasiExtin.tiff", units="cm",          # saves the plot externally as a .tiff-file
  width=16, height=24, res=800, compression = c("lzw"));dev.off()
rm(list=ls(pattern="A.extin"));rm(list=ls(pattern="A.stoch"))








                                        #########################
                                        # CONTOUR PLOT: LAMBDAS #
                                        #########################


# NOTE: Create a contour plot showing two vital rates, keeping the third constant at its average value.


   ### CREATING A FUNCTION FOR EXTRACTING LAMBDAS ###

LamFunc <- function(Pa, F, Pj, nCl)                               # function for extracting the lambdas using a constant matrix
  {  pop.projection(A =
              matrix(
                c(0,       0,       0,       F,
                  Pj,      0,       0,       0,
                  0,       Pa,      0,       0,
                  0,       0,       Pa,      Pa),
                ncol = 4, byrow = F),
      n = nCl)$lambda                                             # extracting the lambda
   }


   ### RANGE AND AVERAGE FOR THE VITAL RATES ADDED TO THE PLOTS ###

Pa.range <- c(min(VitalAdult$survMean, na.rm = T),                # range in adult survival
  max(VitalAdult$survMean, na.rm = T))
PaCI.range <- c(min(VitalAdult$survLcl, na.rm = T),               # range in the CIs for adult survival
  max(VitalAdult$survUcl, na.rm = T))
Pj.range <- c(min(VitalOffsp$survMean[VitalOffsp$survMean>0],
  na.rm = T),                                                     # range in offspring survival
  max(VitalOffsp$survMean[VitalOffsp$survMean>0], na.rm = T))
PjCI.range <- c(min(VitalOffsp$survLcl[VitalOffsp$survLcl>0],
  na.rm = T),                                                     # range in the CIs for offspring survival
  max(VitalOffsp$survUcl[VitalOffsp$survUcl>0], na.rm = T))
Pj.mean <- dataExtTime$P0[1]
Pa.mean <- dataExtTime$P13[1]
F.mean <- dataExtTime$F4.contr[1]                                 # CONTROL (NEST PREDATION AT ITS NATURAL LEVEL): mean fecundity
F.range <- range(dataExtTime$F4.contr)                            # CONTROL (NEST PREDATION AT ITS NATURAL LEVEL): range in fecundity
FSE.range <- range(c(
  (ifelse(VitalOffsp$ClutchStart.se1<0, 0,
     VitalOffsp$ClutchStart.se1)*VitalOffsp$survMean*.5),
  (ifelse(VitalOffsp$ClutchStart.se2<0, 0,
     VitalOffsp$ClutchStart.se2)*VitalOffsp$survMean*.5)),
   na.rm=T)                                                       # CONTROL: range in the SEs for fecundity (a product of sex ratio, survival and start clutch size)
F.meanP <- dataExtTime$F4.NoPred[1]                               # MANIPULATION (NO NEST PREDATION): mean fecundity
F.rangeP <- range(dataExtTime$F4.NoPred)                          # MANIPULATION (NO NEST PREDATION): range in fecundity
FSE.rangeP <- range(c(
  (ifelse(VitalOffsp$ClutchEnd.se1<0, 0,
     VitalOffsp$ClutchEnd.se1)*VitalOffsp$survMean*.5),
  (ifelse(VitalOffsp$ClutchEnd.se2<0, 0,
     VitalOffsp$ClutchEnd.se2)*VitalOffsp$survMean*.5)),
   na.rm=T)                                                       # MANIPULATION (NO NEST PREDATION): range in the SEs for fecundity (a product of sex ratio, survival, predation and start clutch size)


   ### SETTING UP THE PLOT ###

dev.new(width = 24/2.54, height = 10/2.54);
par(mfrow = c(1,3),
    mar = c(4.3,4.3, 1.3, .5),
    cex.lab = 1.4, cex.main = 1.1)

col <- c("darkorange4", "aquamarine4")                            # colours
Pa <- seq(0.5,     1,   length.out = 100)                         # range in adult survival for the plot
F <- seq(min(c(FSE.range, FSE.rangeP)),
   max(c(FSE.range, FSE.rangeP))*1.1,
    length.out = length(Pa))                                      # range in fecundity for the plot


   ### A) ADULT APPARENT SURVIVAL AND FECUNDITY KEEPING JUVENILE SURVIVAL CONSTANT ###

dev <- rep(1,2) + c(0.0039,-0.0039)                               # the difference from the truth for the lines for CONTROL (NEST PREDATION AT ITS NATURAL LEVEL) and the MANIPULATION (NO NEST PREDATION) scenarios not to overlap
z <- matrix(nrow = length(Pa), ncol = length(Pa))                 # setting up the response variables (lambdas)
for (i in 1:length(Pa))                                           # filling it with values
  { for (j in 1:length(F))
      {  z[j,i] <- log(LamFunc(Pa = Pa[i], F = F[j],
            Pj = Pj.mean, nCl = rep(250,4))) }
  }
image(y = Pa, x = F, z, col = topo.colors(1000),                  # showing the results
  xlab = expression("Fecundity ("*italic(F)["3+"]*")"),
  ylab = expression("Adult survial ("*italic(P)["3+"]*")"),
  main = expression("(A) Average juvenile suvival ("*
     bar(italic(P)["0"])*")"))
contour(y = Pa, x = F, z, add = T,                                # adding contours (lambda = 0)
  levels = 0, lwd = 8, labels = "",
  col = "white")
contour(y = Pa, x = F, z, add = T,                                # adding contours
  col = "red", cex = 1.5, lwd = 2,
  labcex = 2, nlevel = 7)
box()
lines(x = rep(F.mean,2), y = c(PaCI.range[1], PaCI.range[2]),
  col = col[1])                                                   # adding the range in 95% CIs for adult survival to the Y-axis for the CONTROL (NEST PREDATION AT ITS NATURAL LEVEL)
lines(x = rep(F.mean,2), y = c(Pa.range[1],Pa.range[2]),lwd = 5,
  col = col[1])                                                   # adding the range in point estimates for adult survival to the Y-axis for the CONTROL (NEST PREDATION AT ITS NATURAL LEVEL)
lines(x = rep(F.meanP,2), y = c(PaCI.range[1], PaCI.range[2]),
  col = col[2])                                                   # adding the range in 95% CIs for adult survival to the Y-axis for the MANIPULATION (NO NEST PREDATION)
lines(x = rep(F.meanP,2), y = c(Pa.range[1],Pa.range[2]),lwd = 5,
 col = col[2])                                                    # adding the range in point estimates for adult survival to the Y-axis for the MANIPULATION (NO NEST PREDATION)

lines(y = rep(Pa.mean,2)*dev[1], x = c(F.range[1], F.range[2]),   # adding the range in point estimates for the fecundities the Y-axis: CONTROL (NEST PREDATION AT ITS NATURAL LEVEL)
  lwd = 7, col = col[1])
lines(y = rep(Pa.mean,2)*dev[1], x = c(FSE.range[1],
  FSE.range[2]), col = col[1])                                    # adding the range in +/- SEs for the fecundities to the X-axis for the CONTROL (NEST PREDATION AT ITS NATURAL LEVEL)
lines(y = rep(Pa.mean,2)*dev[2], x = c(F.rangeP[1], F.rangeP[2]), # adding the range in +/- SEs for the fecundities to the X-axis for the MANIPULATION (NO NEST PREDATION)
  col = col[2], lwd=7)
lines(y = rep(Pa.mean,2)*dev[2], x = c(FSE.rangeP[1],
  FSE.rangeP[2]), lwd = 4, col = col[2])                          # adding the range in point estimates for the Y-axis: MANIPULATION (NO NEST PREDATION)
points((Pa.mean*dev[2])~F.meanP, cex = 3, pch = 21, bg = col[2],  # adding the point for the of average fecundity and adult survival for MANIPULATION (NO NEST PREDATION)
  col = "white")
text(y = (Pa.mean-.04), x = (F.meanP+.04), adj = 0, cex = 1.9,
  "No predation", col= col[2])                                    # labelling the range and point estimates for the MANIPULATION (NO NEST PREDATION) scenario
points(Pa.mean*dev[1]~F.mean,    cex = 3, pch = 21, bg = col[1],
  col = "white")                                                  # adding the point for the of average fecundity and adult survival for CONTROL (NEST PREDATION AT ITS NATURAL LEVEL)
text(y = (Pa.mean+.04), x = (F.mean+.04), adj = 0, cex = 1.9,
  "Control", col= col[1])                                         # labelling the range and point estimates for the CONTROL (NEST PREDATION AT ITS NATURAL LEVEL) scenario


   ### B) JUVENILE APPARENT SURVVIAL AND FECUNDITY KEEPING ADULT SURVIVAL CONSTANT ###

dev <- rep(1,2) + c(0.017,-0.017)
Pj <- seq(0.05,     .87,   length.out = 100)
z <- matrix(nrow = length(Pj), ncol = length(Pj))
for (i in 1:length(Pj))
  { for (j in 1:length(F))
      {  z[j,i] <- log(LamFunc(Pj = Pj[i], F = F[j],
            Pa = Pa.mean, nCl = rep(250,4))) }
  }
image(y = Pj, x = F, z, col = topo.colors(1000),
  xlab = expression("Fecundity ("*italic(F)["3+"]*")"),
  ylab = expression("Juvenile survial ("*italic(P)["o"]*")"),
  main = expression("(B) Average adult suvival ("*
    bar(italic(P)["3+"])*")"))
contour(y = Pj, x = F, z, add = T,
  levels = 0, lwd = 8, labels = "",
  col = "white")
contour(y = Pj, x = F, z, add = T,
  col = "red", cex = 1.5, lwd = 2,
  labcex = 2, nlevel = 7)
box()
lines(x = rep(F.mean,2), y = c(PjCI.range[1],
  PjCI.range[2]), col = col[1])
lines(x = rep(F.mean,2), y = c(Pj.range[1],
  Pj.range[2]),lwd = 5,col = col[1])
lines(x = rep(F.meanP,2), y = c(PjCI.range[1],
  PjCI.range[2]), col = col[2])
lines(x = rep(F.meanP,2), y = c(Pj.range[1],
  Pj.range[2]),lwd = 5,col = col[2])
lines(y = rep(Pj.mean,2)*dev[1], x = c(F.range[1],
  F.range[2]), lwd = 7, col = col[1])
lines(y = rep(Pj.mean,2)*dev[1], x = c(FSE.range[1],
  FSE.range[2]), col = col[1])
lines(y = rep(Pj.mean,2)*dev[2], x = c(F.rangeP[1], F.rangeP[2]),
  col = col[2], lwd=7)
lines(y = rep(Pj.mean,2)*dev[2], x = c(FSE.rangeP[1],
  FSE.rangeP[2]), lwd = 4, col = col[2])
points((Pj.mean*dev[2])~F.meanP, cex = 3, pch = 21,
  bg = col[2],col = "white")
points(Pj.mean*dev[1]~F.mean, cex = 3, pch = 21,
  bg = col[1],col = "white")


   ### C) JUVENILE AND ADULT APPARENT SURVVIAL KEEPING FECUNDITY CONSTANT ###

Pj <- seq(0.05,     .87,   length.out = 100)
Pa <- seq(0.5,     1,   length.out = length(Pj))
z <- matrix(nrow = length(Pj), ncol = length(Pj))
for (i in 1:length(Pa))
  { for (j in 1:length(Pj))
      {  z[j,i] <- log(LamFunc(Pa = Pa[i], Pj = Pj[j],
            F = F.mean, nCl = rep(250,4))) }
  }
image(y = Pa, x = Pj, z, col = topo.colors(1000),
  xlab = expression("Juvenile survial ("*italic(P)["0"]*")"),
  ylab = expression("Adult suvival ("*italic(P)["3+"]*")"),
  main =expression("(C) Average fecundity ("*italic(F)["3+"]*")"))
contour(y = Pj, x = F, z, add = T,
  levels = 0, lwd = 8, labels = "",
  col = "white")
contour(y = Pj, x = F, z, add = T,
  col = "red", cex = 1.5, lwd = 2,
  labcex = 2, nlevel = 6)
box()
lines(x = rep(Pj.mean,2), y = c(PaCI.range[1], PaCI.range[2]))
lines(x = rep(Pj.mean,2), y = c(Pa.range[1],   Pa.range[2]),
  lwd = 5)
lines(y = rep(Pa.mean,2), x = c(PjCI.range[1], PjCI.range[2]))
lines(y = rep(Pa.mean,2), x = c(Pj.range[1],   Pj.range[2]),
  lwd = 5)
points(Pa.mean~Pj.mean,   cex = 3, pch = 16, col = "black")
text(x=0.10, y = c(0.79), col = "red",
  expression(lambda*"<0"%->%""), adj = 0,
  cex = 1.5)
text(x=0.10, y = c(0.76), col = "red",
  expression("population"), adj = 0,
  cex = 1.5)
text(x=0.10, y = c(0.73), col = "red",
  expression("increase"), adj = 0,
  cex = 1.5)
text(x=0.10, y = c(0.95), col = "red",
  expression(lambda*">0"%->%""), adj = 0,
  cex = 1.5)
text(x=0.10, y = c(0.92), col = "red",
  expression("population"), adj = 0,
  cex = 1.5)
text(x=0.10, y = c(0.89), col = "red",
  expression("decline"), adj = 0,
  cex = 1.5)
arrows(x0=0.6638000,x1=0.5952073, y=0.6781989,y1= 0.7442107,
  lwd = 8, col = "white")
arrows(x0=0.6638000,x1=0.5952073, y=0.6781989,y1= 0.7442107,
  lwd = 2, col = "red")
text(x=0.65, y = c(0.65), col = "red",
  expression(lambda==0%->%""), adj = 0.5,
  cex = 1.5)
text(x=0.65, y = c(0.62), col = "red",
  expression("zero growth"), adj = 0.5,
  cex = 1.5)
dev.copy(tiff, filename = "Lambdas-Deterministic.tiff",
  units="cm",width=26, height=10, res=200, compression = c("lzw"))
dev.off()
rm(Pj, Pj.range, Pj.mean, Pa, Pa.range, Pa.mean, F, F.range,
  F.mean, F.meanP, F.rangeP, FSE.rangeP, FSE.range, PaCI.range,
  PjCI.range, i, j, z, dev, col);rm(list=ls(pattern="mp"))









                                        ###############################################################################
                                        # MULTIPLE REGRESSION APPROACH - CONTROL: NEST PREDATION AT ITS NATURAL LEVEL #
                                        ###############################################################################


   ### SCALING THE PREDICTORS ###

tmp <- grep(x=dataExtTime$year, pattern = "average|Early|Late")   # selects the rows for 'year' in 'dataExtinTime' that either contains 'average', 'Early' or 'Late'
tmp <- dataExtTime[-c(tmp),]
tmp$F4.sc <- scale(tmp$F4.contr, scale = T)                       # scaled fecundity ('F4.sc'; zero average and 1 SD)
tmp$P0.sc <- scale(tmp$P0, scale = T)                             # scaled juvenile apparent survival ('P0.sc'; zero average and 1 SD)
tmp$P13.sc <- scale(tmp$P13, scale = T)                           # scaled adult apparent survival ('P13.sc'; zero average and 1 SD)


   ### FITTING MODELS AND REVIEWING DIAGNOSTICS PLOTS ON-SCREEN ###

modAsJsR <- lm(log(lambda.contr)~F4.sc+P0.sc+P13.sc, tmp)         # runs the model with all three vital rates as predictors (scaled)
modAsJsR.R2part <- partialR2(modAsJsR)                            # extracts partial R-square value
dev.new();par(mfrow = c(2,2))
plot(modAsJsR)
modAs <- lm(log(lambda.contr)~P13, tmp)                           # model for adult apparent survival
dev.new();par(mfrow = c(2,2))
plot(modAs)
modJs <- lm(log(lambda.contr)~P0, tmp)                            # model for juvenile survival
dev.new();par(mfrow = c(2,2))
plot(modJs)
modR <- lm(log(lambda.contr)~F4.contr, tmp)                       # model for recruitment
dev.new();par(mfrow = c(2,2))
plot(modR)


   ### SETTING UP THE PLOT ###

dev.new(width=14/2.54, height=23/2.54);par(bty = "l",             # setting up the plot
  mar = c(4, 4, 2.5, 0.35), oma = c(1, 1.5, 1, 0.5),
  mfrow = c(3,1), cex.lab = 1.5, cex.axis = 1.6)


   ### A) PLOTTING THE EFFECT OF FECUNDITY 'F4' KEEPING THE OTHERS CONSTANT ###

tmp1 <- data.frame(response = log(tmp$lambda.contr), predictor =  # re-fitting the model
  tmp$F4.contr)
mod <- lm(response~predictor, tmp1)

newdata <- data.frame(predictor =                                 # data to be used in making the predictions +/- 1 SE
  seq(min(tmp1$predictor,na.rm = T),
  max(tmp1$predictor,na.rm = T), length.out = 100))
newdata$fit <- predict(mod, newdata)
newdata$se.fit1 <- newdata$fit + predict(mod,
  newdata, se.fit = T)$se.fit
newdata$se.fit2 <- newdata$fit - predict(mod,
  newdata, se.fit = T)$se.fit
plot(tmp1$response ~ tmp1$predictor, las = 0,                     # setting up the plot (axes ect.)
  xlab = "", ylab = "", type = "n", ylim = c(min( c(newdata$fit,
  tmp1$response)), max( c(newdata$fit, tmp1$response) )))
abline(h=0, lty = 2, col = "dark grey")
points(tmp1$response ~ tmp1$predictor, cex = 2.5,
  col = colors()[c(131)], pch = 16)
lines(newdata$fit~newdata$predictor, lwd = 2)                     # plotting a line showing the predictions
lines(newdata$se.fit2~newdata$predictor, lwd = 1.5, lty = 2)      # plotting a line showing the predictions + 1 SE
lines(newdata$se.fit1~newdata$predictor, lwd = 1.5, lty = 2)      # plotting a line showing the predictions + 1 SE
a <- formatC(summary(mod)$coef[1,1],format = "f", digits = 2)     # extracts the estimated intercept
a.se <- ifelse(
  round(summary(mod)$coef[1,2],3) == 0,                           # extracts the estimated SE for the intercept
  "<0.001", formatC(summary(mod)$coef[1,2],
  format = "f", digits = 2))
b <- formatC(summary(mod)$coef[2,1],format = "f", digits = 2)     # extracts the estimated slope
b.se <- ifelse(
  round(summary(mod)$coef[2,2],3) == 0,                           # extracts the estimated SE for the slope
  "<0.001", formatC(summary(mod)$coef[2,2],
  format = "f", digits = 2))
sig <- formatC(summary(mod)$sigma,format = "f", digits = 2)       # extracts the residual st. dev. (sigma)
r.sq <- formatC(summary(mod)$r.sq,format = "f", digits = 2)       # extracts the R-square for the model
r.qPart <- formatC(modAsJsR.R2part$PartialData[2,2],format = "f",
  digits = 2)                                                     # extracts the R-square for the model
mtext(side = 1, adj = .9, cex = 1, line = -1.5,                   # adds information about the model to the plot
  as.expression( substitute(italic(lambda) == Value * "(" *
  Valueb* ")" + Value1 * "(" * Value1b* ")" %*% italic(F)["3+"] *
  "  (" * italic(sigma)* " = " * Value2 * ")",
  list( Value = a, Valueb = a.se, Value1 = b, Value1b = b.se,
  Value2 = sig) ) ) )
mtext(side = 3, adj = .1, cex = 1.5, line = -1,                   # adds the 'R-square' value (see 'r.sq' above for details) to the plot
  as.expression( substitute(italic(R^2) == Value,
  list( Value = r.sq) ) ) )
mtext(side = 3, adj = .1, cex = 1.5, line = -4,                   # adds the 'R-square' value (see 'r.sq' above for details) to the plot
  as.expression( substitute(  italic(R[part]^2) == Value,
   list( Value = r.qPart) ) ) )
mtext(side = 3, adj = 0, cex = 1.5, line = 1.3,"a)" )             # labels the sub-plot
mtext(side = 1, expression("Fecundity ("*italic(F)["3+"]*")"),
    line = 3.7, cex = 2)                                          # labels the sub-plot


   ### B) PLOTTING THE EFFECT OF APPARENT ADULT SURVIVAL: 'P13' KEEPING THE OTHERS CONSTANT ###

tmp1 <- data.frame(response = log(tmp$lambda.contr),
  predictor = tmp$P13)
mod <- lm(response~predictor, tmp1)

newdata <- data.frame(predictor = seq(min(tmp1$predictor,
  na.rm = T), 0.95, length.out = 100))
newdata$extrapolation <- ifelse(newdata$predictor <=
  max(tmp1$predictor), FALSE, TRUE)
newdata$fit <- predict(mod, newdata)
newdata$se.fit1 <- newdata$fit + predict(mod,
  newdata, se.fit = T)$se.fit
newdata$se.fit2 <- newdata$fit - predict(mod,
  newdata, se.fit = T)$se.fit
plot(tmp1$response ~ tmp1$predictor, las = 0,
  xlab = "", ylab = "", type = "n", ylim = c(min( c(newdata$fit,
  tmp1$response)), max( c(newdata$fit, tmp1$response) )),
  xlim = c(min(tmp1$predictor), 0.95))
abline(h=0, lty = 2, col = "dark grey")
points(tmp1$response ~ tmp1$predictor, cex = 2.5,
  col = colors()[c(136)], pch = 16)
lines(newdata$fit[newdata$extrapolation!="TRUE"]~
  newdata$predictor[newdata$extrapolation!="TRUE"], lwd = 2)
lines(newdata$fit~newdata$predictor, lwd = 1)
lines(newdata$se.fit2~newdata$predictor, lwd = 1.5, lty = 2)
lines(newdata$se.fit1~newdata$predictor, lwd = 1.5, lty = 2)
a <- formatC(summary(mod)$coef[1,1],format = "f", digits = 2)
a.se <- ifelse(round(summary(mod)$coef[1,2],3) == 0,"<0.001",
  formatC(summary(mod)$coef[1,2], format = "f", digits = 2))
b <- formatC(summary(mod)$coef[2,1],format = "f", digits = 2)
b.se <- ifelse(round(summary(mod)$coef[2,2],3) == 0, "<0.001",
  formatC(summary(mod)$coef[2,2],format = "f", digits = 2))
sig <- formatC(summary(mod)$sigma,format = "f", digits = 2)
r.sq <- formatC(summary(mod)$r.sq,format = "f", digits = 2)
r.qPart <- formatC(modAsJsR.R2part$PartialData[4,2],format = "f",
  digits = 2)
mtext(side = 1, adj = .9, cex = 1, line = -1.5,
  as.expression( substitute(italic(lambda) == Value * "(" *
  Valueb* ")" + Value1 * "(" * Value1b* ")" %*% italic(Phi)["a"] *
  "  (" * italic(sigma)* " = " * Value2 * ")", list( Value = a,
  Valueb = a.se, Value1 = b, Value1b = b.se, Value2 = sig) ) ) )
mtext(side = 3, adj = .1, cex = 1.5, line = -1,
  as.expression( substitute(italic(R^2) == Value,
  list( Value = r.sq) ) ) )
mtext(side = 3, adj = .1, cex = 1.5, line = -4,
  as.expression( substitute(italic(R[part]^2) == Value,
  list( Value = r.qPart) ) ) )
abline(h=0, lty = 2, col = "dark grey")
mtext(side = 3, adj = 0, cex = 1.5, line = 1.3,"b)" )
mtext(side = 1, expression("Adult survival ("*italic(Phi)["a"]*")"),
  line = 3.7, cex = 2)
mtext(side = 2, expression("Growth rate ("*italic(lambda)*")"),
 line = 2.7, cex = 2)


   ### C) PLOTTING THE EFFECT OF APPARENT JUVENILE SURVIVAL: 'P0' KEEPING THE OTHERS CONSTANT ###

tmp1 <- data.frame(response = log(tmp$lambda.contr),
  predictor = tmp$P0)
mod <- lm(response~predictor, tmp1)

newdata <- data.frame(predictor = seq(min(tmp1$predictor,
  na.rm = T), 0.6, length.out = 100))
newdata$extrapolation <- ifelse(newdata$predictor <=
  max(tmp1$predictor), FALSE, TRUE)
newdata$fit <- predict(mod, newdata)
newdata$se.fit1 <- newdata$fit + predict(mod,
  newdata, se.fit = T)$se.fit
newdata$se.fit2 <- newdata$fit - predict(mod,
  newdata, se.fit = T)$se.fit
plot(tmp1$response ~ tmp1$predictor, las = 0,
  xlab = "", ylab = "", type = "n", ylim = c(min( c(newdata$fit,
  tmp1$response)), max( c(newdata$fit, tmp1$response) )),
  xlim = c(min(tmp1$predictor), max(tmp1$predictor)))
abline(h = 0, lty = 2, col = "dark grey")
points(tmp1$response ~ tmp1$predictor, cex = 2.5,
  col = colors()[c(136)], pch = 16)
lines(newdata$fit[newdata$extrapolation!="TRUE"]~
  newdata$predictor[newdata$extrapolation!="TRUE"], lwd = 2)
lines(newdata$fit~newdata$predictor, lwd = 1)
lines(newdata$se.fit2~newdata$predictor, lwd = 1.5, lty = 2)
lines(newdata$se.fit1~newdata$predictor, lwd = 1.5, lty = 2)
a <- formatC(summary(mod)$coef[1,1],format = "f", digits = 2)
a.se <- ifelse(round(summary(mod)$coef[1,2],3) == 0,"<0.001",
  formatC(summary(mod)$coef[1,2], format = "f", digits = 2))
b <- formatC(summary(mod)$coef[2,1],format = "f", digits = 2)
b.se <- ifelse(round(summary(mod)$coef[2,2],3) == 0, "<0.001",
  formatC(summary(mod)$coef[2,2],format = "f", digits = 2))
sig <- formatC(summary(mod)$sigma,format = "f", digits = 2)
r.sq <- formatC(summary(mod)$r.sq,format = "f", digits = 2)
r.qPart <- formatC(modAsJsR.R2part$PartialData[3,2],format = "f",
  digits = 2)
mtext(side = 1, adj = .9, cex = 1, line = -1.5,
  as.expression( substitute(italic(lambda) == Value * "(" *
  Valueb* ")" + Value1 * "(" * Value1b* ")" %*% italic(Phi)["j"] *
  "  (" * italic(sigma)* " = " * Value2 * ")", list( Value = a,
  Valueb = a.se, Value1 = b, Value1b = b.se, Value2 = sig) ) ) )
mtext(side = 3, adj = .1, cex = 1.5, line = -1,
  as.expression( substitute(italic(R^2) == Value,
  list( Value = r.sq) ) ) )
mtext(side = 3, adj = .1, cex = 1.5, line = -4,
  as.expression( substitute(italic(R[part]^2) == Value,
  list( Value = r.qPart) ) ) )
abline(h=0, lty = 2, col = "dark grey")
mtext(side = 3, adj = 0, cex = 1.5, line = 1.3,"b)" )
mtext(side = 1, expression("Juvenile survival ("*italic(Phi)["j"]*")"),
  line = 3.7, cex = 2)


   ### STORING THE RESULTS ###

dev.copy(tiff, filename = "Lambda~CONTROL(regression).tiff",      # saves the plot externally as a .tiff-file
  units="cm", width=14, height=23, res=200,
  compression = c("lzw"));dev.off()

sink("Lambda~CONTROL(regression).txt")                            # storing model output (summary) in a .txt-file
cat(" ALL THREE VITAL RATES (SCALED) AS PREDICTORS\n")
print(summary(modAsJsR));cat("\n")
cat("  PARTIAL R-SQUARES  \n")
print(modAsJsR.R2part$PartialData);cat("\n");cat("\n")
cat("A)	THE EFFECT OF FECUNDITY 'F4' KEEPING THE OTHERS CONSTANT\n")
print(summary(modR));cat("\n");cat("\n")
cat("B) THE EFFECT OF APPARENT ADULT SURVIVAL: 'P13' KEEPING THE OTHERS CONSTANT\n")
print(summary(modAs));cat("\n");cat("\n")
cat("C) THE EFFECT OF APPARENT JUVENILE SURVIVAL: 'P0' KEEPING THE OTHERS CONSTANT\n")
print(summary(modJs));cat("\n");cat("\n")
sink()
rm(sig, r.sq, r.qPart, a, a.se, b, b.se, newdata)
rm(list=ls(pattern="mp"));rm(list=ls(pattern="mod"))










                                        ########################################################################################
                                        # SETTING UP THE AVERAGE MATRICES FOR THE FIXED LIFE TABLE RESPONSE EXPERIMENTS (LTRE) #
                                        ########################################################################################


   ### ALL YEARS - CONTROL: NEST PREDATION AT ITS NATURAL LEVEL ###

tmp <- grep(x = dataExtTime$year, pattern = "average")
Tmp <- dataExtTime[tmp,]                                          # extracts the average for the entire time series
tmp <- Tmp[,grep(x = names(Tmp),                                  # extracts the valued for the 'Control' scenario
   pattern = "^F4.contr$|^P0$|^P13$|^lambda.contr$")]
names(tmp) <- gsub(x=names(tmp), pattern = ".contr",
  replacement ="")
tmp;A.mean                                                        # the average matrix from our study area for the CONTROL (NEST PREDATION AT ITS NATURAL LEVEL): i.e., based on laying clutch size
(lambda.contr <- formatC(log(lambda(A.mean)),digits = 2))         # lambda: log(r)


   ### ALL YEARS - MANIPULATION: NO NEST PREDATION ###

tmp <- Tmp[,grep(x = names(Tmp),
   pattern = "^F4.NoPred$|^P0$|^P13$|^lambda.NoPred$")]
names(tmp) <- gsub(x=names(tmp), pattern = ".NoPred",
  replacement ="")
tmp;A.meanP                                                       # the average matrix from our study area for the MANIPULATION (NO NEST PREDATION): i.e., based on hatching clutch size
lambdaP <- tmp$lambda
(lambda.NoPred <- formatC(log(lambda(A.meanP)),digits = 2))


   ### EARLY - CONTROL: NEST PREDATION AT ITS NATURAL LEVEL ###

tmp <- grep(x = dataExtTime$year, pattern = "Early")
Tmp <- dataExtTime[tmp,]                                          # extracts the average for the early period
tmp <- Tmp[,grep(x = names(Tmp),
   pattern = "^F4.contr$|^P0$|^P13$|^lambda.contr$")]
names(tmp) <- gsub(x=names(tmp), pattern = ".contr",
  replacement ="")
A.meanEarly <- mean.list(A.2005_2010)
A.meanEarly[1,4] <- tmp$F                                         # the across average matrices valued for fecudity are not the same as in the table
tmp;A.meanEarly


   ### EARLY - MANIPULATION: NO NEST PREDATION ###

tmp <- Tmp[,grep(x = names(Tmp),
   pattern = "^F4.NoPred$|^P0$|^P13$|^lambda.NoPred$")]
names(tmp) <- gsub(x=names(tmp), pattern = ".NoPred",
  replacement ="")
A.meanEarlyP <- mean.list(A.2005_2010P)
A.meanEarlyP[1,4] <- tmp$F
tmp;A.meanEarlyP


   ### LATE - CONTROL: NEST PREDATION AT ITS NATURAL LEVEL ###

tmp <- grep(x = dataExtTime$year, pattern = "Late")
Tmp <- dataExtTime[tmp,]                                          # extracts the average for the late period
tmp <- Tmp[,grep(x = names(Tmp),
   pattern = "^F4.contr$|^P0$|^P13$|^lambda.contr$")]
names(tmp) <- gsub(x=names(tmp), pattern = ".contr",
  replacement ="")
A.meanLate <- A.meanEarlyP                                        # the quasi-exitionctions exluced the last year
A.meanLate[1,4] <- tmp$F;
A.meanLate[2,1] <- tmp$P0
A.meanLate[3,2] <- tmp$P13;A.meanLate[4,3] <- tmp$P13
A.meanLate[4,4] <- tmp$P13
tmp;A.meanLate


   ### LATE - MANIPULATION: NO NEST PREDATION ###

tmp <- Tmp[,grep(x = names(Tmp),
   pattern = "^F4.NoPred$|^P0$|^P13$|^lambda.NoPred$")]
names(tmp) <- gsub(x=names(tmp), pattern = ".NoPred",
  replacement ="")
A.meanLateP <- A.meanLate
A.meanLateP[1,4] <- tmp$F
tmp;A.meanLateP


   ### LITERATURE-CONTROL: NEST PREDATION AT ITS AVERAGE LEVEL ###

# NOTES: Apparent juvenile survival is based on the average estimate from our study area (as we could not find this from other studies)

LittEst$F <- (1 - (LittEst$EggPred/100)) *                        # fecundity with egg predation: the average clutch size (across studies) mulitplied by 0.5 (1:1 sex ratio),
  LittEst$ClutchS * tmp$P0 * 0.5                                  #   proportion of eggs not taken by predators and juvenile survival from our study area (as no other studies reported this)
A.Litt <- A.mean;                                                 # setting up matrices based on vital rates from the literature
A.Litt[3,2] <- A.Litt[4,4] <- A.Litt[4,3] <- LittEst$SurvAdu      # apparent adult survival from the literature
A.Litt[1,4]<- LittEst$F                                           # clutch size based on the literature and apparent juvenile survival from our study area
(lambda.Litt <- formatC(log(lambda(A.Litt)),digits = 2))          # population growth based on the matrix from the literature assuming nest predation
# NOTES: apparent juvenile suvival is the similar across the matrices


   ### LITERATURE-MANIPULATION: NO NEST PREDATION ###

LittEst$FP <- LittEst$ClutchS * tmp$P0 * 0.5                      # fecundity without egg predation: the average clutch size (across studies) mulitplied by 0.5 (1:1 sex ratio) and juvenile survival from our study area
A.LittP <- A.Litt;
A.LittP[1,4]<- LittEst$FP
(lambda.LittP <- formatC(log(lambda(A.LittP)),digits = 2))        # population growth based on the matrix from the literature assuming no nest predation








                                        #########################################################################################################
                                        # FIXED LTRE: CONTROL (NEST PREDATION AT ITS NATURAL LEVEL) FROM OUR STUDY AREA AS THE REFERENCE MATRIX #
                                        #########################################################################################################


   ### SETTING UP THE PLOT ###

dev.new(width = 16/2.53, height = 16/2.54)
par(cex.axis = 1.2, cex.lab = 1.8, cex.axis = 1.5,
  mar = c(7, 4.8, 1.3, .1))
layout(mat = matrix(c(1,1,1,2,3,4), nrow=2, byrow = T))


   ### SUM OF EFFECTS ACROSS (TREATMENT) MATRICES ###

A.treat <- list(A.meanP, A.Litt, A.LittP)                         # treatment matrix/matrices
Cm <- LTRE(trts = A.treat, ref = A.mean)                          # performs the fixed LITRE using the function from popbio
pe <- sapply(Cm, sum)                                             # the sums across the vital rates: similar to Angert (2005) and Horvitz et al. (1997)
tmp <- cbind.data.frame(pe,label = 1:length(pe),
  lab = 1:length(pe))

col <- c(colors()[c(131)],colors()[c(136)],colors()[c(578)])      # colours for the plot
label <- c(                                                       # labels for the plot
  as.expression(substitute("Horsvr"[italic(a)],
    list(a="No predation"))),
  as.expression(substitute("Literature"[italic(b)],
    list(b="Control"))),
  as.expression(substitute("Literature"[italic(c)],
    list(c="No predation"     ))))

lambdaLab <- c(                                                   # lambdas for the matrices to be added to the plots
  as.expression(substitute(italic(lambda)*"="*Lambda,
    list(Lambda = lambda.NoPred))),
  as.expression(substitute(italic(lambda)*"="*Lambda,
    list(Lambda = lambda.Litt))),
  as.expression(substitute(italic(lambda)*"="*Lambda,
    list(Lambda = lambda.LittP))))

mp <- barplot(tapply(tmp$pe, tmp$lab, mean),                      # plotting the sums taken from the examples from the popbio-package;
  xlab = "", ylab = "", col= col, ylim = c(min(pe)-.12,           #   (https://rdrr.io/cran/popbio/man/LTRE.html; assessed 20-FEB-2022)
    max(pe)+.02), xaxt = "n")
text(x=mp, y=(par("usr")[3]-.0075), label, srt = 25,              # adding labels to the X-axis
  xpd = T, col = col, cex = 1.5)
mtext(side = 3, expression(
  "a) Control: Nest predation (Reference)"),                      # labelling the plot
  line = -1, cex = 1.1)
text(x=mp, y=pe/2, lambdaLab, srt = 15, xpd = T,                  # adding the lambdas to the bars
  col = "white", cex = 1.5)
mtext(side = 2, expression(paste(sum(a[ij]), " contributions")),  # labelling the Y-axis
      cex = 1.3, line = 2.3)


   ### VITAL RATE CONTRIBUTIONS (SAME LTRE AS ABOVE) ###

label <- letters[2:4]                                             # creates labelling of the plots
Label <- c(                                                       # labels for the X-axes
  expression(italic(F["3+"])),
  expression(italic(P["0"])),
  expression(italic(P["1"])),
  expression(italic(P["2"])),
  expression(italic(P["3+"])))

par(mar = c(3, 4.8, 1.3, .1))
for (i in 1:length(col))                                          # loops over the different matrices in LTRE
  { pe <- Cm[[i]]                                                 # extracts the correct matrix
    tmp <- data.frame(x = c("F3", "P0", "P1", "P2", "P3"),        # data frame with vital rates (x) and their values (y)
      y = c(pe[1,4], pe[2,1],pe[3,2], pe[4,3],  pe[4,4]))
    mp <- barplot(tapply(tmp$y, tmp$x, mean),                     # plotting the contributions as a barplot
      xlab = "", ylab = "", col= col[i], xaxt = "n",
      ylim = c(min(sapply(Cm, min)), max(sapply(Cm, max))))
    mtext(at=mp, Label, srt = 25, xpd = T, cex = 1, side = 1,     # adding labels to the X-axis
      line = 2)
    if(i == 1)                                                    # for the first plot we add Y-axis labelling, but not for the remaining plots
      { mtext(expression(paste("Contribution to variation in ",
          lambda)),cex = 1.3, line = 2.9, side = 2)
        lambdaLab <- c(
          as.expression(substitute(italic(lambda["Reference"])*
             "="*Lambda, list(Lambda = formatC(log(lambda(
             A.mean)), digits = 3, width = 3, format = "f")))),
          as.expression("Control"))
        legend(x = "bottomright", legend = lambdaLab, adj =c(0),
          cex = 1.4, bty = "n")
      }
    mtext(side = 3, as.expression(substitute(value1*")",          # labels the plot
     list(value1 = label[i]))), cex = 1.1, line = 1, adj = 0,
     col = col[i])
  }


   ### STORING AND CLEANING ###

dev.copy(tiff, filename = "LTRE_CONTROL(reference).tiff",
  units="cm", width=16, height=16, res=800,
  compression = c("lzw"));dev.off()
rm(A.treat, lambdaLab, tmp, mp, Cm, pe, label)









                                        ##################################################################################
                                        # FIXED LTRE: CONTROL (NO PREDATION) FROM OUR STUDY AREA AS THE REFERENCE MATRIX #
                                        ##################################################################################


   ### SETTING UP THE PLOT ###

dev.new(width = 16/2.53, height = 16/2.54)
par(cex.axis = 1.2, cex.lab = 1.8, cex.axis = 1.5,
  mar = c(7, 4.8, 1.3, .1))
layout(mat = matrix(c(1,1,1,2,3,4), nrow=2, byrow = T))


   ### SUM OF EFFECTS ACROSS (TREATMENT) MATRICES ###

A.treat <- list(A.mean, A.Litt, A.LittP)
Cm <- LTRE(trts = A.treat, ref = A.meanP)
pe <- sapply(Cm, sum)
tmp <- cbind.data.frame(pe,label = 1:length(pe),
  lab = 1:length(pe))

label <- c(
  as.expression(substitute("Horsvr"[italic(a)],
    list(a="Control"))),
  as.expression(substitute("Literature"[italic(b)],
    list(b="Control"))),
  as.expression(substitute("Literature"[italic(c)],
    list(c="No predation"     ))))

lambdaLab <- c(
  as.expression(substitute(italic(lambda)*"="*Lambda,
    list(Lambda = lambda.contr))),
  as.expression(substitute(italic(lambda)*"="*Lambda,
    list(Lambda = lambda.Litt))),
  as.expression(substitute(italic(lambda)*"="*Lambda,
    list(Lambda = lambda.LittP))))

mp <- barplot(tapply(tmp$pe, tmp$lab, mean),
  xlab = "", ylab = "", col= col, ylim = c(min(pe),
    max(pe)+.02), xaxt = "n")
text(x=mp, y=(par("usr")[3]-.0175), label, srt = 25,
  xpd = T, col = col, cex = 1.5)
mtext(side = 3, expression(
  "a) Manipulation: No nest predation (Reference)"), line = -1, cex = 1.1)
text(x=mp[1], y=pe[1]/2, lambdaLab[1], srt = 15, xpd = T,         # adding the lambdas to the bars, but only the first column as the remaining is the same as in the previous plot
  col = "white", cex = 1.5)
mtext(side = 2, expression(paste(sum(a[ij]), " contributions")),
      cex = 1.3, line = 2.3)


   ### VITAL RATE CONTRIBUTIONS: PERIODS   ###

A.treat <- list(A.mean, A.meanEarly, A.meanLate)                  # the treatment matrix: the average with predation
A.refs <- list(A.meanP, A.meanEarlyP, A.meanLateP)                # the refernece matrices are those without predaton

label <- paste(letters[2:4],")",c(" All years",                   # creates labelling of the plots
  " 2005-10", " 2011-19"), sep ="")
par(mar = c(3, 4.8, 1.3, .1))
for (i in 1:length(label))                                        # loops over the different matrices in LTRE
  { Cm <- LTRE(trts = A.treat[[i]], ref = A.meanP)
    pe <- Cm
    tmp <- data.frame(x = c("F3", "P0", "P1", "P2", "P3"),        # data frame with vital rates (x) and their values (y)
      y = c(pe[1,4], pe[2,1],pe[3,2], pe[4,3],  pe[4,4]))
    mp <- barplot(tapply(tmp$y, tmp$x, mean),                     # plotting the contributions as a barplot
      xlab = "", ylab = "", col= "dark grey", xaxt = "n",
      ylim = c(-.15,0.05))
    mtext(at=mp, Label, srt = 25, xpd = T, cex = 1, side = 1,     # adding labels to the X-axis
      line = 2)
    lambdaLab <- c(
      as.expression(substitute(italic(lambda["Control"])*"="*
        Lambda, list(Lambda = formatC(log(lambda(A.treat[[i]])),
        digits = 3, width = 3, format = "f")))),
      as.expression(substitute(italic(lambda["No predation"])*"="*
        Lambda, list(Lambda = formatC(log(lambda(A.refs[[i]])),
        digits = 3, width = 3, format = "f")))))
    legend(x = "bottomright", legend = lambdaLab, adj = c(0),
       cex = 1.3, bty = "n")
    if(i == 1)                                                    # for the first plot we add Y-axis labelling, but not for the remaining plots
      { mtext(expression(paste("Contribution to variation in ",
          lambda)),cex = 1.3, line = 2.9, side = 2) }
    mtext(side = 3, as.expression(substitute(value1, list(        # labels the plot
      value1 = label[i]))), cex = 1.1, line = 1, adj = 0)
  }


   ### STORING AND CLEANING ###

dev.copy(tiff, filename = "LTRE_NoPredation(reference).tiff",
  units="cm", width=16, height=16, res=800,
  compression = c("lzw"));dev.off()
rm(A.treat, lambdaLab, A.refs, Cm, pe, label, col,
  Label, cex.axis, i)
rm(list=ls(pattern="lambda"));rm(list=ls(pattern="mp"))







                                        ########################
                                        # STORING SESSION INFO #
                                        ########################


sink("SessionInfo.txt")
print(sessionInfo(), locale = F)
sink()
