                                       ############################################################
                                      ####                                                      ####
                                      ####  FUNCTION FOR RUNNING THE LESLIE MATRIX (AGE-)       ####
                                      ####  MODEL:                                              ####
                                      ####                                                      ####
                                      ####  Lesser black-backed gull (Larus fuscus) on Horsvr, ####
                                      ####  Northern Norway                                     ####
                                      ####                                                      ####
                                      ####     Brd-Jrgen Brdsen (bjb@nina.no) & Jan Ove      ####
                                      ####     Bustnes, 31.03.2021                              ####
                                      ####                                                      ####
                                       ############################################################


# NOTE:  The object 'data' (input and output) must contain the following columns:
#    'Time'          -     time
#    'Class'         -     class (age)
#    'nObs'          -     number of animals
#    'Survival'      -     average survival
#    'Reproduction'  -     average reproduction (clutch size or chicks per nest/pair/female)
#    'P'             -     survival coefficients in the matrices
#    'F'             -     fertility coefficients in the matrices

# NOTE:  The object 'dataPop'(input and output) must contain the following columns:
#    'Time'          -     time
#    'Lambda'        -     population growth rate; 'Nt+1/Nt'
#    'Lambda2'       -     population growth rate considering the oldest age-class only (i.e., 'Lambda = N2t+1/N2t'; see 'N2' below)
#    'N'             -     population abundance
#    'N2'            -     population abundance excluding everything except the oldest age class
#    'year'          -     the year-specific empirically-based estimates (both for survival and recruitment) selected in a given time in the run
#    'SurvRank'      -     the year-specific empirically-based rank (for survival) in a given time in the run




                                        ################
                                        # MATRIX MODEL #
                                        ################

MatrixModel <- function(data,                                                 # class-level initial data (the name of the object will be equal to the name of the output data
                        dataPop,                                              # population-level initial data (the name of the object will be equal to the name of the output data
                        TimeLoop,                                             # number of years to simulate
                        maxN,                                                 # the maximum number of individuals allowed in the oldest age class
                        VitalAdult,                                           # data on vital rates for the adults, and must contain the following variables:
                                                                              #   year ('year'), survival ('survMean'), ranked survival ('Survrank') and recruitment ('recr')
                        VitalOffsp,                                           # data on vital rates for the offspring (identical as 'VitalAdult')
                        MaxMin, ProbShape )                                   # parameters going into the 'YearProb' function



{
  VitalOffsp <- data.frame(
     year        = VitalOffsp$year,
     SurvOff     = VitalOffsp$survMean,
     SurvOff.lcl = VitalOffsp$survLcl,
     SurvOff.ucl = VitalOffsp$survUcl)
  YearProbData <- data.frame(
     year        = VitalAdult$year,
     SurvRank    = VitalAdult$SurvRank,
     SurvAdu     = VitalAdult$survMean,
     SurvAdu.lcl = VitalAdult$survLcl,
     SurvAdu.ucl = VitalAdult$survUcl,
     Repr        = VitalAdult$recr,
     ProbYear    = YearProb(rank = VitalAdult$SurvRank, MaxMin = MaxMin,      # creates the given year's probability of occurring in the simulation to come
     ProbShape = ProbShape))
  YearProbData <- merge(VitalOffsp, YearProbData, all = T)                    # creates a common data frame with survival/reproduction (incl. 95% CIs) for offspring and adults
  row.names(YearProbData) <- 1:dim(YearProbData)[1]
  YearProbData$year <- as.numeric(YearProbData$year) +                        # re-defines this variable into a numeric variable
    as.numeric(levels(YearProbData$year)[1]) - 1
  YearProbData$SurvOff <- ifelse(is.na(YearProbData$SurvOff), 0,              # defines NAs for offspring survival as zeros
    YearProbData$SurvOff)
  tmp <- sample(x = row.names(YearProbData), replace = T,                     # creates an index used for extracting the data
    size = TimeLoop, prob = YearProbData$ProbYear)
  EnviroData <- YearProbData[as.numeric(tmp),]                                # creates a randomized environmental data set where each year's occrrence is defined by 'ProbYear'
  EnviroData$Time <- (data$Time[1]+1):TimeLoop                                # adding 'Time' to the environmental data set
  row.names(EnviroData) <- 1:dim(EnviroData)[1]
  tmp <- as.data.frame(as.table(tapply(EnviroData$Time, EnviroData$year,
    length)));names(tmp) <- c("year", "ProbYearObs")
  tmp$ProbYearObs <- tmp$ProbYearObs/TimeLoop
  YearProbData <- merge(YearProbData, tmp, all = T)                           # creates a data frame including both the given and the observed proportions for each year 'ProbYearObs'


  { for (t in (data$Time[1]+1):TimeLoop)                                      # simulation start one year ahead of 'Time' in 'data'


      ### 1. SUBSETTING THE DATA (year [t] = t-1) ###

    { tmp <- subset(data, Time == t-1)                                        # takes a subset of the age class data
      tmp2 <- subset(dataPop, Time == t-1)                                    # takes a subset of the population data
      tmp4 <- subset(EnviroData, Time == t)                                   # takes a subset of the climate data (for 't = t')
      tmp$Time <- rep(t, dim(tmp)[1])
      tmp2$Time <- t


      ### 2. CREATED THE PROJECTION MATRIX (i.e., 't-1') AND STORES IT IN 'A' ###

      a <- matrix(rep(0, (dim(tmp)[1])^2), nrow = dim(tmp)[1],                # constructing the projection matrix for a given year ('a')
        byrow = TRUE, dimnames = list(c(tmp$Class), c(tmp$Class)))
      a[1,] <- c(tmp$F)                                                       # setting up the fecundities ('F') in the top row
      for(i in 2:(dim(a)[1]) )                                                # setting up the survival rates ('P' or 'S')
        { a[(i),(i-1)] <- tmp$P[(i-1)] }
      a[dim(a)[1],dim(a)[1]] <- tmp$P[length(tmp$P)]
      { if (t == 1)
          A <- c( list(a) )                                                   # list of matrices consisting of the projection matrices for each year ('a')

        else if (t > 1)
          A <- c(A, list(a))
      }


      ### 3. PROJECTING POPULATION SIZE FROM LAST YEAR (i.e., 't-1') TO THIS YEAR (i.e., at 't') ###

      p <- pop.projection(A = a, n = tmp$nObs, iterations = 2)                # projecting population size one year ahead (i.e. at 't+1') using the popbio's pop.projection-fucntion
      tmp$nObs <- p$stage.vectors[,2]                                         # population size (for the different classes) one at 't+1'
      tmp$nObs[length(tmp$nObs)] <- ifelse(tmp$nObs[length(tmp$nObs)] > maxN, # the maximum number of ind. in the the oldeest age class is limited by 'maxN'
        maxN, tmp$nObs[length(tmp$nObs)])
      dataPop$Lambda[dataPop$Time == t-1] <- p$lambda                         # extracts lambda defined as Nt+1/Nt


      ### 4. UPDATES SURVIVAL AND REPRODUCTION WITHIN THE CURRENT YEAR (i.e., at 't') ###

      tmp$Survival <- c(                                                      # survival
        tmp4$SurvOff,                                                         # offspring survival
        rep(tmp4$SurvAdu, dim(tmp)[1]-1))                                     # survival for the remaining age-classes

      tmp$Reproduction <-c(                                                   # reproduction
        rep(0, dim(tmp)[1]-1),                                                # offspring do not produce offspring, and neither does the age-classes in-between
        tmp4$Repr)                                                            # reproduction for the adults

      tmp$P <- tmp$Survival                                                   # the survival coefficients ('P') is the probability that an individual in age class 'i' survives
      tmp$F <- tmp$Reproduction*                                              # the fertility coefficients ('F') is calculated by multiplying the numbers of eggs/chicks with their
        tmp$Survival[1]                                                       #   survival probabilities (i.e. how many are produced at 't', survives from 't' to 't+1' and then recruited
                                                                              #   into the population.
      tmp$F <- tmp$F*.5                                                       # we assume a 50:50 sex ratio


      ### 5. WE UPDATE POPULATION SIZES AFTER CALCULATING SURVIVAL AND REPRODUCTION WITHIN THE CURRENT YEAR (i.e., at 't') ###

      tmp2$N <- sum(tmp$nObs)                                                 # population abundance after predation has taken its toll
      tmp2$N2 <- tmp$nObs[length(tmp$nObs)]                                   # abundance of individuals in the oldest age class
      dataPop$Lambda2[dataPop$Time == t-1] <- tmp2$N2/                        # extracts lambda defined as Nt+1/Nt
        dataPop$N2[dataPop$Time == t-1]
      tmp2$year <- tmp4$year
      tmp2$SurvRank <- tmp4$SurvRank


      ### 6. ADDING THE DATA ###

      data <- rbind.data.frame(data, tmp)
      dataPop <- rbind.data.frame(dataPop, tmp2)

    }
    { row.names(data) <- 1:dim(data)[1];                                      # gives the data sets more intuitive row names
         row.names(dataPop) <- 1:dim(dataPop)[1]

      { if ( is.na(dataPop$N[dim(dataPop)[1]]) == FALSE )                     # if there are surviving populations these objects will be returned
        { return( list( data       = data,
                    dataPop        = dataPop,
                    A              = A,
                    MaxMin         = MaxMin,
                    ProbShape      = ProbShape,
                    EnviroData     = EnviroData,
                    YearProbData   = YearProbData) )
        } else                                                                # if all populations went extinct these objects will be returned
        { return( list( data       = data,
                    dataPop        = dataPop,
                    EnviroData     = EnviroData,
                    MaxMin         = MaxMin,
                    ProbShape      = ProbShape,
                    YearProbData   = YearProbData) )
        }
      }
    }
  }
}
