                                       ############################################################
                                      ####                                                      ####
                                      ####  FUNCTION FOR CALCULATING PARTIAL R-SQUARES,         ####
                                      ####  COEFFICIENTS SENSITITIVY TO VARAIBLE DROP-OUTS:     ####
                                      ####                                                      ####
                                      ####  Lesser black-backed gull (Larus fuscus) on Horsvr, ####
                                      ####  Northern Norway                                     ####
                                      ####                                                      ####
                                      ####     Brd-Jrgen Brdsen (bjb@nina.no) & Jan Ove      ####
                                      ####     Bustnes, 17.02.2022                              ####
                                      ####                                                      ####
                                       ############################################################


# NOTES: Partial R-square (returned as 'PartialData') is calculated as follows:
#        'partial R2 = ( R2full-R2i ) / ( 1- R2i )',
#        where 'R2full' is the R-square for the original model (i.e., the full model) and
#        'R2i' is the R-square for a model where variable 'i' is dropped out. This is done
#        similarly when partial adjusted R-squares are calculated

# NOTES: Similarly, the function also assessed parameter sensitivity (returned as 'CoefData'):
#        for each time a variable is dropped out (see above) we re-fit the model using the
#        remaining variables.


partialR2 <- function(object)
  {                                                                                     # 'object' needs to be an lm-model
    modSum <- summary(object)
    R.full <- modSum$r.squared                                                          # R-square for the original model
    R.adj.full <- modSum$adj.r.squared                                                  # adjusted R-square for the original model
    data <- object$mod                                                                  # data in which the original model is fitted to
    names(data) <- c("y", names(data)[-1])
    variable <- c("all")                                                                # variables being considered (see below)
    Part.R.sq <- c(R.full)                                                              # partial R-square is the R-square for the original model when all predictors are included
    Part.R.adj.sq <- c(R.adj.full)                                                      # same with adjusted R-squares
    Coef.data <- data.frame(VarExluded = rep("none", length(coef(object))),             # this function also checks for potential confounding
                            Parameter = names(coef(object)))
    Coef.data <- cbind.data.frame(Coef.data, as.data.frame(summary(object)$coef))

    for( i in row.names(anova(object))[-length(row.names(anova(object)))] )             # loops over the variables included in the model
      { variable <- c(variable, i)
        variables <- row.names(anova(object))[-length(row.names(anova(object)))]
        variables <- variables[variables!=i]
        model1.form.text <- paste("y ~",paste(variables,collapse=" + "),collapse=" ")
        model1.form <- as.formula( model1.form.text )
        Mod <- lm( model1.form, data=data )
        Part.R.sq <- c( Part.R.sq,
           ( (R.full-summary(Mod)$r.squared)/(1-summary(Mod)$r.squared)) )              # partial R-square values for variable 'i'
        Part.R.adj.sq <- c( Part.R.adj.sq,
           ( (R.adj.full-summary(Mod)$adj.r.squared)/(1-summary(Mod)$adj.r.squared)) )  # partial R-square (adjusted) values for variable 'i'
        tmp1 <- data.frame(VarExluded = rep(i, length(coef(Mod))),
                            Parameter = names(coef(Mod)))
        tmp1 <- cbind.data.frame(tmp1, as.data.frame(summary(Mod)$coef))
        Coef.data <- rbind.data.frame(Coef.data, tmp1)
      }
    tmp <- cbind.data.frame(variable, Part.R.sq, Part.R.adj.sq)
    row.names(Coef.data) <- 1:length(Coef.data[,1])
    return ( list (PartialData = tmp,                                                   # data on partial R-squares for the different variables
                   CoefData = Coef.data) )                                              # data on the coefficients for models where one variable is dropped at a time (confounding)
  }


# EXAMPLE #
#x <- seq(1,10, length.out = 100) + rnorm(mean= 0, sd = 5, n = 100)                     # simulated data
#factor <- as.factor(c(rep("Treat",50),rep("Contr",50)))
#x2 <- seq(10, 1, length.out = 100) + rnorm(mean= 0, sd = 1, n = 100)
#y <- x - x2 + rnorm(mean= 0, sd = 4, n = 100)

#mod <- lm(y~1+x+x2)                                                                    # fits a model to the data
#partialR2(mod)

#mod <- lm(y~x*x2*factor)
#partialMod <- partialR2(mod)                                                           # stores the output from 'partialR2'
#partialMod$PartialData                                                                 # printing information on the partial R-squares only
#partialMod$CoefData                                                                    # printing information on the variable sensitivity only
#subset(partialMod$CoefData, Parameter == "x2")                                         # printing information on the variable sensitivity only (for one varaible)
#rm(x, factor, x2, y, mod, partialMod)

