                                      ############################################################
                                      ####                                                      ####
                                      ####  CLUTCH SIZE AND NEST PREDATION:                     ####
                                      ####                                                      ####
                                      ####  Lesser black-backed gull (Larus fuscus fuscus)      ####
                                      ####  on Horsvr, Northern Norway                         ####
                                      ####                                                      ####
                                      ####     Brd-Jrgen Brdsen (bjb@nina.no) & Jan Ove      ####
                                      ####     Bustnes, 22.02.2022                              ####
                                      ####                                                      ####
                                       ############################################################








                                        ################################
                                        # CHANGING OPTIONS & LIBRARIES #
                                        ################################


# NOTES: For this script to function (i.e., for the required input files to load and to store the output in the correct directory), you must ensure that:
#       1) you keep the same structure (directories) as in the original .zip-file;
#       2) you define the current working directory of this R-session to be the same directory as the location of the script file (run 'getwd()' the check this, and use 'setwd(path to
#          this directory on your computer)' to fix it.

install.packages(
  c("openxlsx","AICcmodavg", "nlme", "piecewiseSEM"),             # getting the required packaged (from the CRAN sever in Bergen, Norway)
    repos = "http://cran.ii.uib.no/" )                            #   another server can be selected by changing repos; or by running: 'install.packages(c("openxlsx","AICcmodavg", ...))'
library(openxlsx)                                                 # loads the package 'openxlsx: Read, Write and Edit xlsx Files'
library(AICcmodavg)                                               # loads the package 'AICcmodavg: model selection and multimodel inference based on (Q)AIC(c)'
library(nlme)                                                     # loads the package 'nlme: linear and nonlinear mixed effects models'
library(piecewiseSEM)                                             # loads the package 'piecewiseSEM: Piecewise structural equation modeling in R for ecology, evolution, and systematics'

n <- function(x) { sum(ifelse(is.na(x), 0, 1)) }                  # function to count the number of observations
quant2 <- function(x) { quantile(x, p = 0.25, na.rm =T) }         # function to get the 25th percentile
quant4 <- function(x) { quantile(x, p = 0.75, na.rm =T) }         # function to get the 75th percentile
startDate <- 1                                                    # first of June: "Sildemka legger som regel 2-3 egg i mai og juni, som ruges i omtrent
                                                                  #   24-26 dager. Ungene er flygedyktige etter omtrent 40 dager"
                                                                  #   (https://snl.no/sildem%C3%A5ke assessed 27-05-2020)
DaysIncubation <- 24                                              # number of days to incubate (https://snl.no/sildem%C3%A5ke assessed 27-05-2020)
DaysBetweenCheck <- 2                                             # the minimum days needed in-between checks in order to be included in the analyses
yearExlude <- c(2012:2013,2016)                                   # years to exclude due to breeding collapses (see Appendix S2:Table S5 for details)



wd <- getwd()                                                     # get current working directory
Tmp <- unlist(strsplit(wd,"/"))                                   # splits the current working directory into its sub-directories
Tmp <- Tmp[-length(Tmp)]                                          # removes that last one
Tmp <- paste(Tmp,collapse="/")                                    # puts the path together again (i.e., the same as moving to the parent directory from the current working directory)

dir.create("ReproPredat", showWarnings = F)                       # creates a sub-directory to store output (if not already created))
setwd("ReproPredat")                                              # defines this as the current working directory (everything saved will be stored there automatically)






                                        ###############
                                        # DATA IMPORT #
                                        ###############


   ### EMPIRICAL NEST-LEVEL DATA ON DATE AND CLUTCH SIZE: Data ###

Data <- read.xlsx(                                                # imports the data (see 'Data.xlxs' in Data/Reproduction for metadata)
    xlsxFile = paste(Tmp,"/Data/Reproduction/Data.xlsx",
      sep=""),
    sheet = "Data")
data <- Data                                                      # data to be used in the clutch size analyses below (no subset will be taken)


   ### EXCLUDING THE NESTS WITH LESS THAN TWO DAYS BETWEEN VISITS ###

dim(Data)
Data <- subset(Data, DateDiff > DaysBetweenCheck)                 # removes all rows with too few days in-between visits as defined by 'DaysBetweenCheck'
dim(Data)
attach(Data, warn.conflicts = F)


   ### AGGREGATED DATA ON DAILY PREDATION RATES: NEWCOLONY AND YEAR ###

dataNewColYea <- as.data.frame(as.table(tapply(
  EggPredDay, list(Year, NewColony), mean, na.rm = T)))
names(dataNewColYea) <- c("Year", "NewColony", "EggPredDay.mean") # mean egg predation rate per day
dataNewColYea$EggPredDay.sd <- as.data.frame(as.table(tapply(     # SD for egg predation rate per day
  EggPredDay, list(Year, NewColony), sd, na.rm = T)))[,3]
dataNewColYea$EggPredDay.n <- as.data.frame(as.table(tapply(      # sample size (n) for egg predation rate per day
  EggPredDay, list(Year, NewColony), n)))[,3]
dataNewColYea$EggPredDay.se <- dataNewColYea$EggPredDay.sd/       # SE egg predation rate per day
  sqrt(dataNewColYea$EggPredDay.n)
dataNewColYea$DateDiff.mean <- as.data.frame(as.table(tapply(     # mean number of days in-between visits
  DateDiff, list(Year, NewColony), mean, na.rm = T)))[,3]







                                        #######################################################################
                                        # ANNUAL DAILY NEST PREDATION RATES BASED ON INDIVIDUAL NESTS: 'Data' #
                                        #######################################################################

   ### DATA ORGANIZATION ###

graphics.off()
tmp <- subset(Data, NewColony != "NoGood" &                       # excludes two colonies
  NewColony != "D", select = c("NewColony", "Year", "EggPredDay",
  "DateDiff" ))

tmp$response <- tmp$EggPredDay                                    # defines the response variable
dim(tmp)
tmp$year <- as.character(tmp$Year)                                # creates a factor variable from year
for(i in yearExlude)                                              # removes years that are excluded
  { tmp <- subset(tmp, year != as.character(i) ) }
dim(tmp)

tmp$Year.sc <- scale(tmp$Year, scale = F)[1:dim(tmp)[1]]          # centring year
tmp$YearSq.sc <- tmp$Year.sc^2                                    # second-order polynomial for year.sc
tmp$BinResponse <- ifelse(tmp$response < 0, 1, tmp$response)      # creates a binary response: zero (no eggs disappeared) 1 (at least one egg removed)
tmp <- na.omit(tmp)                                               # removed NAs
dim(tmp)


   ### DEFINING THE BASELINE CANDIDATE MODEL WITH (LME) AND WITHOUT (GLS) RANDOM EFFECTS ###

LME <- lme(response ~ Year.sc + YearSq.sc,
  random =~1|NewColony, tmp, method = "REML", na.action=na.omit)
GLS <- gls(response ~ Year.sc + YearSq.sc,
  tmp, method = "REML",na.action=na.omit)

dev.new(width = 16/2.54, height = 25/2.54)
par(mfrow=c(5,2), mar=c(3,2,2,.5))                                # checking the residuals for violations against the homoscedastic assumption
plot(fitted(GLS),resid(GLS, type = "p"),xlab="", ylab = "")
abline(h = 0)
title(outer = F, line = .9, "GLS - residuals vs. fitted")
plot(fitted(LME),resid(LME, type = "p"),xlab="", ylab = "")
abline(h = 0)
title(outer = F, line = .9, "LME - residuals vs. fitted")


  ### DEFINING AN APPROPRIATE VARIANCE FUNCTION TO TAKE INTO ACCOUNT HETEROGENEITY IN THE RESIDUALS (FITTING HETEROSCEDASTIC MODELS) ###

# NOTE: 'varFunc' Classes [see Pinheiro & Bates (2000: 208-225 (see table 5.1 for an overview) & Zuur et al. (2009: ch. 4)]:
# 'varFunc' are used to specify within-group variance models. The varFunc constructors have the same names as their
# corresponding classes. Two main arguments are required: (i) value and (ii) form. Value specifies the value of
# the variance parameters whereas form is a one sided formula specifying the variance covariate and if necessarily a
# stratification variable for the variance parameters.

GLS.varFixed <- update(GLS, weights = varFixed(value = ~DateDiff) )
LME.varFixed <- update(LME, weights = varFixed(value = ~DateDiff) )
# NOTE: A variance function with no parameters and a single variance covariate (variance is known up to a proportional
# constant). This VarFunc takes a single argument (i.e., the value of the variance parameters). In this case the
# within-group variance is expected to be linearly related to the predictor
    plot(fitted(GLS.varFixed),resid(GLS.varFixed, type = "p"),xlab="", ylab = "")
    abline(h = 0)
    title(outer = F, line = .9, "GLS: varFixed")
    plot(fitted(LME.varFixed),resid(LME.varFixed, type = "p"),xlab="", ylab = "")
    abline(h = 0)
    title(outer = F, line = .9, "LME: varFixed")

GLS.varPower <- update(GLS, weights =
  varPower(form = ~ fitted(.)), control=(msMaxIter=2000))
#LME.varPower <- update(LME, weights =
#   varPower(form = ~ fitted(.)), control=c(msMaxIter=10000, maxIter=5000))
# NOTE: A variance function that is the power of the absolute value of the variance covariate (the power of 0 is always
# 0 so this variance function cannot be used with variance covariates assumed to be zero). The variance parameters are
# unrestricted so this varFunc can model cases when the variance increase or decrease with the absolute value of the
# variance covariate.
    plot(fitted(GLS.varPower),resid(GLS.varPower, type = "p"),xlab="", ylab = "")
    abline(h = 0)
    title(outer = F, line = 0.9, "GLS: varPower")
    plot(1,1, xlab = "", ylab = "", xaxt="n", yaxt="n", type = "n")
    text(1,1, "Convergence issues")
    abline(h = 0)
    title(outer = F, line = 0.9, "LME: varPower")

GLS.varExp <- update(GLS, weights =
  varExp(form = ~ fitted(.)), control=(msMaxIter=2000))
LME.varExp <- update(LME, weights =
  varExp(form = ~ fitted(.)))#fitted(.)))
# NOTE: A variance function that allows the variance to increase exponentially with the variance covariate.
    plot(fitted(GLS.varExp),resid(GLS.varExp, type = "p"));abline(h = 0)
    title(outer = F, line = 0.9, "GLS: varExp")
    plot(fitted(LME.varExp),resid(LME.varExp, type = "p"),xlab="", ylab = "")
    abline(h = 0)
    title(outer = F, line = 0.9, "LME: varExp")

GLS.varConstPower <- update(GLS, weights =
  varConstPower( form = ~fitted(.)) )
LME.varConstPower <- update(LME, weights =
  varConstPower( form = ~fitted(.)) )
# NOTE: A variance function that is a constant plus the power of the absolute value of the variance covariate.
    plot(fitted(GLS.varConstPower),resid(GLS.varConstPower),xlab="", ylab = "")
    abline(h = 0)
    title(outer = F, line = .9, "GLS: varConstPower")
    plot(fitted(LME.varConstPower),resid(LME.varConstPower),xlab="", ylab = "")
    abline(h = 0)
    title(outer = F, line = .9, "LME: varConstPower")

dev.copy(tiff, filename = "EggPredDay_Hetero.tiff",
  units="cm",                                                     # saves the plot externally
  width=16,height=25,res=600, compression = c("lzw"));dev.off()


   ### SELECTING THE MOST PARSIMONIOUS MODEL: RANDOM EFFECTS AND VARIANCE STRUCTURE ###

(summary(LME.varConstPower)$modelStruct$varStruct)                # shows the estimates for the variance structure for the LME
(summary(GLS.varConstPower)$modelStruct$varStruct)                # shows the estimates for the variance structure for the GLS
# NOTE: the constant for these are ~0 so we exclude the .varConstPower models from the set of candidate models. The LME.varPower
#       model did not converge so that is excluded as well.

AICdata.LMEGLS <- data.frame(
  mod =                                                           # model names
  c("LME",                    "GLS",
    "GLS.varFixed",           "LME.varFixed",
    "GLS.varPower",          #"LME.varPower",
    "GLS.varExp",             "LME.varExp"),
  AICc =                                                          # extracts AICc-values
   c(AICc(LME),               AICc(GLS),
    AICc(GLS.varFixed),       AICc(LME.varFixed),
    AICc(GLS.varPower),      #AICc(LME.varPower),
    AICc(GLS.varExp),         AICc(LME.varExp))
)
AICdata.LMEGLS$df <- AIC(                                         # extracts the degrees of freedom
    LME,                      GLS,
    GLS.varFixed,             LME.varFixed,
    GLS.varPower,            #LME.varPower,
    GLS.varExp,               LME.varExp)[,1]
AICdata.LMEGLS$Delta_AICc <- AICdata.LMEGLS$AICc -                # delta AICc-values
  min(AICdata.LMEGLS$AICc)
AICdata.LMEGLS$mod <- as.character(AICdata.LMEGLS$mod)
AICdata.LMEGLS$Aikaike.weights <- exp(-.5*                        # Akaike's weights
  AICdata.LMEGLS$Delta_AICc)
AICdata.LMEGLS$Aikaike.weights <- AICdata.LMEGLS$Aikaike.weights/
  sum(AICdata.LMEGLS$Aikaike.weights)

print(AICdata.LMEGLS)                                            # printing the model selection table
tmp2 <- as.data.frame(subset(AICdata.LMEGLS, Delta_AICc <= 1.5)) # subset of all models with equal or less than a delta AICc of 1.5
tmp2 <- as.character(tmp2$mod[tmp2$df == min(tmp2$df)][1])
mod1 <- get(as.character(tmp2))                                  # selects a 'winning' model and uses that for inference


   ### DEFINING CANDITATE MODELS WITH DIFFERENT FIXED EFFECTS ###

mod1 <- update(mod1, method = "ML")                              # re-fitting the model using ML as we want to select models with different 'fixed effects'
mod2 <- update(mod1, ~.-YearSq.sc)

# NOTE: candidate models (fixed effects)
# i      Year  YearSq
# mod1   X     X
# mod2   X


   ### SELECTING THE MOST PARSIMONIOUS MODEL: FIXED EFFECTS ###

Cand.models <- list()                                            # makes a list of the candidate models (to be used in the 'aictab' function in 'AICcmodavg' package)
Modnames <- c()                                                  # makes a vector of the names of the candidate models (to be used in the 'aictab' function in 'AICcmodavg' package)
 for (j in 1:(length(ls(pattern = "mod"))-1))                    # loop to create a list of models and a vector of model names (to be used in the 'aictab' function
  { Cand.models[[j]] <- get(paste("mod",j, sep = ""))            #  in 'AICcmodavg' package)
    Modnames <- c(Modnames, paste("mod",j, sep = "") )
  }
AICdata <- aictab(cand.set = Cand.models, modnames =             # calculates delta AICc-values for the different candidate models
   Modnames, second.ord = T, sort = T)
AICdata                                                          # printing the output of the model selection assessment
tmp2 <- as.data.frame(subset(AICdata, Delta_AICc <= 1.5))        # creates a list of models with a difference in AICc less than or equal to 1.5
tmp2 <- as.character(tmp2$Modnames[tmp2$K == min(tmp2$K)][1])
Mod <- get(as.character(tmp2))                                   # selects a single model and uses that for inference
Mod <- update(Mod, method = "REML")                              # re-fits the model using REML


   ### MODEL DIAGNOSTICS (SELECTED MODEL) ###

dev.new();par(mfrow = c(2,2))                                    # plotting the results
plot(resid(Mod) ~ fitted(Mod),                                   # model diagnostics: Fitted vs. Residuals plots are used to assess whether the residuals can be
   main = "Residuals vs. Fitted (Mod)", xlab = "Fitted values",  # assumed independent of the fitted/predicted values (the correlation between them should be zero)
   ylab = "Residuals")
   abline (h = 0, col = "blue")
   lines(col = "red", smooth.spline(fitted(Mod),                 # adds a smoothen relationship (df denotes the equivalent number of degrees of freedom)
     resid(Mod), df = 3))
qqnorm(resid(Mod, type = "p"));qqline(resid(Mod, type = "p"))    # model diagnostics: QQ-plots are used to the extent in which data deviates from normality
plot(sqrt(abs(resid(Mod, type = "pearson"))) ~                   # model diagnostics: Scale-location plot (i.e., the square root of the absolute value of the residuals)
   fitted(Mod),                                                  # this is used in order to diminish any potential skewness in the data
   main = "Scale-Location", xlab = "Fitted values",
   ylab = expression(sqrt("Standardized residuals")))
   lines(col = "red", smooth.spline(fitted(Mod),                 # adds a smoothen relationship
     sqrt(abs(resid(Mod, type = "pearson"))), df = 3))
mtext(side = 3, adj = .2, "EggPredDay", cex = 1,
  line = -1, outer = T)                                          # plotting model diagnostics
plot(resid(GLS) ~ fitted(GLS),                                   # model diagnostics: Fitted vs. Residuals plots are used to assess whether the residuals can be
   main = "Residuals vs. Fitted (GLS)", xlab = "Fitted values",  # assumed independent of the fitted/predicted values (the correlation between them should be zero)
   ylab = "Residuals")
   abline (h = 0, col = "blue")
   lines(col = "red", smooth.spline(fitted(GLS),                 # adds a smoothen relationship (df denotes the equivalent number of degrees of freedom)
     resid(GLS), df = 3))
dev.copy(tiff, filename = "EggPredDay_ModDiag.tiff", units="cm",
  width=20, height=20, res=600, compression = c("lzw") )
dev.off()


   ### MODEL SUMMARY 'BIOLOGICAL' AND 'STATISTICAL' SIGNIFICANCE ###

anova(Mod)                                                       # shows the ANOVA table
summary(Mod)$tTable                                              # model summary
(r.sq <- as.numeric(rsquared(Mod)[5]))                           # conditional R2, which describes the proportion of variance explained by both the fixed and random factors (https://jonlefcheck.net/2013/03/13/r2-for-linear-mixed-effects-models/; assessed 2020-05-04)


   ### STORING RESULTS EXTERNALLY ###

sink("EggPredDay_output.txt")                                    # stores results externally
print("-----------------------------------------------")
print("     MODEL SELECTION: RANDOM EFFECTS & varFunc ")
print("-----------------------------------------------")
print(AICdata.LMEGLS);
print("");print("")
print("-----------------------------------------------")
print("     MODEL SELECTION: FIXED EFFECTS            ")
print("-----------------------------------------------")
print(aictab(cand.set = Cand.models, modnames =  Modnames,
  second.ord = T, sort = F), digits = 4);
print(paste("MODEL SELECTED =",as.character(tmp2)));
print("");print("")
print("-----------------------------------------------")
print("     MODEL SUMMARY                             ")
print("-----------------------------------------------")
print(summary(Mod));print("");print("")
print(paste("R.sq (cor(fitted(Mod), tmp$response)^2) =",
  round(cor(fitted(Mod), tmp$response)^2,4)))
print(paste("R.sq (r.squared[piecewiseSEM]) =",
  round(r.sq,4)));print("")
print("ANOVA TABLE")
print(anova(Mod));print("");print("")
print("-----------------------------------------------")
print("     95% CI                                    ")
print("-----------------------------------------------")
print(intervals(Mod));print("");print("")
sink()


   ### RE-FITTING A MODEL WITHOUT ANY CENTERED PREDICTOR(S) ###

{ if(dim(summary(Mod)$tTable)[1]==2)                             # the model used for the plot is based on non-centred data and what fixed effects that was chosen
    { GLM.plot <- update(Mod, ~Year) }                           # only the main effect of year
  else if(dim(summary(Mod)$tTable)[1]==3)
    { GLM.plot <- update(Mod, ~Year + I(Year^2),                 # the second-order polynomial of year
       control=(msMaxIter=2000)) }
}


   ### CREATING THE DATA TO PLOT FROM (BOTH THE PREDICTIONS AND THE AGGREGATED DATA AT THE COLONY- AND YEAR-LEVEL) ###

dataGLM <- data.frame(                                           # data on the observations (i.e., aggregated averages across 'NewColony' and 'year'
  prob = dataNewColYea$EggPredDay.mean,
  TotalNo = dataNewColYea$EggPredDay.n,
  Err = dataNewColYea$EggPredDay.se,
  NewColony = dataNewColYea$NewColony,
  Year = dataNewColYea$Year)
dataGLM <- na.omit(dataGLM)
newdata <- data.frame(Year = seq(min(tmp$Year),                  # creates a data frame used to make predictions
  max(tmp$Year), length.out = 100))
newdata$fit <- predict(GLM.plot, newdata = newdata)
newdata$se.fit1 <- newdata$fit-predictSE(GLM.plot,
  newdata = newdata, se.fit = T)$se.fit
newdata$se.fit2 <- newdata$fit+predictSE(GLM.plot,
  newdata = newdata, se.fit = T)$se.fit


   ### SETTING-UP THE PLOT ###

dev.new(height = 14/2.54, width = 16/2.54);
Col <- colorRampPalette(c("firebrick3",                          # colour palette used to indicate temporal trends
 "dodgerblue3"))(length(levels(as.factor(Data$Year))))
dataGLM$cex <- ((((dataGLM$TotalNo)-(min(dataGLM$TotalNo))+1)/
  (length((dataGLM$TotalNo))))+1)*1.2
dataGLM$year <- jitter(as.numeric(as.character(dataGLM$Year)),
  amount =.25)
dataGLM$err1 <- ifelse(dataGLM$prob+dataGLM$Err >= 0, 0,
  dataGLM$prob+dataGLM$Err)
dataGLM$err2 <- ifelse(dataGLM$prob-dataGLM$Err >= 0, 0,
  dataGLM$prob-dataGLM$Err)
par(cex.axis = 1.2, cex.lab = 1.5, mfrow = c(1,1),
  mar = c(4.5, 4.7, 1.5, 1), bty = "l")
plot(prob ~ year, data = dataGLM,                                # plots the results: proportion of nests being at least partial predated
  type ="n", main = "",
     xlab = expression("Time"*" (year)"),
     ylab = expression("Number of eggs lost"~day^-1%+-%" SE"),
     ylim = range(c(dataGLM$err2, dataGLM$err1)))


   ### PLOTTING THE DATA ###

for(i in 1:length(levels(as.factor(dataGLM$Year))))
  { tmp33 <- subset(dataGLM,
      Year == levels(as.factor(dataGLM$Year))[i])
    points(tmp33$prob ~ tmp33$year,                              # adds points (where size and colours indicate years
      col = "white", bg = Col[i], pch = 21, lwd = 1,
      cex = tmp33$cex)
    for(j in 1:length(tmp33[,1]))
      { lines(x = rep(tmp33$year[j],2),
          y = c(tmp33$err1[j], tmp33$err2[j]),
          col = Col[i])
      }
  }


   ### IDENTYFYING THE EXCLUDED YEARS ###

tmp33 <- as.character(yearExlude)                                # marks the excluded years (defined by 'yearExlude')
tmp33 <- subset(dataGLM, Year%in%tmp33)
points(tmp33$year-.2, tmp33$prob,col = "black",pch = "(",
  cex = 1.25)
points(tmp33$year+.2, tmp33$prob, col = "black", pch = ")",
 cex = 1.25)


   ### ADDING MODEL PREDICTIONS ###

lines(newdata$fit~newdata$Year, lwd = 3)                         # predictions from the model (using the average for the other predictor)
lines(newdata$se.fit1~newdata$Year, lwd = 1, lty = 2)
lines(newdata$se.fit2~newdata$Year, lwd = 1, lty = 2)
a <- formatC(summary(Mod)$tTable[1,1],format = "f", digits = 3)  # extracts the estimated intercept
a.se <- ifelse(
   round(summary(Mod)$tTable[1,2],3) == 0,                       # extracts its estimated SE
   "<0.001", formatC(summary(Mod)$tTable[1,2],
   format = "f", digits = 3))
b <- formatC(summary(Mod)$tTable[2,1],format = "f", digits = 3)  # extracts the estimated slope for year
b.se <- ifelse(
   round(summary(Mod)$tTable[2,2],3) == 0,                       # extracts its estimated SE
   "<0.001", formatC(summary(Mod)$tTable[2,2],
   format = "f", digits = 3))
c <- formatC(summary(Mod)$tTable[3,1],format = "f", digits = 3)  # extracts the estimated slope for the second-order polynomial for year
c.se <- ifelse(
   round(summary(Mod)$tTable[3,2],3) == 0,                       # extracts its estimated SE
   "<0.001", formatC(summary(Mod)$tTable[3,2],
   format = "f", digits = 3))
if(as.numeric(a) > 0)                                            # adds a plus sign for the positive effects
  { a <- paste("+",a, sep = "") }
if(as.numeric(b) > 0)
  { b <- paste("+",b, sep = "") }
if(as.numeric(c) >= 0)
  { c <- paste("+",c, sep = "") }
r.sq <- formatC(cor(fitted(Mod), tmp$response)^2, format = "f",  # the R^2 value
  digits = 2);r.sqInd <- r.sq                                    # stores this for later use in the plot below (comparing the two models)
mtext(side = 3, adj = .9, cex = 1.4, line = -.6,                 # adds information about the model coefficient of determination to the plot
  as.expression( substitute(
   italic(R^2)*"="*Value4,
   list( Value4 = r.sq) ) ) )
mtext(side = 1, adj = .55, cex = 1.15, line = -1.2,              # adds information about the model coefficients to the plot
  as.expression( substitute(
   Value * "("%+-%Valueb* ")"*
   Value2* "("%+-%Value2b* ")Time"*
   Value3* "("%+-%Value3b* ")Time"^2,
   list( Value  = a, Valueb = a.se,
         Value2 = b, Value2b = b.se,
         Value3 = c, Value3b = c.se ) ) ))


   ### STORING THE PREDICTED VALUES PER YEAR ###

MOD.Indi <- Mod                                                  # saves the model and the predictions to compare with the model fitted to the aggregated data below
MODPred.Indi <- newdata

DataReproPred <- data.frame(Year = unique(Data$Year))            # creates a data.frame used to store the predictions from the model
DataReproPred$EggPredDay.fit <-
  predict(GLM.plot, newdata = DataReproPred)                     # the annual expected values from the model
DataReproPred$EggPredDay.sefit <-
  predictSE(GLM.plot, newdata = DataReproPred,se.fit=T)$se.fit   # SE for the annual expected values from the model

rm(list = ls(pattern = "tmp"))
rm(list = ls(pattern = "od"));rm(AICdata)                        # removed redundant objects
rm(j, newdata, GLM.plot, dataGLM, a, a.se, b, b.se, c, c.se,
   Col, r.sq, Tmp)
rm(list = ls(pattern = "LME"));rm(list = ls(pattern = "GLS"))







                                        ###############################################################################
                                        # ANNUAL DAILY NEST PREDATION RATES BASED ON AGGREGATED DATA: 'dataNewColYea' #
                                        ###############################################################################

   ### DATA ORGANIZATION ###

graphics.off()
tmp <- subset(dataNewColYea, NewColony != "NoGood",              # excludes two colonies (see above for detailed comments)
  select = c("NewColony", "Year", "EggPredDay.mean",
  "EggPredDay.se","EggPredDay.sd", "EggPredDay.n",
  "DateDiff.mean"))
names(tmp)[length(names(tmp))] <- "DateDiff"

dim(tmp)
tmp$year <- as.character(tmp$Year)
for(i in yearExlude)
  { tmp <- subset(tmp, year != as.character(i) ) }
dim(tmp)

tmp$Year <- as.numeric(as.character(tmp$Year))
tmp$response <- tmp$EggPredDay.mean
tmp$Year.sc <- scale(tmp$Year, scale = F)[1:dim(tmp)[1]]
tmp$YearSq.sc <- tmp$Year.sc^2
tmp$BinResponse <- ifelse(tmp$response < 0, 1, tmp$response)
tmp <- na.omit(tmp)
dim(tmp)


   ### DEFINING THE BASELINE CANDIDATE MODEL WITH (LME) AND WITHOUT (GLS) RANDOM EFFECTS ###

LME <- lme(response ~ Year.sc + YearSq.sc,
  random =~1|NewColony, tmp, method = "REML", na.action=na.omit)
GLS <- gls(response ~ Year.sc + YearSq.sc,
  tmp, method = "REML",na.action=na.omit)

dev.new(width = 16/2.54, height = 25/2.54)
par(mfrow=c(5,2), mar=c(3,2,2,.5))
plot(fitted(GLS),resid(GLS, type = "p"),xlab="", ylab = "")
abline(h = 0)
title(outer = F, line = .9, "GLS - residuals vs. fitted")
plot(fitted(LME),resid(LME, type = "p"),xlab="", ylab = "")
abline(h = 0)
title(outer = F, line = .9, "LME - residuals vs. fitted")


  ### DEFINING AN APPROPRIATE VARIANCE FUNCTION TO TAKE INTO ACCOUNT HETEROGENEITY IN THE RESIDUALS (FITTING HETEROSCEDASTIC MODELS) ###

GLS.varFixed <- update(GLS, weights = varFixed(value = ~DateDiff) )
LME.varFixed <- update(LME, weights = varFixed(value = ~DateDiff) )
    plot(fitted(GLS.varFixed),resid(GLS.varFixed,
      type = "p"),xlab="", ylab = "")
    abline(h = 0)
    title(outer = F, line = .9, "GLS: varFixed")
    plot(fitted(LME.varFixed),resid(LME.varFixed,
      type = "p"),xlab="", ylab = "")
    abline(h = 0)
    title(outer = F, line = .9, "LME: varFixed")

GLS.varPower <- update(GLS, weights =
                       varPower(form = ~ fitted(.)), control=(msMaxIter=2000))
LME.varPower <- update(LME, weights =
                        varPower(form = ~ fitted(.)))
    plot(fitted(GLS.varPower),resid(GLS.varPower, type = "p"),xlab="", ylab = "")
    abline(h = 0)
    title(outer = F, line = 0.9, "GLS: varPower")
    plot(fitted(LME.varPower),resid(LME.varPower, type = "p"),xlab="", ylab = "")
    abline(h = 0)
    title(outer = F, line = 0.9, "LME: varPower")
GLS.varExp <- update(GLS, weights =
                      varExp(form = ~ fitted(.)), control=c(msMaxIter=10000, maxIter=500))
LME.varExp <- update(LME, weights =
                       varExp(form = ~ fitted(.)))
    plot(fitted(GLS.varExp),resid(GLS.varExp, type = "p"));abline(h = 0)
    title(outer = F, line = 0.9, "GLS: varExp")
    plot(fitted(LME.varExp),resid(LME.varExp, type = "p"),xlab="", ylab = "")
    abline(h = 0)
    title(outer = F, line = 0.9, "LME: varExp")
GLS.varConstPower <- update(GLS, weights =
                             varConstPower( form = ~fitted(.)) )
LME.varConstPower <- update(LME, weights =
                             varConstPower( form = ~fitted(.)) )
    plot(fitted(GLS.varConstPower),resid(GLS.varConstPower),xlab="", ylab = "")
    abline(h = 0)
    title(outer = F, line = .9, "GLS: varConstPower")
    plot(fitted(LME.varConstPower),resid(LME.varConstPower),xlab="", ylab = "")
    abline(h = 0)
    title(outer = F, line = .9, "LME: varConstPower")

dev.copy(tiff, filename = "EggPredDayAggreg_Hetero.tiff",
  units="cm",
  width=16,height=25,res=600, compression = c("lzw"));dev.off()


   ### SELECTING THE MOST PARSIMONIOUS MODEL: RANDOM EFFECTS AND VARIANCE STRUCTURE ###

(summary(LME.varConstPower)$modelStruct$varStruct)                # shows the estimates for the variance structure for the LME
(summary(GLS.varConstPower)$modelStruct$varStruct)                # shows the estimates for the variance structure for the GLS
# NOTE: the constant for these are ~0 so we exclude the .varConstPower models from the set of candidate models. The 'LME.varPower'
#       model did not converge so that is excluded as well.

AICdata.LMEGLS <- data.frame(
  mod =
  c("LME",                    "GLS",
    "GLS.varFixed",           "LME.varFixed",
    "GLS.varPower",           "LME.varPower",
    "GLS.varExp",             "LME.varExp"),
  AICc =
   c(AICc(LME),               AICc(GLS),
    AICc(GLS.varFixed),       AICc(LME.varFixed),
    AICc(GLS.varPower),       AICc(LME.varPower),
    AICc(GLS.varExp),         AICc(LME.varExp)))
AICdata.LMEGLS$df <- AIC(
    LME,                      GLS,
    GLS.varFixed,             LME.varFixed,
    GLS.varPower,             LME.varPower,
    GLS.varExp,               LME.varExp)[,1]
AICdata.LMEGLS$Delta_AICc <- AICdata.LMEGLS$AICc -
  min(AICdata.LMEGLS$AICc)
AICdata.LMEGLS$mod <- as.character(AICdata.LMEGLS$mod)
AICdata.LMEGLS$Aikaike.weights <- exp(-.5*
  AICdata.LMEGLS$Delta_AICc)
AICdata.LMEGLS$Aikaike.weights <- AICdata.LMEGLS$Aikaike.weights/
  sum(AICdata.LMEGLS$Aikaike.weights)

print(AICdata.LMEGLS)
tmp2 <- as.data.frame(subset(AICdata.LMEGLS, Delta_AICc <= 1.5))
tmp2 <- as.character(tmp2$mod[tmp2$df == min(tmp2$df)][1])
mod1 <- get(as.character(tmp2))


   ### DEFINING CANDITATE MODELS WITH DIFFERENT FIXED EFFECTS ###

mod1 <- update(mod1, method = "ML")
mod2 <- update(mod1, ~.-YearSq.sc)


   ### SELECTING THE MOST PARSIMONIOUS MODEL: FIXED EFFECTS ###

Cand.models <- list()
Modnames <- c()
 for (j in 1:(length(ls(pattern = "mod"))-1))
  { Cand.models[[j]] <- get(paste("mod",j, sep = ""))
    Modnames <- c(Modnames, paste("mod",j, sep = "") )
  }
AICdata <- aictab(cand.set = Cand.models, modnames =
   Modnames, second.ord = T, sort = T)
AICdata
tmp2 <- as.data.frame(subset(AICdata, Delta_AICc <= 1.5))
tmp2 <- as.character(tmp2$Modnames[tmp2$K == min(tmp2$K)][1])
Mod <- get(as.character(tmp2))
Mod <- update(Mod, method = "REML")


   ### MODEL DIAGNOSTICS (SELECTED MODEL) ###

dev.new();par(mfrow = c(2,2))
plot(resid(Mod) ~ fitted(Mod),
   main = "Residuals vs. Fitted (Mod)", xlab = "Fitted values",
   ylab = "Residuals")
   abline (h = 0, col = "blue")
   lines(col = "red", smooth.spline(fitted(Mod),
     resid(Mod), df = 3))
qqnorm(resid(Mod, type = "p"));qqline(resid(Mod, type = "p"))
plot(sqrt(abs(resid(Mod, type = "pearson"))) ~
   fitted(Mod),
   main = "Scale-Location", xlab = "Fitted values",
   ylab = expression(sqrt("Standardized residuals")))
   lines(col = "red", smooth.spline(fitted(Mod),
     sqrt(abs(resid(Mod, type = "pearson"))), df = 3))
mtext(side = 3, adj = .2, "EggPredDayAggreg", cex = 1,
  line = -1, outer = T)
plot(resid(GLS) ~ fitted(GLS),
   main = "Residuals vs. Fitted (GLS)", xlab = "Fitted values",
   ylab = "Residuals")
   abline (h = 0, col = "blue")
   lines(col = "red", smooth.spline(fitted(GLS),
     resid(GLS), df = 3))
dev.copy(tiff, filename = "EggPredDayAggreg_ModDiag.tiff", units="cm",
  width=20, height=20, res=600, compression = c("lzw") )
dev.off()


   ### MODEL SUMMARY 'BIOLOGICAL' AND 'STATISTICAL' SIGNIFICANCE ###

anova(Mod)
summary(Mod)$tTable
(r.sq <- as.numeric(rsquared(Mod)[5]))


   ### STORING RESULTS EXTERNALLY ###

sink("EggPredDayAgreg_output.txt")
print("-----------------------------------------------")
print("     MODEL SELECTION: RANDOM EFFECTS & varFunc ")
print("-----------------------------------------------")
print(AICdata.LMEGLS);
print("");print("")
print("-----------------------------------------------")
print("     MODEL SELECTION: FIXED EFFECTS            ")
print("-----------------------------------------------")
print(aictab(cand.set = Cand.models, modnames =  Modnames,
  second.ord = T, sort = F), digits = 4);
print(paste("MODEL SELECTED =",as.character(tmp2)));
print("");print("")
print("-----------------------------------------------")
print("     MODEL SUMMARY                             ")
print("-----------------------------------------------")
print(summary(Mod));print("");print("")
print(paste("R.sq (cor(fitted(Mod), tmp$response)^2) =",
  round(cor(fitted(Mod), tmp$response)^2,4)))
print(paste("R.sq (r.squared[piecewiseSEM]) =",
  round(r.sq,4)));print("")
print("ANOVA TABLE")
print(anova(Mod));print("");print("")
print("-----------------------------------------------")
print("     95% CI                                    ")
print("-----------------------------------------------")
print(intervals(Mod));print("");print("")
sink()



   ### RE-FITTING A MODEL WITHOUT ANY CENTERED PREDICTOR(S) ###

{ if(dim(summary(Mod)$tTable)[1]==2)
    { GLM.plot <- update(Mod, ~Year) }
  else if(dim(summary(Mod)$tTable)[1]==3)
    { GLM.plot <- update(Mod, ~Year + I(Year^2),
       control=(msMaxIter=2000)) }
}

   ### CREATING THE DATA TO PLOT FROM (BOTH THE PREDICTIONS AND THE AGGREGATED DATA AT THE COLONY- AND YEAR-LEVEL) ###

dataGLM <- data.frame(
  prob = dataNewColYea$EggPredDay.mean,
  TotalNo = dataNewColYea$EggPredDay.n,
  Err = dataNewColYea$EggPredDay.se,
  NewColony = dataNewColYea$NewColony,
  Year = dataNewColYea$Year)
dataGLM <- na.omit(dataGLM)
newdata <- data.frame(Year = seq(min(tmp$Year),
  max(tmp$Year), length.out = 100))
newdata$fit <- predict(GLM.plot, newdata = newdata)
newdata$se.fit1 <- newdata$fit-predictSE(GLM.plot,
  newdata = newdata, se.fit = T)$se.fit
newdata$se.fit2 <- newdata$fit+predictSE(GLM.plot,
  newdata = newdata, se.fit = T)$se.fit


   ### SETTING-UP THE PLOT ###

dev.new(height = 14/2.54, width = 16/2.54);
Col <- colorRampPalette(c("firebrick3",
 "dodgerblue3"))(length(levels(as.factor(Data$Year))))
dataGLM$cex <- ((((dataGLM$TotalNo)-(min(dataGLM$TotalNo))+1)/
  (length((dataGLM$TotalNo))))+1)*1.2
dataGLM$year <- jitter(as.numeric(as.character(dataGLM$Year)),
  amount =.25)
dataGLM$err1 <- ifelse(dataGLM$prob+dataGLM$Err >= 0, 0,
  dataGLM$prob+dataGLM$Err)
dataGLM$err2 <- ifelse(dataGLM$prob-dataGLM$Err >= 0, 0,
  dataGLM$prob-dataGLM$Err)
par(cex.axis = 1.2, cex.lab = 1.5, mfrow = c(1,1),
  mar = c(4.5, 4.7, 1.5, 1), bty = "l")
plot(prob ~ year, data = dataGLM,
  type ="n", main = "",
     xlab = expression("Time"*" (year)"),
     ylab = expression("Number of eggs lost"~day^-1%+-%" SE"),
     ylim = range(c(dataGLM$err2, dataGLM$err1)))


   ### PLOTTING THE DATA ###

for(i in 1:length(levels(as.factor(dataGLM$Year))))
  { tmp33 <- subset(dataGLM,
      Year == levels(as.factor(dataGLM$Year))[i])
    points(tmp33$prob ~ tmp33$year,
      col = "white", bg = Col[i], pch = 21, lwd = 1,
      cex = tmp33$cex)
    for(j in 1:length(tmp33[,1]))
      { lines(x = rep(tmp33$year[j],2),
          y = c(tmp33$err1[j], tmp33$err2[j]),
          col = Col[i])
      }
  }


   ### IDENTIFYING THE EXCLUDED YEARS ###

tmp33 <- as.character(yearExlude)
tmp33 <- subset(dataGLM, Year%in%tmp33)
points(tmp33$year-.2, tmp33$prob,col = "black",pch = "(",
  cex = 1.25)
points(tmp33$year+.2, tmp33$prob, col = "black", pch = ")",
 cex = 1.25)


   ### ADDING MODEL PREDICTIONS ###

lines(newdata$fit~newdata$Year, lwd = 3)
lines(newdata$se.fit1~newdata$Year, lwd = 1, lty = 2)
lines(newdata$se.fit2~newdata$Year, lwd = 1, lty = 2)
a <- formatC(summary(Mod)$tTable[1,1],format = "f", digits = 3)
a.se <- ifelse(
   round(summary(Mod)$tTable[1,2],3) == 0,
   "<0.001", formatC(summary(Mod)$tTable[1,2],
   format = "f", digits = 3))
b <- formatC(summary(Mod)$tTable[2,1],format = "f", digits = 3)
b.se <- ifelse(
   round(summary(Mod)$tTable[2,2],3) == 0,
   "<0.001", formatC(summary(Mod)$tTable[2,2],
   format = "f", digits = 3))
c <- formatC(summary(Mod)$tTable[3,1],format = "f", digits = 3)
c.se <- ifelse(
   round(summary(Mod)$tTable[3,2],3) == 0,
   "<0.001", formatC(summary(Mod)$tTable[3,2],
   format = "f", digits = 3))
if(as.numeric(a) > 0)
  { a <- paste("+",a, sep = "") }
if(as.numeric(b) > 0)
  { b <- paste("+",b, sep = "") }
if(as.numeric(c) >= 0)
  { c <- paste("+",c, sep = "") }
r.sq <- formatC(cor(fitted(Mod), tmp$response)^2, format = "f",
  digits = 2)
mtext(side = 3, adj = .9, cex = 1.4, line = -.6,
  as.expression( substitute(
   italic(R^2)*"="*Value4,
   list( Value4 = r.sq) ) ) )
mtext(side = 1, adj = .55, cex = 1.15, line = -1.2,
  as.expression( substitute(
   Value * "("%+-%Valueb* ")"*
   Value2* "("%+-%Value2b* ")Time"*
   Value3* "("%+-%Value3b* ")Time"^2,
   list( Value  = a, Valueb = a.se,
         Value2 = b, Value2b = b.se,
         Value3 = c, Value3b = c.se ) ) ))


   ### STORING THE PREDICTED VALUES PER YEAR ###

MOD.Aggreg <- Mod
MODPred.Aggr <- newdata
DataReproPred$EggPredDayAgreg.fit <-
  predict(GLM.plot, newdata = DataReproPred)
DataReproPred$EggPredDayAgreg.sefit <-
  predictSE(GLM.plot, newdata = DataReproPred,se.fit=T)$se.fit

rm(list = ls(pattern = "tmp"))
rm(list = ls(pattern = "od"));rm(AICdata)
rm(j, newdata, GLM.plot, a, a.se, b, b.se, c, c.se,
   Col)
rm(list = ls(pattern = "LME"));rm(list = ls(pattern = "GLS"))







                                        ##################################################################################
                                        # COMPARING THE TWO MODELS: DAILY NEST PREDATION RATES ACROSS YEARS AND COLONIES #
                                        ##################################################################################


   ### SETTING-UP THE PLOT ###

dev.new(height = 14/2.54, width = 16/2.54);
Col <- colorRampPalette(c("firebrick3",
 "dodgerblue3"))(length(levels(as.factor(Data$Year))))
dataGLM$cex <- ((((dataGLM$TotalNo)-(min(dataGLM$TotalNo))+1)/
  (length((dataGLM$TotalNo))))+1)*1.2
dataGLM$year <- jitter(as.numeric(as.character(dataGLM$Year)),
  amount =.25)
dataGLM$err1 <- ifelse(dataGLM$prob+dataGLM$Err >= 0, 0,
  dataGLM$prob+dataGLM$Err)
dataGLM$err2 <- ifelse(dataGLM$prob-dataGLM$Err >= 0, 0,
  dataGLM$prob-dataGLM$Err)
par(cex.axis = 1.2, cex.lab = 1.5, mfrow = c(1,1),
  mar = c(4.5, 4.7, 1.5, 1), bty = "l")
plot(prob ~ year, data = dataGLM,
  type ="n", main = "",
  xlab = expression("Time"*" (year)"),
  ylab = expression("Number of eggs lost"~day^-1%+-%" SE"),
  ylim = range(c(dataGLM$err2, dataGLM$err1)))


   ### ADDING MODEL PREDICTIONS AND ESTIMATES FOR THE AGGREGATED MODEL ###

polygon(c(MODPred.Aggr$Year, rev(MODPred.Aggr$Year)),
    c(MODPred.Aggr$se.fit1, rev(MODPred.Aggr$se.fit2)),
    col=adjustcolor("light grey",.8), border =NA)
lines(MODPred.Aggr$fit~MODPred.Indi$Year, lwd = 2,
    col = adjustcolor("dark grey",.8))


   ### PLOTTING THE DATA ###

for(i in 1:length(levels(as.factor(dataGLM$Year))))
  { tmp33 <- subset(dataGLM,
      Year == levels(as.factor(dataGLM$Year))[i])
    points(tmp33$prob ~ tmp33$year,
      col = "white", bg = Col[i], pch = 21, lwd = 1,
      cex = tmp33$cex)
    for(j in 1:length(tmp33[,1]))
      { lines(x = rep(tmp33$year[j],2),
          y = c(tmp33$err1[j], tmp33$err2[j]),
          col = Col[i])
      }
  }


   ### IDENTIFYING THE EXCLUDED YEARS ###

tmp33 <- as.character(yearExlude)
tmp33 <- subset(dataGLM, Year%in%tmp33)
points(tmp33$year-.2, tmp33$prob,col = "black",pch = "(",
  cex = 1.25)
points(tmp33$year+.2, tmp33$prob, col = "black", pch = ")",
 cex = 1.25)


   ### ADDING MODEL PREDICTIONS AND ESTIMATES FOR THE INDIVIDUAL-LEVEL MODEL ###

lines(MODPred.Indi$fit~MODPred.Indi$Year, lwd = 3)
lines(MODPred.Indi$se.fit1~MODPred.Indi$Year, lwd = 1, lty = 2)
lines(MODPred.Indi$se.fit2~MODPred.Indi$Year, lwd = 1, lty = 2)
Mod <- MOD.Indi
a <- formatC(summary(Mod)$tTable[1,1],format = "f", digits = 3)
a.se <- ifelse(
   round(summary(Mod)$tTable[1,2],3) == 0,
   "<0.001", formatC(summary(Mod)$tTable[1,2],
   format = "f", digits = 3))
b <- formatC(summary(Mod)$tTable[2,1],format = "f", digits = 3)
b.se <- ifelse(
   round(summary(Mod)$tTable[2,2],3) == 0,
   "<0.001", formatC(summary(Mod)$tTable[2,2],
   format = "f", digits = 3))
c <- formatC(summary(Mod)$tTable[3,1],format = "f", digits = 3)
c.se <- ifelse(
   round(summary(Mod)$tTable[3,2],3) == 0,
   "<0.001", formatC(summary(Mod)$tTable[3,2],
   format = "f", digits = 3))
if(as.numeric(a) > 0)
  { a <- paste("+",a, sep = "") }
if(as.numeric(b) > 0)
  { b <- paste("+",b, sep = "") }
if(as.numeric(c) >= 0)
  { c <- paste("+",c, sep = "") }
r.sq <- r.sqInd
mtext(side = 3, adj = .9, cex = 1.4, line = -.6,
  as.expression( substitute(
   italic(R^2)*"="*Value4,
   list( Value4 = r.sq) ) ) )
mtext(side = 1, adj = .55, cex = 1.15, line = -1.2,
  as.expression( substitute(
   Value * "("%+-%Valueb* ")"*
   Value2* "("%+-%Value2b* ")Time"*
   Value3* "("%+-%Value3b* ")Time"^2,
   list( Value  = a, Valueb = a.se,
         Value2 = b, Value2b = b.se,
         Value3 = c, Value3b = c.se ) ) ))


  ### COMPARING THE PREDICTIONS FROM THE MODELS ###

r <- cor.test(MODPred.Aggr$fit,MODPred.Indi$fit)
r.LCL <- round(r$conf.int[[1]],2)
r.UCL <- round(r$conf.int[[2]],2)
r <- round(r$estimate,2)
mtext(side = 1, adj = .1, cex = 1.15, line = -3.2,
  as.expression( substitute(
   italic(r[pred]) ==
   Value * " [95% CI = "*Valueb* ","*
   Value2* "]",
   list( Value  = r, Valueb = r.LCL,
         Value2 = r.UCL ) ) ))
mean(DataReproPred$EggPredDay.fit)
mean(DataReproPred$EggPredDayAgreg.fit)
sum(DataReproPred$EggPredDay.fit<DataReproPred$EggPredDayAgreg.fit)/
    dim(DataReproPred)[1]


  ### STORING THE PLOT ###

dev.copy(tiff, filename = "EggPredDay.tiff",
  width=16,height=14,res=600, compression = c("lzw"),
  units="cm",);dev.off()

rm(i, j,r.sq, a, a.se, b, b.se, c, c.se, dataGLM, r.sqInd,
   Col, r, r.UCL, r.LCL, tmp33)







                                        ######################################################
                                        # CLUTCH SIZE AS A FUNCTION OF DATE AND YEAR: LAYING #
                                        ######################################################

   ### DATA ORGANIZATION ###

graphics.off()
tmp <- data
tmp$year <- as.character(tmp$Year)                                 # creates a factor variable from year (baseline 2005)
dim(tmp)

tmp$response <- tmp$ClutchMean                                     # response: the across-visit average clutch size
tmp$Date <- tmp$DateMean                                           # the predictor: the across-visit average date (day in June)

dim(tmp)

tmp$Date.sc <- scale(tmp$Date, scale = F)[1:dim(tmp)[1]]
tmp$DateSq.sc <- tmp$Date.sc^2
tmp$year <- as.factor(tmp$Year)
tmp$Year.sc <- scale(tmp$Year, scale = F)[1:dim(tmp)[1]]


   ### DEFINING THE BASELINE CANDIDATE MODEL WITH (LME) AND WITHOUT (GLS) RANDOM EFFECTS ###

LME <- lme(response ~ Date.sc +
           year,
  random =~1|NewColony, tmp, method = "REML", na.action=na.omit)
GLS <- gls(response ~ Date.sc +
           year,
           tmp, method = "REML",na.action=na.omit)

dev.new(width = 16/2.54, height = 25/2.54)
par(mfrow=c(5,2), mar=c(3,2,2,.5))
plot(fitted(GLS),resid(GLS, type = "p"),xlab="", ylab = "")
abline(h = 0)
title(outer = F, line = .9, "GLS - residuals vs. fitted")
plot(fitted(LME),resid(LME, type = "p"),xlab="", ylab = "")
abline(h = 0)
title(outer = F, line = .9, "LME - residuals vs. fitted")


  ### DEFINING AN APPROPRIATE VARIANCE FUNCTION TO TAKE INTO ACCOUNT HETEROGENEITY IN THE RESIDUALS (FITTING HETEROSCEDASTIC MODELS) ###

GLS.varFixed <- update(GLS, weights = varFixed(value = ~DateDiff) )
LME.varFixed <- update(LME, weights = varFixed(value = ~DateDiff) )
    plot(fitted(GLS.varFixed),resid(GLS.varFixed, type = "p"),xlab="", ylab = "")
    abline(h = 0)
    title(outer = F, line = .9, "GLS: varFixed")
    plot(fitted(LME.varFixed),resid(LME.varFixed, type = "p"),xlab="", ylab = "")
    abline(h = 0)
    title(outer = F, line = .9, "LME: varFixed")

GLS.varPower <- update(GLS, weights =
                       varPower(form = ~ fitted(.)))
LME.varPower <- update(LME, weights =
                        varPower(form = ~ fitted(.)))
    plot(fitted(GLS.varPower),resid(GLS.varPower, type = "p"),xlab="", ylab = "")
    abline(h = 0)
    title(outer = F, line = 0.9, "GLS: varPower")
    plot(fitted(LME.varPower),resid(LME.varPower, type = "p"),xlab="", ylab = "")
    abline(h = 0)
    title(outer = F, line = 0.9, "LME: varPower")

GLS.varExp <- update(GLS, weights =
                      varExp(form = ~ fitted(.)))
LME.varExp <- update(LME, weights =
                       varExp(form = ~ fitted(.)))
    plot(fitted(GLS.varExp),resid(GLS.varExp, type = "p"));abline(h = 0)
    title(outer = F, line = 0.9, "GLS: varExp")
    plot(fitted(LME.varExp),resid(LME.varExp, type = "p"),xlab="", ylab = "")
    abline(h = 0)
    title(outer = F, line = 0.9, "LME: varExp")

GLS.varConstPower <- update(GLS, weights =
                            varConstPower( form = ~fitted(.)) )
LME.varConstPower <- update(LME, weights =
                            varConstPower( form = ~fitted(.)) )
    plot(fitted(GLS.varConstPower),resid(GLS.varConstPower),xlab="", ylab = "")
    abline(h = 0)
    title(outer = F, line = .9, "GLS: varConstPower")
    plot(fitted(LME.varConstPower),resid(LME.varConstPower),xlab="", ylab = "")
    abline(h = 0)
    title(outer = F, line = .9, "LME: varConstPower")

dev.copy(tiff, filename = "ClutchDateAdj_Hetero.tiff",
  units="cm",
  width=16,height=25,res=600, compression = c("lzw"));dev.off()


   ### SELECTING THE MOST PARSIMONIOUS MODEL: RANDOM EFFECTS AND VARIANCE STRUCTURE ###

AICdata.LMEGLS <- data.frame(
  mod =
  c("LME",                    "GLS",
   "GLS.varFixed",            "LME.varFixed",
    "GLS.varPower",           "LME.varPower",
    "GLS.varExp",             "LME.varExp",
    "GLS.varConstPower",      "LME.varConstPower"
    ),
  AICc =
   c(AICc(LME),               AICc(GLS),
   AICc(GLS.varFixed),        AICc(LME.varFixed),
    AICc(GLS.varPower),       AICc(LME.varPower),
    AICc(GLS.varExp),         AICc(LME.varExp),
    AICc(GLS.varConstPower),  AICc(LME.varConstPower)))
AICdata.LMEGLS$df <- AIC(
    LME,                      GLS,
    GLS.varFixed,             LME.varFixed,
    GLS.varPower,             LME.varPower,
    GLS.varExp,               LME.varExp,
    GLS.varConstPower,        LME.varConstPower)[,1 ]
AICdata.LMEGLS$Delta_AICc <- AICdata.LMEGLS$AICc -
  min(AICdata.LMEGLS$AICc)
AICdata.LMEGLS$mod <- as.character(AICdata.LMEGLS$mod)
AICdata.LMEGLS$Aikaike.weights <- exp(-.5*AICdata.LMEGLS$Delta_AICc)
AICdata.LMEGLS$Aikaike.weights <- AICdata.LMEGLS$Aikaike.weights/
  sum(AICdata.LMEGLS$Aikaike.weights)

print(AICdata.LMEGLS)
tmp2 <- as.data.frame(subset(AICdata.LMEGLS, Delta_AICc <= 1.5))
tmp2 <- as.character(tmp2$mod[tmp2$df == min(tmp2$df)][1])
mod1 <- get(as.character(tmp2))


   ### DEFINING CANDITATE MODELS WITH DIFFERENT FIXED EFFECTS ###

mod1 <- update(mod1, method = "ML")
mod2 <- update(mod1, ~.-year)
    # i      Date  Year
    # mod1   X     X
    # mod2   X


   ### SELECTING THE MOST PARSIMONIOUS MODEL: FIXED EFFECTS ###

Cand.models <- list()
Modnames <- c()
 for (j in 1:(length(ls(pattern = "mod"))-1))
  { Cand.models[[j]] <- get(paste("mod",j, sep = ""))
    Modnames <- c(Modnames, paste("mod",j, sep = "") )
  }
AICdata <- aictab(cand.set = Cand.models, modnames =
   Modnames, second.ord = T, sort = T)
AICdata
tmp2 <- as.data.frame(subset(AICdata, Delta_AICc <= 1.5))
tmp2 <- as.character(tmp2$Modnames[tmp2$K == min(tmp2$K)][1])
Mod <- get(as.character(tmp2))
Mod <- update(Mod, method = "REML")


   ### MODEL DIAGNOSTICS (SELECTED MODEL) ###

dev.new();par(mfrow = c(2,2))
plot(resid(Mod) ~ fitted(Mod),
   main = "Residuals vs. Fitted (Mod)", xlab = "Fitted values",
   ylab = "Residuals")
   abline (h = 0, col = "blue")
   lines(col = "red", smooth.spline(fitted(Mod),
     resid(Mod), df = 3))
qqnorm(resid(Mod));qqline(resid(Mod))
plot(sqrt(abs(resid(Mod, type = "pearson"))) ~
   fitted(Mod),
   main = "Scale-Location", xlab = "Fitted values",
   ylab = expression(sqrt("Standardized residuals")))
   lines(col = "red", smooth.spline(fitted(Mod),
     sqrt(abs(resid(Mod, type = "pearson"))), df = 3))
mtext(side = 3, adj = .2, "ClutchDateAdj", cex = 1,
  line = -1, outer = T)
plot(resid(GLS) ~ fitted(GLS),
   main = "Residuals vs. Fitted (GLS)", xlab = "Fitted values",
   ylab = "Residuals")
   abline (h = 0, col = "blue")
   lines(col = "red", smooth.spline(fitted(GLS),
     resid(GLS), df = 3))
dev.copy(tiff, filename = "ClutchDateAdj_ModDiag.tiff", units="cm",
  width=20, height=20, res=600, compression = c("lzw") )
dev.off()


   ### MODEL SUMMARY 'BIOLOGICAL' AND 'STATISTICAL' SIGNIFICANCE ###

anova(Mod)
summary(Mod)
(r.sq <- as.numeric(rsquared(Mod)[5]))
TMPP <- na.omit(tmp)

   ### STORING RESULTS EXTERNALLY ###

sink("ClutchDateAdj_output.txt")
print("-----------------------------------------------")
print("     MODEL SELECTION: RANDOM EFFECTS & varFunc ")
print("-----------------------------------------------")
print(AICdata.LMEGLS);
print("");print("")
print("-----------------------------------------------")
print("     MODEL SELECTION: FIXED EFFECTS            ")
print("-----------------------------------------------")
print(aictab(cand.set = Cand.models, modnames =  Modnames,
  second.ord = T, sort = F), digits = 4);
print(paste("MODEL SELECTED =",as.character(tmp2)));
print("");print("")
print("-----------------------------------------------")
print("     MODEL SUMMARY                             ")
print("-----------------------------------------------")
print(summary(Mod));print("");print("")
print(paste("R.sq (cor(fitted(Mod), tmp$response)^2) =",
  round(cor(fitted(update(Mod, data=TMPP)), TMPP$response )^2,4)))
print(paste("R.sq (r.squared[piecewiseSEM]) =",
  round(r.sq,4)));print("")
print("ANOVA TABLE")
print(anova(Mod));print("");print("")
print("-----------------------------------------------")
print("     95% CI                                    ")
print("-----------------------------------------------")
print(intervals(Mod));print("");print("")
sink()


   ### PREDICTIONS: USED IN THE PLOTS AND TO ASSESS LAYING CLUTCH SIZE: I.E., PREDICTED VALUES AT 'startDate' (OUR A PRIORI DEFINED LAYING DATE) ###

tmp <- data.frame(response = tmp$response,
  NewColony = tmp$NewColony, Date.sc = tmp$Date,
  DateSq.sc = tmp$Date^2, year = tmp$year, Year = tmp$Year  )
tmp <- na.omit(tmp)
r.sq <- formatC(r.sq, format = "f", digits = 2)
Mod <- update(Mod,~., tmp)
se <- function(x) { sd(na.omit(x)) / sqrt(length(na.omit(x))) }   # function for calculating SE
for(i in levels(tmp$year))                                        # creates data used for plotting ('Tmp') and in the model for adjusting the ('ClutchAdjust')
    { tmp1 <- data.frame(year = rep(i,100))
      tmp2 <- subset(tmp, Year == i)
      tmp1$Date.sc <- seq(min(tmp2$Date.sc,na.rm=T),
        max(tmp2$Date.sc,na.rm=T),
        length.out=100)
      tmp3 <- data.frame(year = i,
         Data.median = median(tmp2$Date.sc),
         Date.sc = startDate,
         Clutch.mean = mean(tmp2$response,na.omit=T),
         Clutch.se = se(tmp2$response),
         Clutch.min = min(tmp2$response,na.omit=T),
         Clutch.max = max(tmp2$response,na.omit=T),
         Clutch.n = dim(tmp2)[1])

      { if (i == levels(tmp$year)[1])
          { Tmp <- tmp1
            ClutchAdjust <- tmp3 }
        else
          { Tmp <- rbind.data.frame(Tmp, tmp1)
            ClutchAdjust <- rbind.data.frame(ClutchAdjust,tmp3) }
      }
    }

Tmp$fit <- predict(Mod,newdata=Tmp, level = 0)
Tmp$se.fit <- predictSE(Mod,newdata=Tmp,se=T)$se.fit
Tmp$se.fit1 <- Tmp$fit + Tmp$se.fit
Tmp$se.fit2 <- Tmp$fit - Tmp$se.fit

ClutchAdjust$ClutchStart <- predict(Mod,newdata=ClutchAdjust,
  level = 0)
ClutchAdjust$ClutchStartSE <- predictSE(Mod,
  newdata=ClutchAdjust,se=T)$se.fit


   ### PLOTTING ###

graphics.off()
lwd <- c(3.5)                                                     # line width
cex = 1.3                                                         # point size
col <- colorRampPalette(c("firebrick3",                           # colour palette used to indicate temporal trends
   "dodgerblue3"))(length(levels(tmp$year)))
col <- paste(col, "70", sep="")                                   # makes the colours transparent
pch <- rep(16, length(levels(tmp$year)))
Col <- colorRampPalette(c("firebrick3",                           # colour palette used to indicate temporal trends
  "dodgerblue3"))(length(levels(tmp$year)))

dev.new(width = 16/2.54, height = 10/2.54)
par(cex.axis = 1.2, cex.lab = 1.5, mar = c(4.5, 4.5, 1, 1),
  bty = "l")

plot(fit~Date.sc, Tmp,
  ylim = c(min(c(Tmp$se.fit2, na.omit(tmp$response))),
           max(c(Tmp$se.fit1, na.omit(tmp$response)))),
  xlim = c(min(c(startDate, tmp$Date.sc)),
           max(c(tmp$Date.sc))),
   type = "n", xlab = "", ylab ="")
for (j in 1:length(levels(tmp$year)))
  { tmp1 <- subset(tmp, year == levels(tmp$year)[j])
    smoothScatter(nrpoints = 0,tmp1$response ~ tmp1$Date.sc,
      ylim = c(min(c(Tmp$se.fit2, tmp$response)),
      max(c(Tmp$se.fit1, tmp$response))), add = T, colramp =
      colorRampPalette(c(
        rgb(red   = col2rgb("white")[1],
            green = col2rgb("white")[2],
            blue  = col2rgb("white")[3],
              max = 255, alpha = 0.05 ),
        col[j]), alpha = T))
    tmp1$response <- jitter(tmp1$response, factor =
      mean(tmp1$response)*.055)
    tmp1$Date.sc <- jitter(tmp1$Date.sc, factor =
      mean(tmp1$Date.sc)*.015)
    points(tmp1$response ~ tmp1$Date.sc,
      cex = cex, pch = 21, bg = col[j], col = "white")
    rug(side = 1, x = jitter(tmp1$Date.sc, factor =
      mean(tmp1$Date.sc)*9.5), col = Col[j],
      ticksize = ((length(levels(tmp$year)):1)[j])/300)
  }
for (j in 1:length(levels(tmp$year)))
  { lines(fit~Date.sc, subset(Tmp, year ==
      levels(tmp$year)[j]), lwd = lwd, col = Col[j])
    lines(se.fit1~Date.sc, subset(Tmp, year ==
      levels(tmp$year)[j]), lwd = lwd*.5, lty=2,
      col = Col[j])
    lines(se.fit2~Date.sc, subset(Tmp, year ==
      levels(tmp$year)[j]), lwd = lwd*.5, lty=2,
      col = Col[j])
    Tmp1 <- subset(ClutchAdjust, year ==
      levels(tmp$year)[j])
    points(Tmp1$ClutchStart ~
      jitter(Tmp1$Date.sc), cex = cex, pch = 21,
      bg = col[j], col = "white")
  }
mtext(side = 1, expression( "Date (days since June" ~
  "1"^"st"*")"),
  cex = 1.5, outer = T, line = -.8)

legend(y = max(ClutchAdjust$ClutchStart)*1.6, x =
  min(ClutchAdjust$Date.sc),
  legend = levels(tmp$year), title.col = "black",
  text.col = Col, cex = .8,
  title = "Year", bty = "n")
r.sq <- formatC(cor(fitted(Mod), tmp$response)^2, format = "f",
  digits = 2)
mtext(side = 3, adj = .9, cex = 1.4, line = -.6,
  as.expression( substitute(
   italic(R^2)*"="*Value4,
   list( Value4 = r.sq) ) ) )

Tmp <- "Clutch size (# of eggs)"
mtext(side = 2, cex = 1.5, line = -1.9,
  as.expression( substitute(Value %+-% SE,
  list( Value = Tmp ) ) ), outer = T, adj = .6)
dev.copy(tiff, filename = "ClutchDateAdj_RESULT.tiff", units="cm",
  width=16, height=10, res=600, compression = c("lzw"));dev.off()
names(ClutchAdjust)[2:3] <- c("Date.median", "startDate")
rm(list = ls(pattern = "mod"));rm(list = ls(pattern = "AIC"))
rm(list = ls(pattern = "tmp"));rm(list = ls(pattern = "Tmp"))
rm(list = ls(pattern = c("GLS")));rm(list = ls(pattern = c("LME")))
rm(r.sq, j, i, TMPP)







                                        ###############################################################################################
                                        # BOOKKEEPING - EXTRACTING PRECICTED LAYING AND HATCHING CLUTCH SIZE AND PLOTTING THE RESULTS #
                                        ###############################################################################################


   ### PLOTTING THE DIFFERENCE BETWEEN LAYING (ADJUSTED) AND OBSERVED AVERAGE CLUSTER SIZE ###

graphics.off()
dev.new(width = 16/2.54, height = 10/2.54)
par(cex.axis = 1.2, cex.lab = 1.5, mar = c(4.5, 4.5, 1.5, 1),
  bty = "l")

hist(ClutchAdjust$ClutchStart-ClutchAdjust$Clutch.mean, breaks = 20,
  xlab = "Model predictions vs. empirical average",
  main = "Clutch size at laying" )
points(x = mean(ClutchAdjust$ClutchStart-ClutchAdjust$Clutch.mean,na.rm = T),
  y = 1.5, pch = 21, bg = "red", col = "white", cex = 2, lwd = 2)
lines(y = rep(1.5,2), col = "red",  lwd = 2,
  x = c(mean(ClutchAdjust$ClutchStart-ClutchAdjust$Clutch.mean,na.rm = T) -
    sd(ClutchAdjust$ClutchStart-ClutchAdjust$Clutch.mean,na.rm = T),
  y =  mean(ClutchAdjust$ClutchStart-ClutchAdjust$Clutch.mean,na.rm = T) +
    sd(ClutchAdjust$ClutchStart-ClutchAdjust$Clutch.mean,na.rm = T)))
dev.copy(tiff, filename = "ClutchDateAdj.tiff", units="cm",
   width=16, height=10, res=600, compression = c("lzw"));dev.off()

#NOTES:
# 1) 'ClutchStart' is the predicted value from the model (i.e., the predicted clutch
#    size at 'startDate' from the model where clutch size is predicted as a function of year and date).
# 2) 'Clutch.mean' is the empirical clutch size in the same analyses.


   ### MERGING AND STORING MODEL OUTPUT (ANNUAL ESTIMATES): ClutchAdjust (LAYING CLUTCH SIZE) AND DataReproPred (DAILY NEST PREDATION RATES)E ###

ClutchAdjust <- ClutchAdjust[,-c(2:3,6:7)]                          # remove some variables from 'ClutchAdjust'
ClutchAdjust <- merge(ClutchAdjust, DataReproPred,                  # merges the data on laying clutch size and empirical average with the output from the predation analyses
   by.x = "year", by.y = "Year")
ClutchAdjust$EggPredTot <- ClutchAdjust$EggPredDay.fit*             # total number of eggs lost during incubation based on 'EggPredDay.fit' (from the model based on the individual nests)
  DaysIncubation                                                    #  multiplied with the length of the incubation period in days ('DaysIncubation')
DataAdjust <- data.frame(                                           # creates a single data.frame that contains both clutch size at the start of
  type         = c(rep("Start (Control)", dim(ClutchAdjust)[1]),    # the incubation period (date-adjusted) and the end (after the predators
    rep("End (Predated)",dim(ClutchAdjust)[1])),                    # have taken their toll based on the individual data ['EggPredTot'])
  year         = rep(ClutchAdjust$year, 2),
  Clutch.mean  = c(ClutchAdjust$ClutchStart,
    (ClutchAdjust$ClutchStart + ClutchAdjust$EggPredTot)),
  Clutch.se1   = c(ClutchAdjust$ClutchStart +
    ClutchAdjust$ClutchStartSE,
     ClutchAdjust$ClutchStart +(ClutchAdjust$EggPredDay.fit+
     ClutchAdjust$EggPredDay.sefit)*DaysIncubation),
  Clutch.se2   = c(ClutchAdjust$ClutchStart -
    ClutchAdjust$ClutchStartSE,ClutchAdjust$ClutchStart +
    ((ClutchAdjust$EggPredDay.fit-ClutchAdjust$EggPredDay.sefit)*
    DaysIncubation)))


      ### PERFORMING THE LOWESS SMOOTHER AND IMPUTES NEGATIVE VALUES FOR HATCHING CLUTCH SIZE ('END') ###

# NOTES: extracts negative values for hatching clutch size and using a loess smoother to impute new (positive) values for these years

tmp <- DataAdjust
tmp.rows <- row.names(subset(DataAdjust, Clutch.mean < 0))          # extracts the entries with negative values
for(i in 3:5)                                                       # loops over the continuous variables (i.e., the estimated means and its precision)
  { tmp[,i] <- ifelse(tmp[,i] < 0, NA, tmp[,i])
    tmp1 <- subset(tmp, type == "End (Predated)")                   # only relevant to clutch size in the end of incubation (hatching clutch size)
    tmp1$index <- 1:dim(tmp1)[1]
    tmp.low <- loess(tmp1[,i] ~as.numeric(tmp1$year))               # fitting a loess smoother
    tmp.low <- predict(tmp.low, data.frame(year = tmp1$year))       # prediction from the loess models
    tmp1 <- subset(tmp1, is.na(tmp1[,i]))                           # only negative values (defined by 'index') is given NA's
    tmp1[,i] <- tmp.low[tmp1$index]                                 # these values imputed based on the LOESS model
    tmp[row.names(tmp1),i] <- tmp1[,i]
  }


      ### PLOTTING CLUTCH SIZE: 1) AT LAYING (START); 2) AT HATCHING (END); 3) THE EMPIRICAL AVERAGES ###

tmp$measure <- as.factor(as.character(tmp$year))                    # defines 'year' as a factor variable called 'measure'
tmp$response <- tmp$Clutch.mean
tmp$type <- as.factor(tmp$type)                                     # defines 'type' as a factor variable with 'Start ...' as the baseline
tmp$type <- relevel(tmp$type, ref = "Start (Control)")
Mean <- tapply(tmp$response, list(tmp$type, tmp$measure), mean)     # calculates the mean for each response across type
Mean                                                                # shows values on-screen
Sd <- tapply(tmp$Clutch.se2, list(tmp$type, tmp$measure), mean)     # calculates SE2 for each response across type
Lcl <- Mean - Sd                                                    # lower limits for the bars

Sd <- tapply(tmp$Clutch.se1, list(tmp$type, tmp$measure), mean)     # calculates SD for each response across type
Ucl <- Mean + Sd                                                    # upper limits for the bars

col <- c()                                                          # defines transparent colours to be used in the plot
for(i in Col)
  { col <- c(col,
      adjustcolor(i, 0.2),
      adjustcolor(i, 0.8)) }

dev.new(height = 16/2.54, width = 15/2.54);
par(cex.axis = 1.2, cex.lab = 1.5,
  mar = c(4.5, 4.5, 1.75, 1))
mp <- barplot(Mean, beside = T,                                     # plotting the results
  xlab = "Time (year)",
  ylab = expression("Clutch size (# of eggs) "%+-%" SE"),
  col  = col, xaxt = "n",
  ylim = c((min(Mean,na.rm=T)*.9),
    max(Ucl,na.rm=T)),xpd=F, log = "y")
segments(mp, Ucl, mp, Mean, lwd=2, col = "dark grey")
tmp1 <- sapply(data.frame(mp), mean, 2)
tmp1 <- floor(seq(min(tmp1), max(tmp1), length.out = 8))
tmp2 <- floor(seq(min(as.numeric(as.character(ClutchAdjust$year))),
   max(as.numeric(as.character(ClutchAdjust$year))),length.out = 8))

axis(1, seq(min(tmp1), max(tmp1), length.out = 8), labels = FALSE)
text(x = tmp1, y=exp(par("usr")[3]) - 0.165, labels = tmp2,         # adding axis notations with a 45 degrees angle
   srt = 45,  pos = 1, xpd = TRUE, cex = 1.3)

tmp22 <- tmp                                                        # extracts information regarding the excluded years
tmp$year <- as.character(tmp$year)
tmp33 <- subset(tmp22, year%in%yearExlude)
tmp33 <- subset(tmp33, type != "Start (Control)")

for(i in 1:dim(ClutchAdjust)[1])
  { lines(x = rep(mean(mp[,i]),2),                                  # adds SE for the model predictions
      y = c(ClutchAdjust$Clutch.mean[i]-ClutchAdjust$Clutch.se[i],
           ClutchAdjust$Clutch.mean[i]+ClutchAdjust$Clutch.se[i]),
      lwd = 2)
    points(ClutchAdjust$Clutch.mean[i]~mean(mp[,i]),                # arrows to indicate the excluded years
      pch = 21, bg = Col[i], col = "black", cex = 2, lwd = 2)
    { if (ClutchAdjust$year[i]%in%yearExlude)
        { arrows(x0 = mp[2,i], x1 = mp[2,i],
             y1 = Mean[2,i], y0 = min(Sd[2,]),
          lwd = 6, length = .17, col = "orange") }
    }
  }
dev.copy(tiff, filename = "LayHatchSize.PredDay.tiff", units="cm",  # saves the plot as .tiff-file
  width=16, height=15, res=600, compression = c("lzw"));dev.off()


      ### CREATING AN FINAL DATA FRAME WITH LAYING AND HATCHING CLUTCH SIZE (BOTH OBSERVATIONS AND IMPUTED VALUES) ###

DataAdjustEggPred <- DataAdjust[,1:5]                               # makes a new data.frame to store the results
DataAdjustEggPred$Clutch.meanLOWESS <- tmp[,dim(tmp)[2]]            # gets the LOESS smoother-values
DataAdjustEggPred$Clutch.meanImpute <- ifelse(                      # denotes which observations that are imputed
   DataAdjustEggPred$Clutch.meanLOWESS-                             # (i.e., gives negative clutch size at the end
   DataAdjustEggPred$Clutch.mean==0,                                # of incubation)
   "Observed - positive hatching clutch size",
   "Imputed (LOESS smoother) - negative hatching clutch size")

rm(i, Sd, Mean, col, mp)                                            # removed redundant objects
rm(list = ls(pattern = "tmp"))
rm(list = ls(pattern = "Tmp"))


   ### STORING THE RESULTS ###

Metadata <- data.frame(                                             # Metadata: information about the variables in the two data.frames
  DataSets =
    c(rep("DataAdjustEggPred", dim(DataAdjustEggPred)[2]),
      rep("ClutchAdjust", 9)),
  Variables = c(
    "type", "year", "clutch.mean", "clutch.se1", "clutch.se2", "Clutch.meanLOWESS", "Clutch.meanImpute",
    "year", "clutch.mean", "clutch.se", "clutch.n", "ClutchStart", "ClutchStartSE", "EggPredDay.fit/EggPredDayAggreg.fit", "EggPredDay.sefit/EggPredDayAggreg.sefit",
    "EggPredDay.Tot"),
  Interpreation = c(
    c(paste("'Start (Control)' - Start (June ",startDate,"st) and 'End (Predated)' of the breeding season (Appendix S2:Table S5)",sep=""),
    "time (in years)",
    "Clutch size based on models presented in Appendix S2:Table S2 (Start) and Appendix S2:Table S4a (End)",
    "+1 SE for the predictions from the models (Appendix S2:Table S5)",
    "-1 SE for the predictions from the models (Appendix S2:Table S5)",
    "Negative values for 'Clutch.mean', 'Clutch.se1' and 'Clutch.se2' have been replaced by predictions from a LOESS-smoother (Appendix S2:Table S5)",
    "'Observed' means original values and 'Imputed' denotes imputed values (Table S5 & Figure S5 in Appendix S2)",
    "same as 'year' in 'DataAdjustEggPred'",
    "average clutch size (average across first- and second-visit used in the analyses presented in Appendix S2:Table S1-2",
    "+/- 1 SE for 'clutch.mean' (Appendix S2:Table S5)",
    "sample size for clutch size Appendix S2:Table S5",
    paste("predicted clutch size at laying (Appendix S2:Table S1-2): defined at June", startDate),
    "+/- 1 SE for 'ClutchStart' (Table S1-2 and Figure S1 in Appendix S2)",
    "predicted values for the nest predation models (Table S3-4 in Appendix S2): 'EggPredDay' and' EggPredDayAgreg' is based on nest-level and aggregated data (Appendix S2:Figure S2), respectively",
    "+/- 1 SE for the predicted values for models (Appendix S2:Table S4)",
    paste("Total number of eggs removed by predators based on 'EggPredDay.fit' and incubation time from the literature (DaysIncubation=", DaysIncubation, " days)",sep=""))))


save(ClutchAdjust, DataAdjustEggPred,
  Metadata, file = "ClutchAdjust.RDATA")

wb <- createWorkbook()
addWorksheet(  wb, sheetName = "ClutchAdjust")                      # creates the sheet
freezePane(    wb, sheet = "ClutchAdjust", firstRow = TRUE,
  firstCol = TRUE)                                                  # freeze first row and column
setColWidths( wb,  sheet = "ClutchAdjust",widths ="auto",
  cols = 1:10)                                                      # sets column width
writeDataTable(wb, sheet = "ClutchAdjust", x = ClutchAdjust)        # adds data to the sheet
addWorksheet(  wb, sheetName = "DataAdjustEggPred")
freezePane(    wb, sheet = "DataAdjustEggPred", firstRow = TRUE,
 firstCol = TRUE)
setColWidths( wb,  sheet = "DataAdjustEggPred",widths ="auto",
  cols = 1:10)
writeDataTable(wb, sheet = "DataAdjustEggPred",
  x = DataAdjustEggPred)
addWorksheet(  wb, sheetName = "Metadata")
freezePane(    wb, sheet = "Metadata",
  firstRow = TRUE, firstCol = TRUE)
setColWidths( wb,  sheet = "Metadata", widths ="auto",
  cols = 1:10)
writeDataTable(wb, sheet = "Metadata",
  x = Metadata)
saveWorkbook(wb, "ClutchAdjust.xlsx", overwrite = TRUE)             # saves the file









                                        ########################
                                        # STORING SESSION INFO #
                                        ########################


sink("SessionInfo.txt")
print(sessionInfo(), locale = F)
sink()
