                                       ############################################################
                                      ####                                                      ####
                                      ####  ANALYSES OF APPARENT SURVIVAL:                      ####
                                      ####                                                      ####
                                      ####  Lesser black-backed gull (Larus fuscus) on Horsvær, ####
                                      ####  Northern Norway                                     ####
                                      ####                                                      ####
                                      ####     Bård-Jørgen Bårdsen (bjb@nina.no) & Jan Ove      ####
                                      ####     Bustnes, 15.02.2022                              ####
                                      ####                                                      ####
                                       ############################################################




                                        ################################
                                        # CHANGING OPTIONS & LIBRARIES #
                                        ################################


# NOTES: For this script to function (i.e., for the required input files to load and to store the output in the correct directory), you must ensure that:
#       1) you keep the same structure (directories) as in the original .zip-file;
#       2) you define the current working directory of this R-session to be the same directory as the location of the script file (run 'getwd()' the check this, and use 'setwd(path to
#          this directory on your computer)' to fix it.


install.packages( c("marked", "openxlsx", "RMark", "R2ucare"),    # getting the required packaged (from the CRAN sever in Bergen, Norway)
    repos = "http://cran.ii.uib.no/" )                            #   another server can be selected by changing repos; or by running: 'install.packages(c("openxlsx","AICcmodavg", ...))'
library(R2ucare)                                                  # loads the package 'Goodness-of-fit tests for capture-recapture models'
library(openxlsx)                                                 # loads the package 'openxlsx: Read, Write and Edit xlsx Files'
library(RMark)                                                    # loads the package 'RMark: An R Interface for Analysis of Capture-Recapture Data with MARK"

rm(list = ls(all = T))
label <- "SurvivalAnalyses"




                                        #####################
                                        # DATA MANIPULATION #
                                        #####################


  ############################
  ### LOADING THE RAW DATA ###
  ############################

wd <- getwd()                                                     # get current working directory
tmp <- unlist(strsplit(wd,"/"))                                   # splits the current working directory into its sub-directories
tmp <- tmp[-length(tmp)]                                          # removes that last one
tmp <- paste(tmp,collapse="/")                                    # puts the path together again (i.e., the same as moving to the parent directory from the current working directory)
tmp <- paste(tmp, sep="",                                         # create a path to where the survival data is located
  "/Data/DataManipulation_(2022.02)/dataManip(ExclExt-FALSE).RData")

load(tmp);rm(tmp)
dir.create(label, showWarnings = F)
setwd(label);rm(label)
data$initial.age <- ifelse(data$MarkStat == "Adult", 3, 0)        # creates a variable with initial age of each individual (3 = marked as adults; 0 = marked as chicks); the 'process.data' function needs this variable
data$ch <- as.character(data$ch)




                                        ########################################################################################
                                        # TESTING GOODNESS OF FIT (GOF) USING RELEASE (VIA 'RMark'/MARK) AND 'r2care' (U-CARE) #
                                        ########################################################################################


   #############################
   ### NO GROUPING STRUCTURE ###
   #############################


tmp <- splitCH(as.character(data$ch))                             # data on re-sighting used in the test


   ### STANDARD GOF: ASSUMING NO TRANSIENCE NOR TRAP DEPENDENCE ###

(oCJS <- overall_CJS(X = tmp,freq = rep(1, length(tmp[,1]))))     # GOF TESTS II: TEST 2 and TEST 3 as reported by the program R2ucare (see https://doi.org/10.1101/192468 for details)
                                                                  #   we first assess the overall fit of the CJS model by using the function overall_CJS: if significant the CJS models does not fit the data well
(t2ct <- test2ct(X = tmp, freq = rep(1, length(tmp[,1])),         # TEST 2 CT test if "Missed individuals have the same chance to be recaptured at the next occasion as currently captured individuals (null hypothesis
  verbose = TRUE))                                                #     of Test 2.CT)": i.e., a 'test of trap-dependence' (Gimez et al. 2017:3).
(t3sr <- test3sr(X = tmp, freq = rep(1, length(tmp[,1])),         # TEST 3 SR test if "Newly encountered individuals have the same chance to be later reobserved as recaptured (previously encountered) individuals
  verbose = TRUE))                                                #     (null hypothesis of Test 3.SR)": i.e., a 'test of transience' (Gimez et al. 2017:3).
(t3sm <- test3sm(X = tmp, freq = rep(1, length(tmp[,1])),         # TEST 3 SM test if "Have newly encountered not immediately recaptured individuals the same timing of re-en-counters as previously captured not
  verbose = TRUE))                                                #     immediately recaptured individuals (null hypothesis of Test 3.Sm)?": i.e., an examination of long-term features of the data (Gimez et al. 2017:3)
(t2cl <- test2cl(X = tmp, freq = rep(1, length(tmp[,1])),         # TEST 3 CL test if "Have missed not immediately recaptured individuals the same timing of reencounters as currently captured not immediately
  verbose = TRUE))                                                #     recaptured individuals (null hypothesis of Test 2.CL)?": yet another examination of long-term features of the data
(Chi_sq <- t2ct$test2ct[1] + t3sr$test3sr[1] + t3sm$test3sm[1] +  # TEST THE COMPONENST THAT MAKES UP THE 'overall_CJS' chi-square value
t2cl$test2cl[1]);oCJS[1]
(df <- t2ct$test2ct[2] + t3sr$test3sr[2] + t3sm$test3sm[2] +      # TEST THE COMPONENST THAT MAKES UP THE 'overall_CJS' df
t2cl$test2cl[2]);oCJS[2]
(c.hat <- Chi_sq/df);oCJS[1]/oCJS[2]


   ### ASSESSING GOF: ASSUMING TRANSIENCE AND TRAP DEPENDENCE ###

(Chi_sq.n <- oCJS$chi -
   (t2ct$test2ct[1] + t3sr$test3sr[1]))                           # calculates the Chi-square for a model contains both transience and trap dependence effect might be better (as test3sr and test2ct was significant)
(df.n <- oCJS[2] - (t2ct$test2ct[2] + t3sr$test3sr[2]))           # calculates df  for a model contains both transience and trap dependence effect might be better (as test3sr and test2ct was significant)
1-pchisq(Chi_sq.n, as.numeric(df.n))                              # calculates the test statistics for the new GOF
(c.hat.n <- Chi_sq.n/df.n)                                        # the c-hat for this model


   ### ASSESSING GOF: RELEASE (RMark/MARK) ###

tmp <- data;tmp$ch <- as.character(tmp$ch)
data.proc <- process.data(data=tmp,begin.time=2005)               # this function initializes several variables (e.g., number of capture occasions, time intervals) that are often specific to the capture-recapture model
                                                                  # being fitted to the data. It also is used to 1) define groups in the data that represent different levels of one or more factor covariates (e.g., sex),
                                                                  # 2) define time intervals between capture occasions (if not 1), and 3) create an age structure for the data, if any
                                                                  #  groups is a vector of factor variable names (in double quotes) used to create groups in the data.
                                                                  #  time.inetvals is a vector time between capture occasions
(release.gof(data.proc, view = F, invisible = F))


   ### STORING THE RESULT ###

sink("GOF-NoGrouping.txt")                                        # creates a text-file for some summary stats (including the various GOF-STATISTICS)
cat("####################################\n")
cat("### SUMMARY OF DATA: NO GROUPING ###\n")
cat("####################################\n");cat("\n");
cat("TOTAL NUMBER OF MARKED IND.\n")
print(dim(data)[1]);cat("\n");
cat("\n");cat("\n");cat("\n");
cat("###########################################\n")
cat("### GOODNESS OF FIT (GOF) USING R2ucare ###\n")
cat("###########################################\n")
cat("          -- Overall test --\n");
print(oCJS);cat("\n")
print(paste("                c-hat =", round(c.hat,3)))
print(paste("                c-hat = chi2/degree_of_freedom"));cat("\n")
cat("  --> EXPLAINATIONS: TAKEN FROM GIMENEZ ET AL. (2017a, 2018) <--  \n");
cat("          -- TEST2.CT: Trap happiness --\n");
cat("                'tests the hypothesis that there is no difference in the\n")
cat("                probability of being recaptured at i+1 between those captured\n")
cat("                and not captured at occasion i, conditional on presence at\n")
cat("                both occasions'\n")
                       print(t2ct$test2ct);cat("\n")
cat("          -- TEST3.SR: Transience --\n")
cat("                'tests the hypothesis that there is no difference among previously\n")
cat("                and newly marked individuals captured at time i in the probability\n")
cat("                of being recaptured later on (i.e., at time > i), It thus tests if\n")
cat("                re-encounters is a function of whether or not an ind. is newly marked'\n")
                       print(t3sr$test3sr);cat("\n")
cat("          -- TEST3.SM: Time of first recapture for 'new' vs. 'old' ind. at time i -- \n")
cat("                'tests the hypothesis that there is no difference in the expected\n")
cat("                time of first recapture between the 'new' and 'old' individuals\n")
cat("                captured at occasion i and seen again at least once'\n")
                       print(t3sm$test3sm);cat("\n")
cat("          -- TEST2.CL: Time to next recapture for ind. captured or not at time i --\n");
cat("                'tests the hypothesis that there is no difference in the expected\n")
cat("                time of next recapture between the individuals captured and not\n")
cat("                captured at occasion i conditional on presence at both occasions\n")
cat("                i and i+2'\n")
                       print(t2cl$test2cl);cat("\n");cat("\n");cat("\n")
cat("    ### GOF ACCOUNTING FOR TRAP DEPENDENCY (TEST2.CT) AND TRANSIENCE (TEST3.SR) ###\n");cat("\n")
cat("          -- Overall test --\n")
cat("                'calculates the Chi-square for a model contain both transience\n")
cat("                and trap dependence and test if this model have better fit\n")
cat("                (as TEST3.SR and TEST2.CT was significant)'. The new overall TEST: \n")
cat("                 Chi_sq (chi2)          = overall Chi-sq (from above) - TEST2.CT's Chi-sq -\n")
cat("                                                                   TEST3.SR's Chi-sq\n")
cat("                 df (degree_of_freedom) = Overall df (above)     - TEST2.CT's df - \n")
cat("                                                                   TEST3.SR's df\n")
cat("                 p (p_value)            = 1-pchisq(Chi_sq, as.numeric(df))\n");cat("\n")
cat("                 The new test removes the contribution of trap dependence (TEST2.CT)\n")
cat("                 and transience (TEST3.SR)\n")
cat("\n");cat("\n")
print(paste("                chi2              = ", round(Chi_sq.n,4)))
print(paste("                degree_of_freedom = ", round(df.n,4)))
print(paste("                p_value           = ",
                               round((1-pchisq(Chi_sq.n, as.numeric(df.n))),4)))
print(paste("                c-hat             =", round(c.hat.n,3)))
print(paste("                c-hat             = chi2/degree_of_freedom"));cat("\n");cat("\n")
cat("###########################################\n")
cat("### GOODNESS OF FIT (GOF) USING RELEASE ###\n")
cat("###########################################\n")
print(release.gof(data.proc, view = F, invisible = F));cat("\n")
cat("    ### GOF: RECAPTURE TEST (TEST2) AND SURVIVAL (TEST3) ###\n");cat("\n")
cat("  --> EXPLAINATIONS: TAKEN FROM COOCH AND WHITE (2018) <--  \n");
cat("          -- Assesses the assumption that all individuals in the population: \n")
cat("                 1) at given point in time (t) has the same probability of being recaptured (pt); and \n")
cat("                 2) all individuals in the population immediately after t has the same probability of surviving until t+1. \n")
cat("          -- TEST2 tests if the probability of observing an individual at t+1 is a function of whether the indvidual was \n")
cat("                 observed at t provided that the invidual suvived (from t to t+1). \n")
cat("          -- TEST3 tests if all marked animals alive at t have the same probability of surviving to t+1. \n")
cat("          -- TEST2 and TEST3 in RELEASE may loosely be referred to as the 'recapture test' and 'survival test', and reflects the two\n")
cat("                 above-mentioned assumptions. \n")
cat("                 'These tests assess, at each point in time, the equality of future recapture histories of previously marked animals\n")
cat("                 vs. those just marked' (Lebreton et al. 1992:83). \n")
cat("\n");cat("\n")
cat("REFERENCES FOR FURTHER DETAILS\n")
cat("Cooch, E. G. and White, G. C. (eds.). 2018. Chapter 5: Goodness of fit testing... Program MARK: a gentle introduction. - Colorado State University.\n")
cat("Gimenez, O., et al. 2017. R2ucare: an R package to perform goodness-of-fit tests for capture-recapture models. - bioRxiv 192468: 1-17.\n")
cat("Gimenez, O., et al. 2018. R2ucare: an r package to perform goodness-of-fit tests for capture–recapture models. - Methods in Ecology and Evolution 9: 1749-1754.\n")
cat("Lebreton, J.-D., et al. 1992. Modeling survival and testing biological hypotheses using marked animals: a unified approach with case studies. - Ecological Monographs 62: 67-118.\n")
sink()




   #####################################
   ### GROUPING BASED ON 'NewColony' ###
   #####################################

for (i in levels(as.factor(as.character(data$NewColony))))        # loops over the levels in 'NewColony'
       { Tmp <- subset(data, NewColony == i)
         tmp <- splitCH(as.character(Tmp$ch))                     # data on re-sighting used in the test
         print(paste("###", " NewColony = ", i ," ###",
           sep=""))


             ### STANDARD GOF: ASSUMING NO TRANSIENCE NOR TRAP DEPENDENCE ###

         (oCJS <- overall_CJS(X = tmp,freq = rep(1, length(tmp[,1]))))
         (t2ct <- test2ct(X = tmp, freq = rep(1, length(tmp[,1])),
           verbose = TRUE))
         (t3sr <- test3sr(X = tmp, freq = rep(1, length(tmp[,1])),
           verbose = TRUE))
         (t3sm <- test3sm(X = tmp, freq = rep(1, length(tmp[,1])),
           verbose = TRUE))
         (t2cl <- test2cl(X = tmp, freq = rep(1, length(tmp[,1])),
           verbose = TRUE))
         (Chi_sq <- t2ct$test2ct[1] + t3sr$test3sr[1] +
           t3sm$test3sm[1] + t2cl$test2cl[1]);oCJS[1]
         (df <- t2ct$test2ct[2] + t3sr$test3sr[2] + t3sm$test3sm[2] +
           t2cl$test2cl[2]);oCJS[2]
         (c.hat <- Chi_sq/df);oCJS[1]/oCJS[2]


             ### ASSASSESSING GOF: ASSUMING TRANSIENCE AND TRAP DEPENDENCE ###

         (Chi_sq.n <- oCJS$chi -
           (t2ct$test2ct[1] + t3sr$test3sr[1]))
         (df.n <- oCJS[2] - (t2ct$test2ct[2] + t3sr$test3sr[2]))
           1-pchisq(Chi_sq.n, as.numeric(df.n))
         (c.hat.n <- Chi_sq.n/df.n)


             ### SAVING THE OUPUT ###

        sink(paste("GOF-NewColony.txt",sep=""),append = T)
        cat("#########################################\n")
        print(paste("###", " NewColony = ", i," ###",
          sep=""))
        cat("#########################################\n");cat("\n");
        cat("TOTAL NUMBER OF MARKED IND.\n")
        print(dim(Tmp)[1]);cat("\n");
            cat("\n");cat("\n");cat("\n");
        cat("     ###########################################\n")
        cat("     ### GOODNESS OF FIT (GOF) USING R2ucare ###\n")
        cat("     ###########################################\n")
        cat("\n")
        cat("    ### ORIGINAL DATA ###\n");cat("\n")
        cat("          -- Overall test --\n");
        print(oCJS);cat("\n")
        print(paste("                c-hat =", round(c.hat,3)))
        cat("          -- TEST2.CT --\n");
                       print(t2ct$test2ct);cat("\n")
        cat("          -- TEST3.SR --\n")
                       print(t3sr$test3sr);cat("\n")
        cat("          -- TEST3.SM -- \n")
                       print(t3sm$test3sm);cat("\n")
        cat("          -- TEST2.CL --\n");
                       print(t2cl$test2cl);cat("\n");cat("\n");cat("\n")
        cat("    ### GOF ACCOUNTING FOR TRAP DEPENDENCY (TEST2.CT) AND TRANSIENCE (TEST3.SR) ###\n");cat("\n")
        cat("          -- Overall test --\n")
        print(paste("                chi2              = ", round(Chi_sq.n,4)))
        print(paste("                degree_of_freedom = ", round(df.n,4)))
        print(paste("                p_value           = ",
                      round((1-pchisq(Chi_sq.n, as.numeric(df.n))),4)))
        print(paste("                c-hat             =", round(c.hat.n,3)))

        cat("\n");cat("\n")
        sink()
   }

remove.files <- list.files(".", pattern="OUT")                    # delete all RELEASE FILES
do.call(file.remove,list(remove.files))
remove.files <- list.files(".", pattern="tmp")                    # delete all RELEASE FILES
do.call(file.remove,list(remove.files))





                                        #######################################################################################
                                        # DATA MANIPULATION PRIOR TO THE SURVIVAL ANALYSES: USING 'marked' BUILT-IN FUNCTIONS #
                                        #######################################################################################


   #############################################
   ### PROCESSING THE ENCOUNTER HISTORY DATA ###
   #############################################

library(marked)                                                   # loads the package 'marked: an R package for maximum-likelihood and MCMC analysis of capture-recapture data'

data.proc <- process.data(
  data = data,                                                    # data frame with at least one field named ch which is the capture (encounter) history stored as a character string
  begin.time = 2005,                                              # time of first capture occasion or vector of times if different for each group
  model = "CJS",                                                  # type of analysis model
  groups = NULL,                                                  # vector of factor variable names (in double quotes) in data that will be used to create groups in the data
  allgroups = FALSE,                                              # logical and if TRUE all groups are created from factors defined in groups even in lack of observations in the group
  nocc = length(min(data$MarkYear):max(data$MarkYear)),           # number of occasions for Nest type; either nocc or time.intervals must be specified
  accumulate  = F,                                                # logical and  if TRUE aggregates data with same values and creates freq field for count of records
  strata.labels = NULL )                                          # labels for strata used in capture history; they are converted to numeric in the order listed (only needed to
                                                                  #   specify unobserved strata; for any unobserved strata p=0)



   ####################################################################################
   ### CREATING A DESIGN DATA FRAME FOR crm WITH 2 AGE-CLASSES: 0, 1+ YEARS OF AGE  ###
   ####################################################################################

design.Phi   <- list(static     = c("NewColony", "MarkStat"),
                   age.bins     = c(0,1,100))                     # creates a list for the type of parameter for estimating capture probabilities ('p')
design.p   <- list(time.varying = c("td", "TimOb"),
                   static       = c("NewColony", "MarkStat"),
                   age.bins     = c(0,3,100))                     # creates a list for the type of parameter for estimating capture probabilities ('p')

design.parameters <- list(Phi = design.Phi, p = design.p)         # merges the two lists in the same object (used as input in the 'parameters' argument in the
                                                                  #  'make.design.data' function (see details below)

data.ddl_age2 <- make.design.data(                                # makes a design data.frame based on processed data a set of parameter(s) for the 'Phi' and 'p'
  data = data.proc,                                               # processed data list; resulting value from process.data
  parameters = design.parameters )                                # optional list containing a list for each type of parameter (list of lists); each
                                                                  #   parameter list is named with the parameter name (e.g., Phi); each parameter list
                                                                  #   can contain vectors named as follows:
                                                                  #     'age.bins'     -   bins for binning ages
                                                                  #     'time.varying' -   vector of field names that are time varying for this parameter
                                                                  #     'static'       -   vector of field names that are to be included that are not time varying for this parameter

rm(design.parameters, design.Phi, design.p)

names(data.ddl_age2$Phi);data.ddl_age2$Phi[1:5,]                  # shows details about the design.data for 'Phi'
names(data.ddl_age2$p);data.ddl_age2$p[1:5,]                      # shows details about the design.data for 'p'






                                        ######################################
                                        # SURVIVAL ANALYSES: TWO AGE-CLASSES #
                                        ######################################


   #########################################
   ### RUNNING A SET OF DIFFERENT MODELS ###
   #########################################


   ### DEFINING THE MODELS ###

fit.models <- function()
  {
    Phi.age.time <- list(formula  = ~age + time        )
    Phi.time     <- list(formula  = ~time              )

    p.dot        <- list(formula  = ~1                 )
    p.age        <- list(formula  = ~age               )
    p.time       <- list(formula  = ~time              )
    p.age.time   <- list(formula  = ~time + age        )
    p.td         <- list(formula  = ~td                )
    p.time.td    <- list(formula  = ~time + td         )
    p.Col.td     <- list(formula  = ~NewColony +
                                     td                )
    p.Colxtd     <- list(formula  = ~NewColony +
                                     td +
                                     NewColony:td      )

    p.TimOb      <- list(formula  = ~TimOb             )
    p.time.TimOb <- list(formula  = ~time + TimOb      )
    p.Col.TimOb  <- list(formula  = ~NewColony +
                                     TimOb             )
    p.ColxTimOb  <- list(formula  = ~NewColony +
                                     TimOb +
                                     NewColony:TimOb   )

    p.td.TimOb   <- list(formula  = ~td +
                                     TimOb             )

    p.Col        <- list(formula  = ~NewColony         )
    p.time.Col   <- list(formula  = ~time +
                                     NewColony         )

    cml <- create.model.list(c("Phi","p"))                        # matrix of model names
    results <- crm.wrapper(
      model.list  = cml,                                          # list of models associated with different parameters above
      data        = data.proc,                                    # data on (processed) detection histories (variable 'ch')
      ddl         = data.ddl_age2,                                # design data.frame
      external    = FALSE,                                        # if TRUE, model results are stored externally; otherwise, they are stored in 'crmlist'
      accumulate  = FALSE,
      hessian     = TRUE)

    return(results)
  }
# NOTES: 'crm.wrapper' contains functions that automates running a set of crm models.


   ### RUNNING THE MODELS ###

Gull.models <- fit.models()                                       # runs the models
Gull.models                                                       # prints the list of the models with model names, number of parameters, AIC and deltaAIC values



   ####################################################
   ### EXTRACTING ESTIMATES FROM THE SELECTED MODEL ###
   ####################################################


   ### THE SELECTED MODEL ###

Gull.model <-                                                     # extracts the selected model (i.e., the one with the smallest deltaAIC)
  Gull.models[[as.numeric(row.names(Gull.models$model.table)[1])]]
Gull.model

Phi <- predict(Gull.model,     vcv = T)$Phi                       # extracts 'reals' for the model's survival estimates
p <-   predict(Gull.model,     vcv = T)$p                         # extracts 'reals' for the model's recapture estimates


   ### PLOTTING ESTIMATED SURVIVAL ('Phi') ###

Gull_Phi <- Phi                                                   # stores data on the estimated survival ('Phi')
Gull_Phi$Age <- as.numeric(Gull_Phi$age)-1                        # makes age into a numeric variable
Gull_Phi$age <- ifelse(Gull_Phi$Age<=1, 1, 3)

dev.new();par(cex.axis = 1.2, cex.lab = 1.5,
  mar = c(4.5, 4.5, .1, 1))
Mean <- tapply(Gull_Phi$estimate, list(Gull_Phi$age,              # extracting point estimates to create plots of the results across age and time
  Gull_Phi$time), mean)
Lcl <- tapply(Gull_Phi$lcl, list(Gull_Phi$age,                    # extracting lower confidence intervals
  Gull_Phi$time), mean)
Ucl <- tapply(Gull_Phi$ucl, list(Gull_Phi$age,                    # extracting upper confidence intervals
  Gull_Phi$time), mean)
mp <- barplot(Mean, beside = T,                                   # plotting the results
  xlab = "Time (year)", ylab = expression("Apparent survival ("*
  italic(Phi)*")"%+-%" 95% CI"),
  col=colors()[c(131,136)],
  ylim = c(0,1.1))
segments(mp, Ucl, mp, Lcl, lwd=2, col = "dark grey")
legend(x = 32.5, y = 1.1, legend = c("[0,1]", "[2,-)"),bty = "n", # adds labels to the plot
  text.col = colors()[c(131,136)], title = "Age", pch = 15,
  box.col = NULL, pt.cex = 2.5, col = colors()[c(131,136)],
  cex = 1.3, title.col = "black")
dev.copy(tiff, filename =
  "Phi(AgeClass2).tiff", units="cm",
  width=16, height=15, res=600, compression = c("lzw"));dev.off()

    Gull_Phi$age <- ifelse(Gull_Phi$Age == 0, "[0-1]",            # creates the age-classes
      as.character(Gull_Phi$age))
    Gull_Phi$age <- ifelse(Gull_Phi$Age == 1, NA,
      as.character(Gull_Phi$age))
    Gull_Phi$age <- ifelse(Gull_Phi$Age >= 2, "[2,-)",
      as.character(Gull_Phi$age))
    Gull_Phi$age <- as.factor(Gull_Phi$age)
    for(i in levels(Gull_Phi$time) )                              # shortening the data.frame only one row per age-class
      { tmp <- subset(Gull_Phi, time == i)
       if(dim(tmp)[1] <= 2)
         { tmp2 <- tmp }
       else
         { for(j in levels(Gull_Phi$age))
             { tmp1 <- subset(tmp, age == j)
               tmp1 <- tmp1[1,]
               if (j == levels(Gull_Phi$age)[1])
                 { tmp2 <- tmp1 }
               else
                 { tmp2 <- rbind.data.frame(tmp2, tmp1) }
             }
         }
       { if (i == levels(Gull_Phi$time)[1])
           { tmp3 <- tmp2 }
         else
           { tmp3 <- rbind.data.frame(tmp3, tmp2) }
       }
     }
   Gull_Phi <- na.omit(tmp3)
   row.names(Gull_Phi) <- 1:dim(Gull_Phi)[1]
   rm(tmp, tmp1, tmp2, tmp3, i, j)
Gull_Phi <- Gull_Phi[,grep(x=names(Gull_Phi),                     # selects a few variables
   pattern="age|time|estimate|se|lcl|ucl")]


   ### PLOTTING ESTIMATED DETECTION PROBABILITY ('p') ###

Gull_p <- p                                                       # extracts 'reals' for the model's recapture estimates
Gull_p$Age <- as.numeric(Gull_p$age)-1                            # makes age into a numeric variable
Gull_p$age <- ifelse(Gull_p$Age<=2, 1, 3)

dev.new();par(cex.axis = 1.2, cex.lab = 1.5,
  mar = c(4.5, 4.5, .1, 1))
Mean <- tapply(Gull_p$estimate, list(Gull_p$age,                  # extracting data to create plots of the results across age and time
  Gull_p$time), mean)
Lcl <- tapply(Gull_p$lcl, list(Gull_p$age,                        # extracting lower confidence intervals
  Gull_p$time), mean)
Ucl <- tapply(Gull_p$ucl, list(Gull_p$age,                        # extracting upper confidence intervals
  Gull_p$time), mean)
mp <- barplot(Mean, beside = T,                                   # plotting the results
  xlab = "Time (year)", ylab = expression("Recapture probability ("*
  italic(p)*")"%+-%" 95% CI"),
  col=colors()[c(131,136)],
  ylim = c(0,1.1))
segments(mp, Ucl, mp, Lcl, lwd=2, col = "dark grey")
legend(x = 21, y = 1.11, legend = c("[1,3]", "[4,-)"),bty = "n",  # adds labels to the plot
  text.col = colors()[c(131,136)], title = "Age", pch = 15,
  box.col = NULL, pt.cex = 2.5, col = colors()[c(131,136)],
  cex = 1.3, title.col = "black")
dev.copy(tiff, filename =
  "p(AgeClass2).tiff", units="cm",
  width=16, height=15, res=600, compression = c("lzw"));dev.off()

Gull_p$age <- ifelse(Gull_p$Age == 0, NA,                         # creates the age-classes
  as.character(Gull_p$age))
Gull_p$age <- ifelse(Gull_p$Age == 2, NA,
  as.character(Gull_p$age))
Gull_p$age <- ifelse(Gull_p$Age == 3, NA,
  as.character(Gull_p$age))
Gull_p$age <- ifelse(Gull_p$Age == 1, "[1,3]",
  as.character(Gull_p$age))
Gull_p$age <- ifelse(Gull_p$Age >= 4, "[4,)",
  as.character(Gull_p$age))
Gull_p$age <- as.factor(Gull_p$age)
for(i in levels(Gull_p$time) )                                    # shortening the data.frame only one row per age-class
  { tmp <- subset(Gull_p, time == i)
    if(dim(tmp)[1] <= 3)
      { tmp2 <- tmp }
    else
      { for(j in levels(Gull_p$age))
          { tmp1 <- subset(tmp, age == j)
            tmp1 <- tmp1[1,]
            if (j == levels(Gull_p$age)[1])
              { tmp2 <- tmp1 }
            else
              { tmp2 <- rbind.data.frame(tmp2, tmp1) }
          }
      }
    { if (i == levels(Gull_p$time)[1])
        { tmp3 <- tmp2 }
      else
        { tmp3 <- rbind.data.frame(tmp3, tmp2) }
    }
  }
Gull_p <- na.omit(tmp3)
row.names(Gull_p) <- 1:dim(Gull_p)[1]
rm(tmp, tmp1, tmp2, tmp3, i, j)
Gull_p <- Gull_p[,grep(x=names(Gull_p),                           # selects a few variables
   pattern="age|time|estimate|se|lcl|ucl")]



   #######################
   ### STORING RESULTS ###
   #######################

tmp <- data.frame(Gull.models[length(Gull.models)])               # needs to re-define 'Gull.models' into a data.frame
tmp <- data.frame(i = row.names(tmp), tmp)                        # adds the models name' or number ('i')
names(tmp) <- gsub(x= names(tmp), pattern="model.table.",         # removes 'model.table.' from the names of 'tmp'
  replacement="")
wb <- createWorkbook()
addWorksheet(  wb, sheetName = "Gull_Phi")                        # creates the sheet
freezePane(    wb, sheet = "Gull_Phi", firstRow = TRUE,
  firstCol = TRUE)                                                # freeze first row and column
setColWidths( wb,  sheet = "Gull_Phi",widths ="auto",
  cols = 1:10)                                                    # sets column width
writeDataTable(wb, sheet = "Gull_Phi", x = Gull_Phi)              # adds data to the sheet
addWorksheet(  wb, sheetName = "Gull_p")
freezePane(    wb, sheet = "Gull_p", firstRow = TRUE,
 firstCol = TRUE)
setColWidths( wb,  sheet = "Gull_p",widths ="auto",
  cols = 1:10)
writeDataTable(wb, sheet = "Gull_p", x = Gull_p)
addWorksheet(  wb, sheetName = "Gull.models")
freezePane(    wb, sheet = "Gull.models",
  firstRow = TRUE, firstCol = TRUE)
setColWidths( wb,  sheet = "Gull.models", widths ="auto",
  cols = 1:10)
writeDataTable(wb, sheet = "Gull.models", x = tmp)
saveWorkbook(wb, "SurvivalAnalysis(AgeClass2).xlsx",              # saves the file
  overwrite = TRUE);rm(tmp)

options(width = 300)
sink("SurvivalAnalysis(AgeClass2).txt")
  print("--------------------------------------------------------")
  print("--------------------------------------------------------")
  print("      OUTPUT FROM THE ANALYSES USING 2 AGE-CLASSES      ")
  print("       age = 0, 2+                                      ")
  print("--------------------------------------------------------")
  print("--------------------------------------------------------")
  print("");print("")
  print("-----------------------------------------------")
  print("     MODEL SELECTION (Gull.models)             ")
  print("-----------------------------------------------")
  print(Gull.models);print("");print("")
  print("-----------------------------------------------")
  print("     MODEL SUMMARY (Gull.model)                ")
  print("-----------------------------------------------")
  print(Gull.model);print("");print("")
  print("-----------------------------------------------")
  print("     REAL ESTIMATES OF SURVIVAL  (Gull_Phi)    ")
  print("-----------------------------------------------")
  print(Gull_Phi);print("");print("")
  print("-----------------------------------------------")
  print("     REAL ESTIMATES OF RECAPTURE  (Gull_p)     ")
  print("-----------------------------------------------")
  print(Gull_p);print("");print("")
sink();options(width = 80)
save(
   Gull_p, Gull_Phi,
   file = "SurvivalAnalysis(AgeClass2).RData")










                                        ################################################################
                                        # SURVIVAL ANALYSES: ONE AGE CLASS (ADULTS MARKED ON THE NEST) #
                                        ################################################################


   ##############################
   ### RE-ORGANIZING THE DATA ###
   ##############################

data.procAdu <- process.data(
  data = subset(data, MarkStat == "Adult"),                  # selects only individuals marked as adults
  begin.time = 2005,
  model = "CJS",
  groups = NULL,
  allgroups = FALSE,
  initial.ages = data$AgeInit,
  nocc = length(levels(as.factor(data$MarkYear))),
  accumulate  = F,
  strata.labels = NULL )

design.PhiAdu   <- list(static     = c("NewColony"))
design.pAdu   <- list(time.varying = c("td", "TimOb"),
                   static       = c("NewColony", "MarkStat"))

design.parametersAdu <- list(Phi = design.PhiAdu, p = design.pAdu)

data.ddlAdu <- make.design.data(
  data = data.procAdu,
  parameters = design.parametersAdu )

names(data.ddlAdu$Phi);data.ddlAdu$Phi[1:5,]
names(data.ddlAdu$p);data.ddlAdu$p[1:5,]


   #########################################
   ### RUNNING A SET OF DIFFERENT MODELS ###
   #########################################


   ### DEFINING THE MODELS ###

fit.models <- function()
  {
    Phi.dot      <- list(formula  = ~1                 )
    Phi.time     <- list(formula  = ~time              )

    p.dot        <- list(formula  = ~1                 )
    p.time       <- list(formula  = ~time              )

    p.td         <- list(formula  = ~td                )
    p.time.td    <- list(formula  = ~time + td         )
    p.Col.td     <- list(formula  = ~NewColony +
                                     td                )

    p.TimOb      <- list(formula  = ~TimOb             )
    p.time.TimOb <- list(formula  = ~time + TimOb      )
    p.Col.TimOb  <- list(formula  = ~NewColony +
                                     TimOb             )
    p.ColxTimOb  <- list(formula  = ~NewColony +
                                     TimOb +
                                     NewColony:TimOb   )

    p.td.TimOb   <- list(formula  = ~td +
                                     TimOb             )

    p.Col        <- list(formula  = ~NewColony         )
    p.time.Col   <- list(formula  = ~time +
                                     NewColony         )
    p.Colxtd     <- list(formula  = ~NewColony +
                                     td +
                                     NewColony:td      )

    cml <- create.model.list(c("Phi","p"))
    results <- crm.wrapper(
      model.list  = cml,
      data        = data.procAdu,
      ddl         = data.ddlAdu,
      external    = FALSE,
      accumulate  = FALSE,
      hessian = T)

    return(results)
  }


   ### RUNNING A SET OF DIFFERENT MODELS ###

GullAdu.models <- fit.models()
GullAdu.models



   ####################################################
   ### EXTRACTING ESTIMATES FROM THE SELECTED MODEL ###
   ####################################################


   ### THE SELECTED MODEL ###

GullAdu.model <-
  GullAdu.models[[as.numeric(row.names(GullAdu.models$model.table)[1])]]
GullAdu.model

PhiAdu <- predict(GullAdu.model,     vcv = T)$Phi
pAdu <-   predict(GullAdu.model,     vcv = T)$p


   ### PLOTTING ESTIMATED SURVIVAL ('Phi') ###

dev.new();par(cex.axis = 1.2, cex.lab = 1.5,
  mar = c(4.5, 4.5, 1, 1))
Mean <- tapply(PhiAdu$estimate, list(PhiAdu$occ),
   mean)
names(Mean) <- c("2005-19")
Lcl <- tapply(PhiAdu$lcl, list(PhiAdu$occ),
  mean)
Ucl <- tapply(PhiAdu$ucl, list(PhiAdu$occ),
  mean)
mp <- barplot(Mean, beside = T,
  xlab = "Time (year)", ylab = expression("Apparent survival ("*
  italic(Phi)*")"%+-%" 95% CI"),
  col=colors()[c(131,136)],
  ylim = c(0,1.1))
segments(mp, Ucl, mp, Lcl, lwd=2, col = "dark grey")
dev.copy(tiff, filename =
  "Phi~(Adults).tiff", units="cm",
  width=16, height=15, res=600, compression = c("lzw"));dev.off()
PhiAdu <- PhiAdu[,grep(x=names(PhiAdu),                           # selects a few variables
   pattern="age|time|estimate|se|lcl|ucl")]


   ### PLOTTING ESTIMATED DETECTION PROBABILITY ('p') ###

dev.new();par(cex.axis = 1.2, cex.lab = 1.5,
  mar = c(4.5, 4.5, 1, 1))
Mean <- tapply(pAdu$estimate, list(
  pAdu$td, pAdu$time), mean)
Lcl <- tapply(pAdu$lcl, list(
  pAdu$td, pAdu$time), mean)
Ucl <- tapply(pAdu$ucl, list(
  pAdu$td, pAdu$time), mean)
mp <- barplot(Mean, beside = T,
  xlab = "Time (year)", ylab = expression("Recapture probability ("*
  italic(p)*")"%+-%" 95% CI"),
  col=colors()[c(131,136)],,
  ylim = c(0,1.1))
segments(mp, Ucl, mp, Lcl, lwd=2, col = "dark grey")
legend(x = 15, y = 1.11, legend = c("0", "1"),bty = "n",
  text.col = colors()[c(131,136)], title = "Trap dependence", pch = 15,
  box.col = NULL, pt.cex = 2.5, col = colors()[c(131,136)],
  cex = 1.3, title.col = "black")
dev.copy(tiff, filename =
  "p~(Adult).tiff", units="cm",
  width=16, height=15, res=600, compression = c("lzw"));dev.off()
pAdu <- pAdu[,grep(x=names(pAdu),                                 # selects a few variables
   pattern="td|time|estimate|se|lcl|ucl")]



   #######################
   ### STORING RESULTS ###
   #######################



tmp <- data.frame(GullAdu.models[length(GullAdu.models)])
tmp <- data.frame(i = row.names(tmp), tmp)
names(tmp) <- gsub(x= names(tmp), pattern="model.table.",
  replacement="")
wb <- createWorkbook()
addWorksheet(  wb, sheetName = "Gull_Phi")
freezePane(    wb, sheet = "Gull_Phi", firstRow = TRUE,
  firstCol = TRUE)
setColWidths( wb,  sheet = "Gull_Phi",widths ="auto",
  cols = 1:10)
writeDataTable(wb, sheet = "Gull_Phi", x = PhiAdu)
addWorksheet(  wb, sheetName = "Gull_p")
freezePane(    wb, sheet = "Gull_p", firstRow = TRUE,
 firstCol = TRUE)
setColWidths( wb,  sheet = "Gull_p",widths ="auto",
  cols = 1:10)
writeDataTable(wb, sheet = "Gull_p", x = pAdu)
addWorksheet(  wb, sheetName = "Gull.models")
freezePane(    wb, sheet = "Gull.models",
  firstRow = TRUE, firstCol = TRUE)
setColWidths( wb,  sheet = "Gull.models", widths ="auto",
  cols = 1:10)
writeDataTable(wb, sheet = "Gull.models", x = tmp)
saveWorkbook(wb, "SurvivalAnalysis(Adults).xlsx",                # saves the file
  overwrite = TRUE);rm(tmp)

options(width = 300)
sink("SurvivalAnalysis(Adults).txt")
  print("--------------------------------------------------------")
  print("--------------------------------------------------------")
  print("      OUTPUT FROM THE ANALYSES                          ")
  print("--------------------------------------------------------")
  print("--------------------------------------------------------")
  print("");print("")
  print("-----------------------------------------------")
  print("     MODEL SELECTION (GullAdu.models)          ")
  print("-----------------------------------------------")
  print(GullAdu.models);print("");print("")
  print("-----------------------------------------------")
  print("     MODEL SUMMARY (GullAdu.model)             ")
  print("-----------------------------------------------")
  print(GullAdu.model);print("");print("")
  print("-----------------------------------------------")
  print("     REAL ESTIMATES OF SURVIVAL  (PhiAdu)      ")
  print("-----------------------------------------------")
  print(PhiAdu);print("");print("")
  print("-----------------------------------------------")
  print("     REAL ESTIMATES OF RECAPTURE  (pAdu)       ")
  print("-----------------------------------------------")
  print(pAdu);print("");print("")
sink();options(width = 80)









                                        ########################
                                        # STORING SESSION INFO #
                                        ########################


sink("SessionInfo.txt")
print(sessionInfo(), locale = F)
sink()
